#include "audio/RtAudio.h"
#include "audio/FileRead.h"
#include "defs.h"
#include "display/glFuncs.h"

#include "display/playhead.h"
#include "display/waveformDisplayWithScroll.h"
#include "display/acquireViewWaveformDisplay.h"
#include "display/uiElement.h"
#include "display/audioElement.h"
#include "display/audioPlayer.h"
#include "display/scrapsWindow.h"
#include "display/scrapSummaryDisplay.h"
#include "display/fileBrowseWindow.h"
#include "display/timeGridUtil.h"
#include "display/processesBrowseWindow.h"
#include "display/acquireControlsWindow.h"
#include "display/scrapDetailWindow.h"
#include "display/processWindow.h"

#include "display/glLibs.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <pthread.h>
#include <vector>
#include <string>

using namespace std;

enum 
{ 
	ACQUIRE_WAVEFORM_DISPLAY = 0, 
	SCRAPS_WINDOW_DISPLAY,
	FILE_BROWSE_WINDOW_DISPLAY,
	ACQUIRE_CONTROLS_WINDOW_DISPLAY,
	SCRAP_DETAIL_WINDOW_DISPLAY
};

enum
{
	CREATE_SCRAPS_WINDOW = 0,
	CREATE_PROCESSES_WINDOW,
	CREATE_PROCESS_WINDOW
};

enum
{
	ACQUIRE = 0,
	CREATE
};


//Window sizing variables
GLsizei screenWidth = 1280;
GLsizei screenHeight = 800;
double windowSizeFraction = 1.0;
GLsizei g_windowWidth = screenWidth * windowSizeFraction;
GLsizei g_windowHeight = screenWidth * windowSizeFraction;
GLboolean g_fullscreen = FALSE;

//Keep track of the active element for mouse dragging operations
UiElement* g_dragActiveElement;

//Keep track of the active element for playing audio
AudioElement* g_audioActiveElement;

//Keep track of mouse location
Point2D g_mouseLoc;

//Global audio players - manage audio output
AudioPlayer* g_spaceBarAudioPlayer;
AudioElement* g_tempAudioActiveElement;
bool g_tempAudioElementHolding;

//Current view
int g_currentView = ACQUIRE;

//Acquire view uiElements
vector<UiElement*> g_acquireViewUiElements;
//Create view uiElements
vector<UiElement*> g_createViewUiElements;

//Global time grid utility
TimeGridUtil* g_timeGridUtil;


//Global thread locking / management variables
pthread_mutex_t g_mutex;
GLboolean g_ready = false;


SAMPLE** g_inFileBuffer;
SAMPLE** g_scrapDetailBuffer;

//For dragging the acquire waveform height
float g_waveformDragStartY;
float g_windowHeightAdjust;
float g_windowHeightAdjustOld;

//Audio callback fn ----------------------------------------------------------------
int tick( void *outputBuffer, void *inputBuffer, unsigned int nBufferFrames,
           double streamTime, RtAudioStreamStatus status, void* noData )
{
  //output warning if stream status isn't good
  if ( status ) cerr << "Stream over/underflow detected." << endl;
  
  //recast output and input buffer pointers
  SAMPLE *oSamples = (SAMPLE *) outputBuffer;
  SAMPLE *iSamples = (SAMPLE *) inputBuffer;  

	if( g_spaceBarAudioPlayer )
	{
		if( g_spaceBarAudioPlayer->isPlaying )
		{
			for( int i = 0; i < nBufferFrames; i++)
			{
				
				if( g_spaceBarAudioPlayer->isPlaying )
				{
					//Output the next samples
					if( g_spaceBarAudioPlayer->numChannels == 2 )
					{
						for( int channel=0; channel < 2; channel++ )
							*oSamples++ = g_spaceBarAudioPlayer->bufferPointer[channel][g_spaceBarAudioPlayer->currentIndex];
					}
					else
					{
					for( int channel=0; channel < 2; channel++ )
						*oSamples++ = g_spaceBarAudioPlayer->bufferPointer[0][g_spaceBarAudioPlayer->currentIndex];	
					}
					
					//Update the index
					if( g_spaceBarAudioPlayer->playLength >= 0 )
						g_spaceBarAudioPlayer->currentIndex++;
					else
						g_spaceBarAudioPlayer->currentIndex--;	
				
					//Make sure we are still in the play region and the song. Loop if we should.
					if( g_spaceBarAudioPlayer->playLength > 0 )
					{
						
						if( g_spaceBarAudioPlayer->currentIndex > g_spaceBarAudioPlayer->startSample + g_spaceBarAudioPlayer->playLength )
						{
							if( g_spaceBarAudioPlayer->isLooping )
								g_spaceBarAudioPlayer->currentIndex = g_spaceBarAudioPlayer->startSample;
							else
							{
								g_spaceBarAudioPlayer->isPlaying = false;
								
								//Restore temp element if we were holding one
								if( g_tempAudioElementHolding )
								{
									g_audioActiveElement = g_tempAudioActiveElement;
									g_spaceBarAudioPlayer = g_audioActiveElement->getAudioPlayer();
									g_tempAudioActiveElement = NULL;
									g_tempAudioElementHolding = false;	
								}

							}
						}	
						
					}
					else if( g_spaceBarAudioPlayer->playLength < 0 )
					{
						
						if( g_spaceBarAudioPlayer->currentIndex < g_spaceBarAudioPlayer->startSample + g_spaceBarAudioPlayer->playLength )
						{
							if( g_spaceBarAudioPlayer->isLooping )
								g_spaceBarAudioPlayer->currentIndex = g_spaceBarAudioPlayer->startSample;
							else
							{
								g_spaceBarAudioPlayer->isPlaying = false;
								
								//Restore temp element if we were holding one
								if( g_tempAudioElementHolding )
								{
									g_audioActiveElement = g_tempAudioActiveElement;
									g_spaceBarAudioPlayer = g_audioActiveElement->getAudioPlayer();
									g_tempAudioActiveElement = NULL;
									g_tempAudioElementHolding = false;	
								}
							}								
						}
						
					}
					else
					{
						if( g_spaceBarAudioPlayer->currentIndex > g_spaceBarAudioPlayer->bufferLength - 1 )
						{
							g_spaceBarAudioPlayer->isPlaying = false;

							//Restore temp element if we were holding one
							if( g_tempAudioElementHolding )
							{
								g_audioActiveElement = g_tempAudioActiveElement;
								g_spaceBarAudioPlayer = g_audioActiveElement->getAudioPlayer();
								g_tempAudioActiveElement = NULL;
								g_tempAudioElementHolding = false;	
							}							
						}
					}
				}
				else
				{
					for( int channel=0; channel < g_spaceBarAudioPlayer->numChannels; channel++ )
						*oSamples++ = 0.0;
				}
				
			}
		}
		else
		{
			for( int i = 0; i < nBufferFrames; i++)
			{
				for( int channel=0; channel < 2; channel++ )
					*oSamples++ = 0.0;
			}
		}		
				
	}
	else
	{
		for( int i = 0; i < nBufferFrames; i++)
		{
			for( int channel=0; channel < 2; channel++ )
					*oSamples++ = 0.0;
		}
	}

  pthread_mutex_lock(&g_mutex);
	  	  
	  g_ready = true;
	  
  pthread_mutex_unlock(&g_mutex);
   
  return 0;
}


//Main -----------------------------------------------------------------------------
int main( int argc, char ** argv )
{	
	
	// initialize GLUT
	glutInit( &argc, argv );
	// double buffer, use rgb color, enable depth buffer				//RGBA for alpha transparency
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
	// initialize the window size
	glutInitWindowSize( g_windowWidth, g_windowHeight );
	// set the window postion
	glutInitWindowPosition( 0, 0 );
	// create the window
	glutCreateWindow( "S.mash" );
	
	// set the idle function - called when idle
	glutIdleFunc( idleFunc );
	// set the display function - called when redrawing
	glutDisplayFunc( displayFunc );
	// set the reshape function - called when client area changes
	glutReshapeFunc( reshapeFunc );
	// set the keyboard function - called on keyboard events
	glutKeyboardFunc( keyboardFunc );
	// set the special keyboard keys function - called on special keyboard events
	glutSpecialFunc( keyboardSpecial);
	// set the mouse function - called on mouse stuff
	glutMouseFunc( mouseFunc );	
	// set the mouse drag function
	glutMotionFunc( mouseDrag );
	// set the mouse passive motion function	
	glutPassiveMotionFunc( passiveFunc );


	
	pthread_mutex_init(&g_mutex, NULL);

	g_timeGridUtil = new TimeGridUtil( 240.0 );
	
	g_inFileBuffer = new SAMPLE*[2];
	for( int i=0; i < 2; i++ )
	{
		g_inFileBuffer[i] = new SAMPLE[10];
		for( int j = 0; j < 10; j ++ )
			g_inFileBuffer[i][j] = 0.0;	
	}
	
	g_scrapDetailBuffer = new SAMPLE*[2];
	for( int i=0; i < 2; i++ )
	{
		g_scrapDetailBuffer[i] = new SAMPLE[10];
		for( int j = 0; j < 10; j ++ )
			g_scrapDetailBuffer[i][j] = 0.0;	
	}
	
	//Acquire View
	AcquireControlsWindow* controlsWindow = new AcquireControlsWindow(0, 1, 0, 1);
		
	g_acquireViewUiElements.push_back( new AcquireViewWaveformDisplay(1, g_windowWidth, 1, 300.0, g_inFileBuffer, 2, 10, g_timeGridUtil, false, 2, 1, 1.0/4, false) );
	
	ScrapDetailWindow* detailWindow = new ScrapDetailWindow(g_windowWidth *4.0 / 5.0, g_windowWidth, g_windowHeight / 2.0, g_windowHeight, g_scrapDetailBuffer, 2, 10, g_timeGridUtil);
	
	ScrapsWindow* g_scrapsWindow = new ScrapsWindow(302, 750, 302.0, g_windowHeight - 480, g_windowWidth, g_windowHeight, detailWindow, controlsWindow);
	g_acquireViewUiElements.push_back( g_scrapsWindow );
	
	g_acquireViewUiElements.push_back( new FileBrowseWindow(1, 300, 302.0, g_windowHeight - 480, g_windowWidth, g_windowHeight, g_inFileBuffer, (AcquireViewWaveformDisplay*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ], controlsWindow) );
	
	g_acquireViewUiElements.push_back( controlsWindow );
	g_acquireViewUiElements.push_back( detailWindow );
	
	//Create View
	ProcessWindow* processWindow = new ProcessWindow(0, 1, 0, 1);
	g_createViewUiElements.push_back( g_scrapsWindow );
	g_createViewUiElements.push_back( new ProcessesBrowseWindow( 0, 1, 0, 1, g_windowWidth, g_windowHeight, processWindow) );
	g_createViewUiElements.push_back( processWindow );
	
	g_dragActiveElement = NULL;
	g_audioActiveElement = (AudioElement*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
	g_spaceBarAudioPlayer = g_audioActiveElement->getAudioPlayer();	
	g_tempAudioActiveElement = NULL;
	g_tempAudioElementHolding = false;
	
		
	// RtAudio Variables ------------------------------------------------------------
  unsigned int inChannels = 2, outChannels = 2, fs = SAMPRATE, bufferFrames = 1024, iDevice = 0, oDevice = 0, iOffset = 0, oOffset = 0;


  RtAudio adac;
  if ( adac.getDeviceCount() < 1 ) {
    std::cout << "\nNo audio devices found!\n";
    exit( 1 );
  }

  // Let RtAudio print messages to stderr.
  adac.showWarnings( true );

  // Set the number of channels for input and output.
  RtAudio::StreamParameters iParams, oParams;
  
  // Input
  if ( iDevice == 0 )
    iParams.deviceId = adac.getDefaultInputDevice();
  else
    iParams.deviceId = iDevice - 1;
    
  iParams.nChannels = inChannels;
  iParams.firstChannel = iOffset;  
  
  // Output
  if ( oDevice == 0 )
    oParams.deviceId = adac.getDefaultOutputDevice();
  else
    oParams.deviceId = oDevice - 1;
    
  oParams.nChannels = outChannels;
  oParams.firstChannel = oOffset;

	//Create options struct in case we want to set some of these
  RtAudio::StreamOptions options;

  try {
  		adac.openStream( &oParams, &iParams, RTAUDIO_FLOAT64, fs, &bufferFrames, &tick, (void*) NULL, &options );
  }
  catch ( RtError& err ) {
    err.printMessage();
    exit( 1 );
  }

	// Try to open audio stream
  try {
    adac.startStream();
  }
  catch ( RtError& err ) {
    err.printMessage();
    goto cleanup;
  }
  
  
	//Enable alpha blending
  glEnable(GL_BLEND);
  glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);   
  
//	glEnable(GL_LINE_SMOOTH);
//	glBlendFunc(GL_LINE_SMOOTH_HINT, GL_NICEST); 
  
  // Main loop -----------------------------------------
	// let GLUT handle the current thread from here
				glutMainLoop();
  // ---------------------------------------------------

  // If we get here, stop the stream.
  try
  {
    adac.stopStream();
  }
  catch( RtError& err )
  {
    err.printMessage();
  }


cleanup:
  if ( adac.isStreamOpen() ) adac.closeStream();
  return 0;
}

#include "display/glFuncs.cpp"