#include "acquireTimeGrid.h"

AcquireTimeGrid :: AcquireTimeGrid(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, float xMinDispIn, float xMaxDispIn, int numBeatsIn, float beatDivisionIn, TimeGridUtil* globalTimeGridIn, bool activeIn): UiElement(xMinIn, xMaxIn, yMinIn, yMaxIn), xMinDisp(xMinDispIn), xMaxDisp(xMaxDispIn), numBeats(numBeatsIn), beatDivision(beatDivisionIn), active(activeIn) {
	globalTimeGrid = globalTimeGridIn;
	
	calcNumSamplesTotal();
	calcNumSamplesPerDivision();
}


void AcquireTimeGrid :: setNumBeats(int numBeatsIn)
{
	numBeats = numBeatsIn;
	calcNumSamplesTotal();	
}


void AcquireTimeGrid :: setBeatDivision(float beatDivisionIn)
{
	beatDivision = beatDivisionIn;
	calcNumSamplesPerDivision();
	numBeats = ceil(numSamplesTotal / numSamplesPerDivision);	
	calcNumSamplesTotal();
}


void AcquireTimeGrid :: updateForNewTempo()
{
	calcNumSamplesTotal();
	calcNumSamplesPerDivision();	
}

void AcquireTimeGrid :: calcNumSamplesTotal()
{
	numSamplesTotal = ceil(globalTimeGrid->getNumSamplesPerBeat() * numBeats * beatDivision);
}


void AcquireTimeGrid :: calcNumSamplesPerDivision()
{
	numSamplesPerDivision = globalTimeGrid->getNumSamplesPerBeat() * beatDivision;	
}


void AcquireTimeGrid :: draw()
{
	//Draw main playhead (will be at mouse location)
  glColor4f( 0.0, 1.2*136/255.0, 1.2*158/255.0, 0.8);
  glLineWidth( 2.0 );
	
	if( xMin >= xMinDisp && xMin <= xMaxDisp)
	{
	 	glBegin( GL_LINES );
	 		glVertex3f(xMin, yMin, 0.0);
	 		glVertex3f(xMin, yMax, 0.0);
		glEnd();
	}
	
	//Draw opaque rectangle background
	if( numSamplesTotal != 0 )
	{
		glColor4f( 0.0, 136/255.0, 158/255.0, 0.4);
		
		//Make sure it is inside display area
		float currXMin;
		float currXMax;
		
		if( xMax > xMin )
		{
			currXMin = xMin;
			currXMax = xMax;	
		}
		else
		{
			currXMin = xMax;
			currXMax = xMin;	
		}
		
		if( currXMin < xMinDisp )
			currXMin = xMinDisp;
			
		if( currXMax > xMaxDisp )
			currXMax = xMaxDisp;
		
		glRectf( currXMin, yMin, currXMax, yMax );
	}
	
	//If we are displaying more than one division, draw light division lines
	int numLines = 0;
	if( numBeats > 0 )
		numLines = numBeats;
	else
		numLines = -numBeats;
		
	if( numLines > 1 )
	{
	  glColor4f( 0.0, 1.2*136/255.0, 1.2*158/255.0, 0.6);
	  glLineWidth( 0.5 );
	  
		for( int i=1; i<numLines; i++ )
		{
			float lineX = xMin + i*(xMax-xMin)/numLines;
			
			if( lineX >= xMinDisp && lineX <= xMaxDisp )
			{
			 	glBegin( GL_LINES );
		 			glVertex3f(lineX, yMin, 0.0);
			 		glVertex3f(lineX, yMax, 0.0);
				glEnd();
			}
		}	
	}
	
}