#include "acquireViewScrollBar.h"

AcquireViewScrollBar :: 	AcquireViewScrollBar(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, bool horizIn, long indexLengthIn, long minIndexActiveIn, long maxIndexActiveIn, float summaryTranslateFactorIn): UiElement(xMinIn, xMaxIn, yMinIn, yMaxIn) 
{ 
	horiz = horizIn; 
	maxIndex = indexLengthIn - 1; 
	minIndexActive = minIndexActiveIn; 
	maxIndexActive = maxIndexActiveIn; 
	
	activeRectMinX = xMin + 1 + (1.0 * minIndexActive)/maxIndex * ((xMax-2) - (xMin+2));
	activeRectMaxX = xMin + 2 + (1.0 * maxIndexActive)/maxIndex * ((xMax-2) - (xMin+2));
	minimumActiveRectWidth = 20.0;
	
	summaryTranslateFactor = summaryTranslateFactorIn;
}


void AcquireViewScrollBar :: draw()
{
  glColor3f( 1.0, 1.0, 1.0);
  glLineWidth( 1.0 );	  	

 	glBegin( GL_LINE_STRIP );
 		glVertex3f(xMin, yMin, 0.0);
 		glVertex3f(xMin, yMax, 0.0);
 		glVertex3f(xMax, yMax, 0.0);
 		glVertex3f(xMax, yMin, 0.0);
 		glVertex3f(xMin, yMin, 0.0);
	glEnd();

	glRectf(activeRectMinX , yMin + 1, activeRectMaxX, yMax - 2 );
}


void AcquireViewScrollBar :: changeIndexScale(long newIndexLength)
{
	//Preserve relative locations
	float translateFactor = (1.0 * (newIndexLength)) / (maxIndex + 1);
	
	minIndexActive = floor(minIndexActive * translateFactor);
	maxIndexActive = ceil(maxIndexActive * translateFactor);
	initialActiveWidth = floor(initialActiveWidth * translateFactor);
	maxIndex = newIndexLength - 1;
		
	if( maxIndexActive < maxIndex )
		maxIndexActive = maxIndex;
		
	summaryTranslateFactor = summaryTranslateFactor / translateFactor;
}


void AcquireViewScrollBar :: mouseDown(Point2D mouseLoc)
{		
		active = true;	
		initialMouseLoc = mouseLoc;
		initialActiveWidth = maxIndexActive - minIndexActive;
		
		//If mouse click was not in the active rectangle, force immediate update (scroll)
		if( !isInsideActiveRect(mouseLoc) )
			mouseMotion(mouseLoc);
}


void AcquireViewScrollBar :: mouseUp(Point2D mouseLoc)
{
	active = false;
}


void AcquireViewScrollBar :: mouseMotion(Point2D mouseLoc)
{
	float indexZoomScaleFactor = 1.0 - (mouseLoc.y - initialMouseLoc.y)/300.0 * (maxIndex / initialActiveWidth);
	
	float zoomFactorForFiveSecs = (5.0*SAMPRATE) / (initialActiveWidth * summaryTranslateFactor);
//	std::cout << "Zoom for Five Sec: " << zoomFactorForFiveSecs << std::endl;

	//Do something for nicer zoom in!	
	if( indexZoomScaleFactor < zoomFactorForFiveSecs )
	{

	}
	
	//If we would be showing less than one index per pixel, show one index per pixel (Limit on zoom in)
	if( indexZoomScaleFactor * initialActiveWidth  < (xMax - xMin) )
		indexZoomScaleFactor = (xMax - xMin) / initialActiveWidth;
		
//	std::cout << "Zoom scale factor: " << indexZoomScaleFactor << std::endl;		
	
	long centerIndexActive = floor((1.0 * (mouseLoc.x - xMin))/(xMax - xMin) * maxIndex);
	
//	std::cout << "Center index: " << centerIndexActive << std::endl;	
	
	long wouldBeMinIndexActive = floor(centerIndexActive - (initialActiveWidth / 2.0) * indexZoomScaleFactor);
	long wouldBeMaxIndexActive = ceil(centerIndexActive + (initialActiveWidth / 2.0) * indexZoomScaleFactor);		
	
	if( wouldBeMinIndexActive < 0 )
	{
		minIndexActive = 0;
		maxIndexActive = ceil(initialActiveWidth * indexZoomScaleFactor);	
		
		if( maxIndexActive > maxIndex)
			maxIndexActive = maxIndex;
	}
	else if( wouldBeMaxIndexActive > maxIndex )
	{
		maxIndexActive = maxIndex;
		minIndexActive = floor(maxIndex - initialActiveWidth * indexZoomScaleFactor);	
		
		if( minIndexActive < 0)
			minIndexActive = 0;
	}
	else
	{
		minIndexActive = wouldBeMinIndexActive;
		maxIndexActive = wouldBeMaxIndexActive;	
	}
	
	activeRectMinX = xMin + 1 + (1.0 * minIndexActive)/maxIndex * ((xMax-2) - (xMin+2));
	activeRectMaxX = xMin + 2 + (1.0 * maxIndexActive)/maxIndex * ((xMax-2) - (xMin+2));
	
	//Don't let the active rectangle get too small, even when very zoomed in! 
	if( activeRectMaxX - activeRectMinX < minimumActiveRectWidth )
	{
		float activeRectCenter = (activeRectMaxX + activeRectMinX) / 2.0;
		float wouldBeActiveRectMinX =	activeRectCenter - minimumActiveRectWidth/2.0;
		float wouldBeActiveRectMaxX = activeRectCenter + minimumActiveRectWidth/2.0;
		
		if( wouldBeActiveRectMinX < xMin + 1.0 )
		{
			activeRectMinX = xMin + 1.0;
			activeRectMaxX = activeRectMinX + minimumActiveRectWidth;	
		}
		else if( wouldBeActiveRectMaxX > xMax - 2.0 )
		{
			activeRectMaxX = xMax - 2.0;
			activeRectMinX = activeRectMaxX - minimumActiveRectWidth;
		}
		else
		{
			activeRectMinX = wouldBeActiveRectMinX;
			activeRectMaxX = wouldBeActiveRectMaxX;	
		}
	}
	
}


void AcquireViewScrollBar :: setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn)
{
	xMin = xMinIn;
	xMax = xMaxIn;
	yMin = yMinIn;
	yMax = yMaxIn;
	
	activeRectMinX = xMin + 1 + (1.0 * minIndexActive)/maxIndex * ((xMax-2) - (xMin+2));
	activeRectMaxX = xMin + 2 + (1.0 * maxIndexActive)/maxIndex * ((xMax-2) - (xMin+2));
	
	//Don't let the active rectangle get too small, even when very zoomed in! 
	if( activeRectMaxX - activeRectMinX < minimumActiveRectWidth )
	{
		float activeRectCenter = (activeRectMaxX + activeRectMinX) / 2.0;
		float wouldBeActiveRectMinX =	activeRectCenter - minimumActiveRectWidth/2.0;
		float wouldBeActiveRectMaxX = activeRectCenter + minimumActiveRectWidth/2.0;
		
		if( wouldBeActiveRectMinX < xMin + 1.0 )
		{
			activeRectMinX = xMin + 1.0;
			activeRectMaxX = activeRectMinX + minimumActiveRectWidth;	
		}
		else if( wouldBeActiveRectMaxX > xMax - 2.0 )
		{
			activeRectMaxX = xMax - 2.0;
			activeRectMinX = activeRectMaxX - minimumActiveRectWidth;
		}
		else
		{
			activeRectMinX = wouldBeActiveRectMinX;
			activeRectMaxX = wouldBeActiveRectMaxX;	
		}
	}
		
}