#include "acquireViewWaveformDisplay.h"

AcquireViewWaveformDisplay :: AcquireViewWaveformDisplay(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, SAMPLE** dataIn, int numChannelsIn, long dataLengthIn, TimeGridUtil* globalTimeGridUtilIn, bool scrollOnBottomIn, int numChannelsDisplayedIn, int timeGridNumBeatsIn, float timeGridBeatDivisionIn, bool acquireTimeGridActiveIn): AudioElement(xMinIn, xMaxIn, yMinIn, yMaxIn)
{
	lockedForDataUpdate = true;
	
	numChannels = numChannelsIn;
	data =	new SAMPLE*[numChannels];
	
	for( int i=0; i < numChannels; i++)
		data[i] = dataIn[i];
	
	dataLength = dataLengthIn;

/* Not currently implemented	
	if( numChannels > 1 )
	{
		monoData = new SAMPLE[dataLength];	
		for( int i=0; i < dataLength; i++ )
			monoData[i] = data[0][i] + data[1][i];	//Do a max abs here to get envelope? Normalize?
	}
*/

	numIndicesPerSummarySample = 1.0;
	usingSummary = false;
	summaryData = new SAMPLE*[numChannels];
	summaryLength = 250000;
	//For good display performance, we don't want to iterate through more than summaryLength points
	if( dataLength > summaryLength )
	{
		usingSummary = true;
		for( int channel = 0; channel < numChannels; channel++)
		{
			summaryData[channel] = new SAMPLE[summaryLength];
			
			numIndicesPerSummarySample = (1.0 * dataLength) / summaryLength;
			
			SAMPLE currentMin;
			SAMPLE currentMax;
			
			for( int i = 0; i < summaryLength; i++ )
			{
				currentMin = 1.0;
				currentMax = -1.0;
				
				for( int j = floor(i*numIndicesPerSummarySample); j < floor((i+1)*numIndicesPerSummarySample); j++ )
				{
					if( data[channel][j] > currentMax )
						currentMax = data[channel][j];
						
					if( data[channel][j] < currentMin )
						currentMin = data[channel][j];	
				}
	
				summaryData[channel][i++] = currentMin;
				summaryData[channel][i] = currentMax;
			}
		}

	}
	
	//Display all channels if no number was specified	
	if( numChannelsDisplayedIn == 0 )
		numChannelsDisplayed = numChannelsIn;
	else
		numChannelsDisplayed = numChannelsDisplayedIn;
		
	scrollBarWidth = 15.0;
	waveformDisplayPadding = 5.0;
	
	//Default to display the entire waveform, start to finish, -1.0 to 1.0 amplitude, with a y = 0 line
	//Display size is smaller to leave space for scroll bars
	float xMinDisp = xMin + waveformDisplayPadding;
	float xMaxDisp = xMax - waveformDisplayPadding;
	float yMinDisp;
	float yMaxDisp;
	
	//Initialize displays and scroller in the correct positions
	if( scrollOnBottomIn )
	{
		yMinDisp = yMin + scrollBarWidth + waveformDisplayPadding;
		yMaxDisp = yMax - waveformDisplayPadding;
		if(usingSummary)
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMin, yMin+scrollBarWidth, true, summaryLength, 0, summaryLength-1, numIndicesPerSummarySample);
		else
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMin, yMin+scrollBarWidth, true, dataLength, 0, dataLength-1, numIndicesPerSummarySample);	
	}
	else
	{
		yMinDisp = yMin + waveformDisplayPadding;
		yMaxDisp = yMax - scrollBarWidth - waveformDisplayPadding;
		if(usingSummary)		
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMax-scrollBarWidth, yMax, true, summaryLength, 0, summaryLength-1, numIndicesPerSummarySample);
		else
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMax-scrollBarWidth, yMax, true, dataLength, 0, dataLength-1, numIndicesPerSummarySample);
	}
	
	//Initialize waveform displays
	waveformDisplays = new LineDisplay2D*[numChannels];
	if(usingSummary)
	{
		for( int i=0; i < numChannels; i++)
		{
			waveformDisplays[i] = new LineDisplay2D( xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)), summaryData[i], summaryLength, 0, summaryLength-1, 1.0, true );
			waveformDisplays[i]->setActive(true);
		}
	}
	else
	{
		for( int i=0; i < numChannels; i++)
		{
			waveformDisplays[i] = new LineDisplay2D( xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)), data[i], dataLength, 0, dataLength-1, 1.0, true );
			waveformDisplays[i]->setActive(true);			
		}
	}
		
	//Initialize the playheads
	playheadStart = new Playhead(-1.0, -1.0, yMinDisp, yMaxDisp, xMin + 0.5*waveformDisplayPadding, xMaxDisp);
	playheadStart->setPlayRegionOn(true);
	playheadMoving = new Playhead(-1.0, -1.0, yMinDisp, yMaxDisp, xMin + 0.5*waveformDisplayPadding, xMaxDisp);
	mouseFollowPlayheadForBeatLockedTimeGridMode = new Playhead(-1.0, -1.0, yMinDisp, yMaxDisp, xMin + 0.5*waveformDisplayPadding, xMaxDisp);
	drawBeatLockedModePlayhead = false;
	
	//Initialize the audio player
	audioPlayer = new AudioPlayer(data, dataLength, numChannels, 0, 0, false, false);
	
	scrollOnBottom = scrollOnBottomIn;
	
	//Initialize playhead time grid
	playheadStartTimeGrid = new AcquireTimeGrid(-1.0, -1.0, yMinDisp, yMaxDisp, xMin + 0.5*waveformDisplayPadding, xMaxDisp, timeGridNumBeatsIn, timeGridBeatDivisionIn, globalTimeGridUtilIn, false);
	drawPlayheadStartTimeGrid = false;
	
	//Initialize mouse time grid
	acquireTimeGrid = new AcquireTimeGrid(-1.0, -1.0, yMinDisp, yMaxDisp, xMin + 0.5*waveformDisplayPadding, xMaxDisp, timeGridNumBeatsIn, timeGridBeatDivisionIn, globalTimeGridUtilIn, acquireTimeGridActiveIn);
	drawAcquireTimeGrid = false;
	
	//Ready to draw
	lockedForDataUpdate = false;
}


AcquireViewWaveformDisplay :: ~AcquireViewWaveformDisplay()
{
	delete []data;
	
	for( int i=0; i < numChannels; i++ )
	{
		delete waveformDisplays[i];
		if( summaryData[i] )
			delete summaryData[i];	
	}
	
	delete []waveformDisplays;
	delete []summaryData;
	delete hScroll;
	delete acquireTimeGrid;
	delete playheadStartTimeGrid;
	delete mouseFollowPlayheadForBeatLockedTimeGridMode;
	delete playheadStart;
	delete playheadMoving;
}


void AcquireViewWaveformDisplay :: draw()
{
  glColor3f( 1.0, 1.0, 1.0);
  glLineWidth( 1.0 );	  	

 	glBegin( GL_LINE_STRIP );
 		glVertex3f(xMin, yMin, 0.0);
 		glVertex3f(xMin, yMax, 0.0);
 		glVertex3f(xMax, yMax, 0.0);
 		glVertex3f(xMax, yMin, 0.0);
 		glVertex3f(xMin, yMin, 0.0);
	glEnd();
	
	if( !lockedForDataUpdate )
	{
		
		
		for( int i = 0; i < numChannelsDisplayed; i++ )
			waveformDisplays[i]->draw();
		
		hScroll->draw();
	
		//Only draw if we have a playhead down
		if( drawPlayheadStartTimeGrid && playheadStart->getStartSample() != -1 && playheadStart->getPlayLength() != 0 )
			playheadStartTimeGrid->draw();
		else
			playheadStart->draw();	
		
		if( audioPlayer->isPlaying )
		{		
			playheadMoving->setStartSample( audioPlayer->currentIndex );
			
			//Solve for playheadMoving position in terms of display
			float playheadMovingX;
			playheadMovingX = dataIndexValueToXPos(playheadMoving->getStartSample());	
			playheadMoving->setXMin(playheadMovingX);
			
			playheadMoving->draw();		
		}
		else if( drawAcquireTimeGrid )
		{
			acquireTimeGrid->draw();
		}
		else if( drawBeatLockedModePlayhead )
		{
			mouseFollowPlayheadForBeatLockedTimeGridMode->draw();	
		}
		
		
	}
	
}


//--------------------------------------------------------------------------------
//
//				MOUSE CLICK (DOWN) FUNCTION
//
//--------------------------------------------------------------------------------
void AcquireViewWaveformDisplay :: mouseDown(Point2D mouseLoc)
{
	
	//If we clicked on the scroller -------------------------------
	if( hScroll->isInside(mouseLoc) )
	{
		bool originallyInsideActiveRect = true;
		if( !hScroll->isInsideActiveRect(mouseLoc) )
			originallyInsideActiveRect = false;

		//Do the scrollBar's mouseDown. This sets the scrollBar active.
		hScroll->mouseDown(mouseLoc);
		
		//If not inside the active rect -----------------------------
		//  Do immediate update (scroll)
		if( !originallyInsideActiveRect )
		{
			//Solve for playheadStart position in terms of display
			float playheadStartX;
			playheadStartX = dataIndexValueToXPos(playheadStart->getStartSample());		
			playheadStart->setXMin(playheadStartX);
			playheadStartTimeGrid->setXMin(playheadStartX);
			
			//Solve for play region end in terms of display, if one exists
			if( playheadStart->getPlayLength() != 0 )
			{
				float playheadEndX;
				playheadEndX = dataIndexValueToXPos(playheadStart->getStartSample() + playheadStart->getPlayLength());		
				playheadStart->setXMax(playheadEndX);
				playheadStartTimeGrid->setXMax(playheadEndX);				
			}
						
			//Don't need to check if useSummary should change since we cannot have any zoom in or out yet.
			//  Just adjust the waveform displays to scroll
			for( int channel = 0; channel < numChannels; channel++ )
				waveformDisplays[channel]->changeDisplayedIndexRange( hScroll->getMinIndexActive(), hScroll->getMaxIndexActive() );
		}
	}
	
	
	//Else if we clicked inside waveform displays -----------------
	else
	{
		//Only allow playhead repositioning if audio is not playing
		if( !audioPlayer->isPlaying )
		{
		
			//Limit mouse position to be within the padded waveform displays
			if(mouseLoc.x <= xMin + waveformDisplayPadding) 
				mouseLoc.x = xMin + waveformDisplayPadding;		
			else if(mouseLoc.x >= xMax - waveformDisplayPadding )
				mouseLoc.x = xMax - waveformDisplayPadding;
			
			//Mark that the waveform display section is active
			active = true;
			
			//Solve for playheadStart position in terms of display
			float translatedIndex;
			translatedIndex = xPosToDataIndexValue(mouseLoc.x);
						
			playheadStart->setStartSample(translatedIndex);
			playheadStart->setXMin(mouseLoc.x);
			
			//If in acquire time grid mode, update playhead time grid to match the acquire time grid
			if( acquireTimeGrid->isActive() )
			{
				playheadStartTimeGrid->setXMin(mouseLoc.x);				
				playheadStartTimeGrid->setNumBeats( acquireTimeGrid->getNumBeats() );
//				playheadStartTimeGrid->setBeatDivision( acquireTimeGrid->getBeatDivision() );
			}
			
			//update audio player
			audioPlayer->startSample = playheadStart->getStartSample();				
			
			
			//If in time grid mode, automatically select the specified region
			if( acquireTimeGrid->isActive() )
			{
				//Solve for the end of the play region in terms of the display
				
				//The end position is the start position plus the length of the current time grid
				float endPosX = dataIndexValueToXPos( playheadStart->getStartSample() + acquireTimeGrid->getNumSamplesTotal() );
	
				playheadStart->setXMax(endPosX);
				playheadStartTimeGrid->setXMax(endPosX);
				//Update the play length to the current time grid length
				playheadStart->setPlayLength( acquireTimeGrid->getNumSamplesTotal() );
			}
			//otherwise, reset play region to be nothing - must drag to make a region		
			else
			{		
				playheadStart->setPlayLength( 0 );
				playheadStart->setXMax(mouseLoc.x);
				playheadStartTimeGrid->setXMin( mouseLoc.x );
				playheadStartTimeGrid->setXMax( mouseLoc.x );
			}
			
			//Set the audio player's playLength to be that of the playhead region
			audioPlayer->playLength = playheadStart->getPlayLength();
			
		}
	}
}


void AcquireViewWaveformDisplay :: mouseUp(Point2D mouseLoc)
{
	hScroll->mouseUp(mouseLoc);
	
	//If the acquire time grid is on, we originally clicked on the waveform view, and the mouse is still inside the waveform viewer, update the grid location so that it is drawn, even if we don't move the mouse after letting go of the button
	if( acquireTimeGrid->isActive() && active && isInside(mouseLoc) )
		displayAcquireTimeGrid(mouseLoc);	
	
	//If in playhead start time grid mode, update the time grid division length to match where we dragged to
	if( drawPlayheadStartTimeGrid && active && !drawAcquireTimeGrid )
	{
		long selectedRegionLengthSamples = xPosToDataIndexValue(mouseLoc.x) - playheadStart->getStartSample();
		playheadStartTimeGrid->setNumSamplesTotal( selectedRegionLengthSamples );
		playheadStartTimeGrid->setNumSamplesPerDivision( selectedRegionLengthSamples / playheadStartTimeGrid->getNumBeats() );
	}
		
	active = false;
}



//--------------------------------------------------------------------------------
//
//				MOUSE MOTION FUNCTION
//
//--------------------------------------------------------------------------------
void AcquireViewWaveformDisplay :: mouseMotion(Point2D mouseLoc)
{
	
	//If we clicked on the scroll bar originally
	if( hScroll->isActive() )
	{	
		
		//See if we need to switch into or out of summary mode due to zooming in / out
		if( usingSummary && ( (hScroll->getMaxIndexActive() - hScroll->getMinIndexActive()) * numIndicesPerSummarySample < summaryLength) )
		{
			for( int channel = 0; channel < numChannels; channel++ )
				waveformDisplays[channel]->setData( data[channel], dataLength );
				
			hScroll->changeIndexScale( dataLength );
			usingSummary = false;
		}
		else if( !usingSummary && ( hScroll->getMaxIndexActive() - hScroll->getMinIndexActive() > summaryLength) )
		{
			for( int channel = 0; channel < numChannels; channel++ )
				waveformDisplays[channel]->setData( summaryData[channel], summaryLength );									
				
			hScroll->changeIndexScale( summaryLength );
			usingSummary = true;					
		}
	
		//Call the scrollbar's mouseMotion fn. This handles updating the zoom, active scrollbar rectangle size, and active / displayed index region.
		hScroll->mouseMotion(mouseLoc);
		
		
		//Solve for playheadStart position in terms of display, if we have a playhead (program begins without one)
		if( playheadStart->getStartSample() != -1 )
		{
			float playheadStartX;
			playheadStartX = dataIndexValueToXPos(playheadStart->getStartSample());		
			playheadStart->setXMin(playheadStartX);
			playheadStartTimeGrid->setXMin(playheadStartX);			
		}
		
		//Solve for play region end in terms of display, if one exists
		if( playheadStart->getPlayLength() != 0 )
		{
			float playheadEndX;
			playheadEndX = dataIndexValueToXPos(playheadStart->getStartSample() + playheadStart->getPlayLength());		
			playheadStart->setXMax(playheadEndX);
			playheadStartTimeGrid->setXMax(playheadEndX);
			
/*	//If in playhead start time grid mode, update the time grid division length to match where we dragged to
	if( drawPlayheadStartTimeGrid && active && !drawAcquireTimeGrid )
	{
		long selectedRegionLengthSamples = xPosToDataIndexValue(mouseLoc.x) - playheadStart->getStartSample();
		playheadStartTimeGrid->setNumSamplesTotal( selectedRegionLengthSamples );
		playheadStartTimeGrid->setNumSamplesPerDivision( selectedRegionLengthSamples / playheadStartTimeGrid->getNumBeats() );
	}			
	
	*/
		}
		
		//if we are displaying a time grid region, update those values as well so that the timegrid region follows the mouse, even when scrolling.
		if( drawAcquireTimeGrid )
		{
			long hoverIndexStart = xPosToDataIndexValue(mouseLoc.x);
			float endPosX = dataIndexValueToXPos(hoverIndexStart + acquireTimeGrid->getNumSamplesTotal() );
			
			float startPosX;
			if( mouseLoc.x < xMin + waveformDisplayPadding )
				startPosX = xMin + waveformDisplayPadding;
			else if( mouseLoc.x > xMax - waveformDisplayPadding )
				startPosX = xMax - waveformDisplayPadding;
			else
				startPosX = mouseLoc.x;
			
			acquireTimeGrid->setXMin(startPosX);
			acquireTimeGrid->setXMax(endPosX);			
		}

		
		//Update waveform displays with the new displayed index range		
		for( int channel = 0; channel < numChannelsDisplayed; channel++ )
			waveformDisplays[channel]->changeDisplayedIndexRange( hScroll->getMinIndexActive(), hScroll->getMaxIndexActive() );
	}
	
	//Otherwise, if the waveform display region was clicked on
	if( active )
	{
		//Only allow dragging if acquire time grid is NOT active (tempo dictated grid length)
		if( !acquireTimeGrid->isActive() )
		{
			//Solve for the play region length using the current mouse location and the original click location
			float translatedIndex;
			translatedIndex = xPosToDataIndexValue(mouseLoc.x);
			long playLength = translatedIndex - playheadStart->getStartSample();
			long playheadEndSample = playheadStart->getStartSample() + playLength;
			
			//Limit the play region to be inside the data
			if( playheadEndSample > dataLength )
				playheadEndSample = dataLength - 1;
			else if( playheadEndSample < 0 )
				playheadEndSample = 0;
				
			playLength = playheadEndSample - playheadStart->getStartSample();
	
			//Solve for play region end in terms of display, if there is one (won't display if playLength = 0 anyways)
			if( playLength != 0)
			{				
				float playheadEndX;
				playheadEndX = dataIndexValueToXPos(playheadEndSample);		
				playheadStart->setXMax(playheadEndX);
				playheadStartTimeGrid->setXMax(playheadEndX);

				//If in playhead start time grid mode, update the time grid division length to match where we dragged to
				if( drawPlayheadStartTimeGrid && !drawAcquireTimeGrid )
				{
					long selectedRegionLengthSamples = xPosToDataIndexValue(mouseLoc.x) - playheadStart->getStartSample();
					playheadStartTimeGrid->setNumSamplesTotal( selectedRegionLengthSamples );
					playheadStartTimeGrid->setNumSamplesPerDivision( selectedRegionLengthSamples / playheadStartTimeGrid->getNumBeats() );
				}
			}
	
			//Update playLengths
			playheadStart->setPlayLength(playLength);
			audioPlayer->playLength = playheadStart->getPlayLength();
		}

	}

}


bool AcquireViewWaveformDisplay :: isAudioElement(Point2D mouseLoc)
{
	if( isInside(mouseLoc) && !hScroll->isInside(mouseLoc) ) 
		return true; 
	else 
		return false; 	
}


float AcquireViewWaveformDisplay :: dataIndexValueToXPos( long indexVal )
{
	float xPosition;
	
	if( usingSummary )
	{
		xPosition = xMin + waveformDisplayPadding + ( ((indexVal / numIndicesPerSummarySample) - hScroll->getMinIndexActive()) / (1.0 * (hScroll->getMaxIndexActive() - hScroll->getMinIndexActive())) ) * (1.0 * (xMax - xMin - waveformDisplayPadding*2));
	}
	else
	{
		xPosition = xMin + waveformDisplayPadding + ( (1.0 * (indexVal - hScroll->getMinIndexActive())) / (1.0 * (hScroll->getMaxIndexActive() - hScroll->getMinIndexActive())) ) * (xMax - xMin - waveformDisplayPadding*2);
	}
	
	return xPosition;
	
}


long AcquireViewWaveformDisplay :: xPosToDataIndexValue( float xPos )
{
	long translatedIndex;
		
	if( usingSummary )
		translatedIndex =(  hScroll->getMinIndexActive() + (1.0 * (xPos - (xMin+waveformDisplayPadding))) / (xMax - xMin - waveformDisplayPadding*2) * (hScroll->getMaxIndexActive() - hScroll->getMinIndexActive()) ) * numIndicesPerSummarySample;
	else
		translatedIndex = hScroll->getMinIndexActive() + (1.0 * (xPos - (xMin+waveformDisplayPadding))) / (xMax - xMin - waveformDisplayPadding*2) * (hScroll->getMaxIndexActive() - hScroll->getMinIndexActive());
			
	if( translatedIndex < 0 )
		translatedIndex = 0;
	else if( translatedIndex > dataLength )
		translatedIndex = dataLength;

	return translatedIndex;
}


void AcquireViewWaveformDisplay :: reverseSelection()
{
	playheadStart->setStartSample(playheadStart->getStartSample() + playheadStart->getPlayLength());
	playheadStart->setPlayLength(-playheadStart->getPlayLength());
	
	//Solve for playheadStart position in terms of display
	if( playheadStart->getStartSample() != -1 )
	{
		float playheadStartX;
		playheadStartX = dataIndexValueToXPos(playheadStart->getStartSample());		
		playheadStart->setXMin(playheadStartX);
		playheadStartTimeGrid->setXMin(playheadStartX);		
	}
		
	//Solve for play region end in terms of display
	if( playheadStart->getPlayLength() != 0)
	{				
		float playheadEndX;
		playheadEndX = dataIndexValueToXPos(playheadStart->getStartSample() + playheadStart->getPlayLength());		
		playheadStart->setXMax(playheadEndX);
		playheadStartTimeGrid->setXMax(playheadEndX);		
	}
	
	audioPlayer->startSample = playheadStart->getStartSample();	
	audioPlayer->playLength = playheadStart->getPlayLength();
}


void AcquireViewWaveformDisplay :: setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn)
{
	xMin = xMinIn;
	xMax = xMaxIn;
	yMin = yMinIn;
	yMax = yMaxIn;
	
	float xMinDisp = xMin + waveformDisplayPadding;
	float xMaxDisp = xMax - waveformDisplayPadding;
	float yMinDisp;
	float yMaxDisp;
	
	if( scrollOnBottom )
	{
		yMinDisp = yMin + scrollBarWidth + waveformDisplayPadding;
		yMaxDisp = yMax - waveformDisplayPadding;
		hScroll->setPosition(xMin, xMax, yMin, yMin+scrollBarWidth);
	}
	else
	{
		yMinDisp = yMin + waveformDisplayPadding;
		yMaxDisp = yMax - scrollBarWidth - waveformDisplayPadding;		
		hScroll->setPosition(xMin, xMax, yMax-scrollBarWidth, yMax);
	}
	
	for( int i=0; i < numChannels; i++ )
	{
		waveformDisplays[i]->resize(xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)));
		waveformDisplays[i]->generateDisplayVals();
	}
		
	playheadStart->setYPositions(yMinDisp, yMaxDisp);
	playheadStart->setXMinMaxDisp(xMinDisp, xMaxDisp);	
	
	playheadStartTimeGrid->setYPositions(yMinDisp, yMaxDisp);
	playheadStartTimeGrid->setXMinMaxDisp(xMinDisp, xMaxDisp);	
	
	playheadMoving->setYPositions(yMinDisp, yMaxDisp);
	playheadMoving->setXMinMaxDisp(xMinDisp, xMaxDisp);	
	
	mouseFollowPlayheadForBeatLockedTimeGridMode->setYPositions(yMinDisp, yMaxDisp);
	mouseFollowPlayheadForBeatLockedTimeGridMode->setXMinMaxDisp(xMinDisp, xMaxDisp);	

	acquireTimeGrid->setYPositions(yMinDisp, yMaxDisp);
	acquireTimeGrid->setXMinMaxDisp(xMinDisp, xMaxDisp);	
}



void AcquireViewWaveformDisplay :: displayAcquireTimeGrid(Point2D mouseLoc)
{	
	float startPosX;
	if( mouseLoc.x < xMin + waveformDisplayPadding )
		startPosX = xMin + waveformDisplayPadding;
	else if( mouseLoc.x > xMax - waveformDisplayPadding )
		startPosX = xMax - waveformDisplayPadding;		
	else
		startPosX = mouseLoc.x;
		
	long hoverIndexStart = xPosToDataIndexValue(startPosX);
	float endPosX = dataIndexValueToXPos(hoverIndexStart + acquireTimeGrid->getNumSamplesTotal());		
		
	acquireTimeGrid->setXMin(startPosX);
	acquireTimeGrid->setXMax(endPosX);	
	
	drawAcquireTimeGrid = true;
}


void AcquireViewWaveformDisplay :: setPlayRegion(long startSampleIn, long playLengthIn)
{
	playheadStart->setStartSample(startSampleIn);
	playheadStart->setPlayLength(playLengthIn);
	audioPlayer->startSample = startSampleIn;
	audioPlayer->playLength = playLengthIn;
	
	//Solve for playheadStart position in terms of display
	if( playheadStart->getStartSample() != -1 )
	{
		float playheadStartX;
		playheadStartX = dataIndexValueToXPos(playheadStart->getStartSample());		
		playheadStart->setXMin(playheadStartX);
		playheadStartTimeGrid->setXMin(playheadStartX);		
	}
		
	//Solve for play region end in terms of display
	if( playheadStart->getPlayLength() != 0)
	{				
		float playheadEndX;
		playheadEndX = dataIndexValueToXPos(playheadStart->getStartSample() + playheadStart->getPlayLength());		
		playheadStart->setXMax(playheadEndX);
		playheadStartTimeGrid->setXMax(playheadEndX);		
	}
}


void AcquireViewWaveformDisplay :: displayBeatLockedModePlayhead(Point2D mouseLoc)
{
	float startPosX;
	if( mouseLoc.x < xMin + waveformDisplayPadding )
		startPosX = xMin + waveformDisplayPadding;
	else if( mouseLoc.x > xMax - waveformDisplayPadding )
		startPosX = xMax - waveformDisplayPadding;		
	else
		startPosX = mouseLoc.x;
		
	mouseFollowPlayheadForBeatLockedTimeGridMode->setXMin(startPosX);
	
	drawBeatLockedModePlayhead = true;	
}


void AcquireViewWaveformDisplay :: updatePlayheadTimeGridToMatchAcquireSpecs()
{
	//Only do this if we have a playhead down with some selection (starts at -1 when we don't display anything)
	if( playheadStart->getStartSample() != -1 )
	{
		playheadStart->setPlayLength( acquireTimeGrid->getNumSamplesTotal() );
		audioPlayer->playLength = acquireTimeGrid->getNumSamplesTotal();
		
		//Solve for playheadStart position in terms of display
		if( playheadStart->getStartSample() != -1 )
		{
			float playheadStartX;
			playheadStartX = dataIndexValueToXPos(playheadStart->getStartSample());		
			playheadStart->setXMin(playheadStartX);
			playheadStartTimeGrid->setXMin(playheadStartX);		
		}
			
		//Solve for play region end in terms of display
		if( playheadStart->getPlayLength() != 0)
		{				
			float playheadEndX;
			playheadEndX = dataIndexValueToXPos(playheadStart->getStartSample() + playheadStart->getPlayLength());		
			playheadStart->setXMax(playheadEndX);
			playheadStartTimeGrid->setXMax(playheadEndX);		
		}
	}
}


void AcquireViewWaveformDisplay :: updateData( SAMPLE** dataIn, int numChannelsIn, long dataLengthIn, int numChannelsDisplayedIn )
{
	lockedForDataUpdate = true;

	//Clear stuff allocated for old data.	
	delete []data;
	delete hScroll;
	
	for( int i=0; i < numChannels; i++ )
	{
		delete waveformDisplays[i];
		if( dataLength > summaryLength )
		{
			delete summaryData[i];	
		}
	}
	
	delete []waveformDisplays;	
	delete []summaryData;	
	delete audioPlayer;
	
	//Load new data
	numChannels = numChannelsIn;
	data =	new SAMPLE*[numChannels];
	
	for( int i=0; i < numChannels; i++)
		data[i] = dataIn[i];
	
	dataLength = dataLengthIn;
	
	numIndicesPerSummarySample = 1.0;
	usingSummary = false;
	summaryData = new SAMPLE*[numChannels];
	summaryLength = 250000;
	//For good display performance, we don't want to iterate through more than summaryLength points
	if( dataLength > summaryLength )
	{
		usingSummary = true;
		for( int channel = 0; channel < numChannels; channel++)
		{
			summaryData[channel] = new SAMPLE[summaryLength];
			
			numIndicesPerSummarySample = (1.0 * dataLength) / summaryLength;
			
			SAMPLE currentMin;
			SAMPLE currentMax;
			
			for( int i = 0; i < summaryLength; i++ )
			{
				currentMin = 1.0;
				currentMax = -1.0;
				
				for( int j = floor(i*numIndicesPerSummarySample); j < floor((i+1)*numIndicesPerSummarySample); j++ )
				{
					if( data[channel][j] > currentMax )
						currentMax = data[channel][j];
						
					if( data[channel][j] < currentMin )
						currentMin = data[channel][j];	
				}
	
				summaryData[channel][i++] = currentMin;
				summaryData[channel][i] = currentMax;
			}
		}

	}
	
	//Display all channels if no number was specified	
	if( numChannelsDisplayedIn == 0 )
		numChannelsDisplayed = numChannelsIn;
	else
		numChannelsDisplayed = numChannelsDisplayedIn;

	//Default to display the entire waveform, start to finish, -1.0 to 1.0 amplitude, with a y = 0 line
	//Display size is smaller to leave space for scroll bars
	float xMinDisp = xMin + waveformDisplayPadding;
	float xMaxDisp = xMax - waveformDisplayPadding;
	float yMinDisp;
	float yMaxDisp;
	
	//Initialize displays and scroller in the correct positions
	if( scrollOnBottom )
	{
		yMinDisp = yMin + scrollBarWidth + waveformDisplayPadding;
		yMaxDisp = yMax - waveformDisplayPadding;
		if(usingSummary)
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMin, yMin+scrollBarWidth, true, summaryLength, 0, summaryLength-1, numIndicesPerSummarySample);
		else
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMin, yMin+scrollBarWidth, true, dataLength, 0, dataLength-1, numIndicesPerSummarySample);	
	}
	else
	{
		yMinDisp = yMin + waveformDisplayPadding;
		yMaxDisp = yMax - scrollBarWidth - waveformDisplayPadding;
		if(usingSummary)		
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMax-scrollBarWidth, yMax, true, summaryLength, 0, summaryLength-1, numIndicesPerSummarySample);
		else
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMax-scrollBarWidth, yMax, true, dataLength, 0, dataLength-1, numIndicesPerSummarySample);
	}
	
	//Initialize waveform displays
	waveformDisplays = new LineDisplay2D*[numChannels];
	if(usingSummary)
	{
		for( int i=0; i < numChannels; i++)
		{
			waveformDisplays[i] = new LineDisplay2D( xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)), summaryData[i], summaryLength, 0, summaryLength-1, 1.0, true );
			waveformDisplays[i]->setActive(true);
		}
	}
	else
	{
		for( int i=0; i < numChannels; i++)
		{
			waveformDisplays[i] = new LineDisplay2D( xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)), data[i], dataLength, 0, dataLength-1, 1.0, true );
			waveformDisplays[i]->setActive(true);			
		}
	}
	
	//Initialize the audio player
	audioPlayer = new AudioPlayer(data, dataLength, numChannels, 0, 0, false, false);
	
	playheadStart->setStartSample( -1 );
	playheadStart->setPlayLength( 0 );
	playheadStart->setXMin( -1 );
	playheadStart->setXMax( -1 );
//	playheadStartTimeGrid->setXMin( -1 );
//	playheadStartTimeGrid->setXMax( -1 );
	
	
	lockedForDataUpdate = false;
}


void AcquireViewWaveformDisplay :: setPlayheadStartTimeGridNumBeats(int timeGridNumBeatsIn)
{ 
	long numSamplesTotal = playheadStartTimeGrid->getNumSamplesTotal();
	
	//Update number of beats
	playheadStartTimeGrid->setNumBeats(timeGridNumBeatsIn); 
	
	//If in playhead start time grid mode, update the time grid division length to match this new number of beats
	if( drawPlayheadStartTimeGrid && !drawAcquireTimeGrid )
	{
		playheadStartTimeGrid->setNumSamplesTotal( numSamplesTotal );
		playheadStartTimeGrid->setNumSamplesPerDivision( numSamplesTotal / playheadStartTimeGrid->getNumBeats() );
	}

}