#ifndef ACQUIRE_VIEW_WAVEFORM_DISPLAY
#define ACQUIRE_VIEW_WAVEFORM_DISPLAY

//Class for the acquire view waveform display

#include "audioElement.h"
#include "acquireViewScrollBar.h"
#include "playhead.h"
#include "lineDisplay2D.h"
#include "audioPlayer.h"
#include "timeGridUtil.h"
#include "acquireTimeGrid.h"
#include <math.h>
#include <iostream>

class AcquireViewWaveformDisplay : public AudioElement
{
public:

	//Constructor
	AcquireViewWaveformDisplay(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, SAMPLE** dataIn, int numChannelsIn, long dataLengthIn, TimeGridUtil* globalTimeGridUtilIn, bool scrollOnBottomIn = true, int numChannelsDisplayedIn = 0, int timeGridNumBeatsIn = 1.0, float timeGridBeatDivisionIn = 1.0, bool acquireTimeGridActiveIn = false);

	//Destructor
	~AcquireViewWaveformDisplay();
	
	void draw();
	
	void mouseDown(Point2D mouseLoc);
	void mouseUp(Point2D mouseLoc);
  void mouseMotion(Point2D mouseLoc);
  
  bool isAudioElement(Point2D mouseLoc);
  bool isInPlayRegion(Point2D mouseLoc) { return playheadStart->isInPlayRegion(mouseLoc); }
  
  void setPlayRegion(long startSampleIn, long playLengthIn);
	void updatePlayheadTimeGridToMatchAcquireSpecs();
	
	void updateData( SAMPLE** dataIn, int numChannelsIn, long dataLengthIn, int numChannelsDisplayedIn = 0 );
  
  //time grid handling fns
  bool acquireTimeGridIsActive() { return acquireTimeGrid->isActive(); };
  void setAcquireTimeGridActive(bool acquireTimeGridActiveIn) { acquireTimeGrid->setActive(acquireTimeGridActiveIn); };  
  void setAcquireTimeGridNumBeats(int timeGridNumBeatsIn) { acquireTimeGrid->setNumBeats(timeGridNumBeatsIn); };
  int getAcquireTimeGridNumBeats() { return acquireTimeGrid->getNumBeats(); };
  long getAcquireTimeGridNumSamplesPerDivision() { return acquireTimeGrid->getNumSamplesPerDivision(); };  
  void setAcquireTimeGridBeatDivision(float beatDivisionIn) { acquireTimeGrid->setBeatDivision(beatDivisionIn); };
  float getAcquireTimeGridBeatDivision() { return acquireTimeGrid->getBeatDivision(); };
  void displayAcquireTimeGrid(Point2D mouseLoc);
  void setDrawAcquireTimeGrid(bool drawAcquireTimeGridIn) { drawAcquireTimeGrid = drawAcquireTimeGridIn; };

  bool playheadStartTimeGridIsActive() { return playheadStartTimeGrid->isActive(); };
  void setPlayheadStartTimeGridActive(bool playheadStartTimeGridActiveIn) { playheadStartTimeGrid->setActive(playheadStartTimeGridActiveIn); };    
  
  void setPlayheadStartTimeGridNumBeats(int timeGridNumBeatsIn);
  
  int getPlayheadStartTimeGridNumBeats() { return playheadStartTimeGrid->getNumBeats(); };
  long getPlayheadStartTimeGridNumSamplesPerDivision() { return playheadStartTimeGrid->getNumSamplesPerDivision(); };
  void setPlayheadStartTimeGridBeatDivision(float beatDivisionIn) { playheadStartTimeGrid->setBeatDivision(beatDivisionIn); };
  float getPlayheadStartTimeGridBeatDivision() { return playheadStartTimeGrid->getBeatDivision(); };
  void displayPlayheadStartTimeGrid(Point2D mouseLoc);
  void setDrawPlayheadStartTimeGrid(bool drawPlayheadStartTimeGridIn) { drawPlayheadStartTimeGrid = drawPlayheadStartTimeGridIn; };
  
  void setDrawBeatLockedModePlayhead(bool drawIn) { drawBeatLockedModePlayhead = drawIn; };
  void displayBeatLockedModePlayhead(Point2D mouseLoc);
  
  void reverseSelection();
	void setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn);

protected:

	SAMPLE** data;
	SAMPLE* monoData;	//Stores a mono mix if data is stereo for summary display
	SAMPLE** summaryData;	//Stores a reduced "envelope" of the data for more efficient display
	long dataLength;
	long summaryLength;
	float numIndicesPerSummarySample;
	bool usingSummary;
	
	long playLoc;	
	
	int numChannels;
	int numChannelsDisplayed;	
	
	float scrollBarWidth;
	float waveformDisplayPadding;
	bool scrollOnBottom;
	
	AcquireViewScrollBar* hScroll;
	LineDisplay2D** waveformDisplays;
	
	Playhead* playheadStart;
	bool drawPlayheadStartTimeGrid;	
	AcquireTimeGrid* playheadStartTimeGrid;
	Playhead* playheadMoving;
	Playhead* mouseFollowPlayheadForBeatLockedTimeGridMode;
	bool drawBeatLockedModePlayhead;
	
	bool drawAcquireTimeGrid;
	AcquireTimeGrid* acquireTimeGrid;
	
	float dataIndexValueToXPos(long indexVal);
	long xPosToDataIndexValue( float xPos );
	
	bool lockedForDataUpdate;
 	
};

#endif