#include "blockItemScrollBar.h"

BlockItemScrollBar :: BlockItemScrollBar(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, float numPixelsPerItemIn, long numItemsIn, float minDisplayedPixelOffsetIn): UiElement(xMinIn, xMaxIn, yMinIn, yMaxIn) 
{ 
	numPixelsPerItem = numPixelsPerItemIn;
	numItems = numItemsIn; 
	maxPixelPopulated = numItems * numPixelsPerItem;
	minDisplayedPixelOffset = minDisplayedPixelOffsetIn;
	displayHeight = yMax - yMin;
	
	minimumActiveRectWidth = 20.0;		
	
	//If we have less to display than the display region can fit, make the active rect the whole bar
	if( maxPixelPopulated < displayHeight )
	{
		activeRectMaxY = yMax - 2;		
		activeRectMinY = yMin + 1;
		
		activeWidth = activeRectMaxY - activeRectMinY;		
	}
	else
	{
		float wouldBeActiveRectMaxY = yMax - 2 - minDisplayedPixelOffset/maxPixelPopulated * ((yMax-2) - (yMin+1));		
		float wouldBeActiveRectMinY = wouldBeActiveRectMaxY - (displayHeight/maxPixelPopulated) * ((yMax-2) - (yMin+1));
		
		activeWidth = wouldBeActiveRectMaxY - wouldBeActiveRectMinY;
		//ENFORCE MIN ACTIVE WIDTH?
		
		if( wouldBeActiveRectMinY < yMin+1 )
		{
			wouldBeActiveRectMinY = yMin+1;
			wouldBeActiveRectMaxY = wouldBeActiveRectMinY + activeWidth;
		}
		
		activeRectMaxY = wouldBeActiveRectMaxY;		
		activeRectMinY = wouldBeActiveRectMinY;		
		
	}

}



void BlockItemScrollBar :: draw()
{
  glColor3f( 1.0, 1.0, 1.0);
  glLineWidth( 1.0 );	  	

 	glBegin( GL_LINE_STRIP );
 		glVertex3f(xMin, yMin, 0.0);
 		glVertex3f(xMin, yMax, 0.0);
 		glVertex3f(xMax, yMax, 0.0);
 		glVertex3f(xMax, yMin, 0.0);
 		glVertex3f(xMin, yMin, 0.0);
	glEnd();
	
	glRectf(xMin + 1, activeRectMinY, xMax - 2, activeRectMaxY);
}



void BlockItemScrollBar :: mouseDown(Point2D mouseLoc)
{		
		active = true;	
		initialMouseLoc = mouseLoc;
		initialUpperEdgeActiveY = activeRectMaxY;
}



void BlockItemScrollBar :: mouseUp(Point2D mouseLoc)
{
	active = false;
}


void BlockItemScrollBar :: mouseMotion(Point2D mouseLoc)
{	
	//Only scroll if we have more to display than we can display at once
	if( maxPixelPopulated > displayHeight )
	{
		//Y position of the upper edge of the active rectangle
		float upperEdgeActiveY = initialUpperEdgeActiveY + (mouseLoc.y - initialMouseLoc.y);
		
		if( upperEdgeActiveY > yMax - 2 )
			upperEdgeActiveY = yMax - 2;
		else if( upperEdgeActiveY < yMin + 1 + activeWidth )
			upperEdgeActiveY = yMin + 1 + activeWidth;
			
		minDisplayedPixelOffset = ( (upperEdgeActiveY - (yMax - 2)) / ((yMin + 1) - (yMax - 2) + activeWidth) ) * (maxPixelPopulated-displayHeight);
		
		activeRectMaxY = upperEdgeActiveY;
		activeRectMinY = upperEdgeActiveY - activeWidth;	
	}
	
}



void BlockItemScrollBar :: setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn)
{
	xMin = xMinIn;
	xMax = xMaxIn;
	yMin = yMinIn;
	yMax = yMaxIn;
	
	displayHeight = yMax - yMin;
	
	//If we have less to display than the display region can fit, make the active rect the whole bar
	if( maxPixelPopulated < displayHeight )
	{
		activeRectMaxY = yMax - 2;		
		activeRectMinY = yMin + 1;
		
		activeWidth = activeRectMaxY - activeRectMinY;		
	}
	else
	{
		float wouldBeActiveRectMaxY = yMax - 2 - minDisplayedPixelOffset/maxPixelPopulated * ((yMax-2) - (yMin+1));		
		float wouldBeActiveRectMinY = wouldBeActiveRectMaxY - (displayHeight/maxPixelPopulated) * ((yMax-2) - (yMin+1));
		
		activeWidth = wouldBeActiveRectMaxY - wouldBeActiveRectMinY;
		//ENFORCE MIN ACTIVE WIDTH?
		
		if( wouldBeActiveRectMinY < yMin+1 )
		{
			wouldBeActiveRectMinY = yMin+1;
			wouldBeActiveRectMaxY = wouldBeActiveRectMinY + activeWidth;
		}
	
		activeRectMaxY = wouldBeActiveRectMaxY;		
		activeRectMinY = wouldBeActiveRectMinY;	
		
	}

}



void BlockItemScrollBar :: setNumItems(long numItemsIn)
{
	bool itemRemoved = (numItemsIn < numItems);
	
	numItems = numItemsIn;
	maxPixelPopulated = numItems * numPixelsPerItem;
	
	//If we have less to display than the display region can fit, make the active rect the whole bar
	if( maxPixelPopulated < displayHeight )
	{
		activeRectMaxY = yMax - 2;		
		activeRectMinY = yMin + 1;
		
		activeWidth = activeRectMaxY - activeRectMinY;		
	}
	else
	{
		float wouldBeActiveRectMaxY = yMax - 2 - minDisplayedPixelOffset/maxPixelPopulated * ((yMax-2) - (yMin+1));		
		float wouldBeActiveRectMinY = wouldBeActiveRectMaxY - (displayHeight/maxPixelPopulated) * ((yMax-2) - (yMin+1));
		
		activeWidth = wouldBeActiveRectMaxY - wouldBeActiveRectMinY;
		//ENFORCE MIN ACTIVE WIDTH?
		
		if( wouldBeActiveRectMinY < yMin+1 )
		{
			wouldBeActiveRectMinY = yMin+1;
			wouldBeActiveRectMaxY = wouldBeActiveRectMinY + activeWidth;
		}

		activeRectMaxY = wouldBeActiveRectMaxY;		
		activeRectMinY = wouldBeActiveRectMinY;
		
	}
	
	if( itemRemoved && maxPixelPopulated > displayHeight )
		minDisplayedPixelOffset = (yMax - activeRectMaxY) / ((yMax-2)-(yMin+1) - activeWidth) * (maxPixelPopulated-displayHeight);
	else if( itemRemoved && maxPixelPopulated < displayHeight )
		minDisplayedPixelOffset = 0.0;
	
}



void BlockItemScrollBar :: enforceMinActiveRectWidth()
{
/*	
	//Don't let the active rectangle get too small, even when very zoomed in! 
	if( activeRectMaxY - activeRectMinY < minimumActiveRectWidth )
	{
		float activeRectCenter = (activeRectMaxY + activeRectMinY) / 2.0;
		float wouldBeActiveRectMinY =	activeRectCenter - minimumActiveRectWidth/2.0;
		float wouldBeActiveRectMaxY = activeRectCenter + minimumActiveRectWidth/2.0;
		
		if( wouldBeActiveRectMinY < yMin + 1.0 )
		{
			activeRectMinY = yMin + 1.0;
			activeRectMaxY = activeRectMinY + minimumActiveRectWidth;	
		}
		else if( wouldBeActiveRectMaxY > yMax - 2.0 )
		{
			activeRectMaxY = yMax - 2.0;
			activeRectMinY = activeRectMaxY - minimumActiveRectWidth;
		}
		else
		{
			activeRectMinY = wouldBeActiveRectMinY;
			activeRectMaxY = wouldBeActiveRectMaxY;	
		}
	}	
	*/
}



void BlockItemScrollBar :: clickOutsideActiveRect(Point2D mouseLoc)
{
	float wouldBeActiveRectMaxY;
	
	if( mouseLoc.y > activeRectMaxY )
		wouldBeActiveRectMaxY = activeRectMaxY + ((yMax-2)-(yMin+1))*0.1;	//((yMax-2)-(yMin+1) - activeWidth)*0.1
	else
		wouldBeActiveRectMaxY = activeRectMaxY - ((yMax-2)-(yMin+1))*0.1;
	
	if( wouldBeActiveRectMaxY > yMax - 2 )
		wouldBeActiveRectMaxY = yMax - 2;
	else if( wouldBeActiveRectMaxY - activeWidth < yMin + 1 )
		wouldBeActiveRectMaxY = yMin + 1 + activeWidth;
		
	activeRectMaxY = wouldBeActiveRectMaxY;
	activeRectMinY = activeRectMaxY - activeWidth;
	
	minDisplayedPixelOffset = (yMax - activeRectMaxY) / ((yMax-2)-(yMin+1) - activeWidth) * (maxPixelPopulated-displayHeight);
}



void BlockItemScrollBar :: setMinDisplayedPixelOffset( float offsetIn )
{
	minDisplayedPixelOffset = offsetIn;
	
	//If we have less to display than the display region can fit, make the active rect the whole bar
	if( maxPixelPopulated < displayHeight )
	{
		activeRectMaxY = yMax - 2;		
		activeRectMinY = yMin + 1;
		
		activeWidth = activeRectMaxY - activeRectMinY;		
	}
	else
	{
		float wouldBeActiveRectMaxY = yMax - 2 - minDisplayedPixelOffset/maxPixelPopulated * ((yMax-2) - (yMin+1));		
		float wouldBeActiveRectMinY = wouldBeActiveRectMaxY - (displayHeight/maxPixelPopulated) * ((yMax-2) - (yMin+1));
		
		activeWidth = wouldBeActiveRectMaxY - wouldBeActiveRectMinY;
		//ENFORCE MIN ACTIVE WIDTH?
		
		if( wouldBeActiveRectMinY < yMin+1 )
		{
			wouldBeActiveRectMinY = yMin+1;
			wouldBeActiveRectMaxY = wouldBeActiveRectMinY + activeWidth;
		}
		
		activeRectMaxY = wouldBeActiveRectMaxY;		
		activeRectMinY = wouldBeActiveRectMinY;
		
	}

}