#include "fileBrowseWindow.h"

FileBrowseWindow :: FileBrowseWindow(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, float windowWidthIn, float windowHeightIn,  SAMPLE** inFileBufferIn, AcquireViewWaveformDisplay* waveformDisplayIn, AcquireControlsWindow* acquireControlsWindowIn, std::string currentDirIn): UiElement(xMinIn, xMaxIn, yMinIn, yMaxIn)
{
	entryHeight = 15.0;	
	scrollBarWidth = 15.0;
	titleBarHeight = 23.0;
	
	windowWidth = windowWidthIn;
	windowHeight = windowHeightIn;	
	
	itemDisplayPaddingX = 3.0;
	itemDisplayPaddingY = 0.0;	
	
	vScroll = new BlockItemScrollBar(xMax-scrollBarWidth, xMax, yMin, yMax - 1 - titleBarHeight - 1, entryHeight, 0, 0);	
	
	minDisplayedPixelOffset = 0.0;
  numItemsFitDisplay = ceil((yMax - yMin - titleBarHeight - 3 - 14)/entryHeight) + 1;
  minItemIndexDisplayed = floor(minDisplayedPixelOffset/entryHeight);	
  
	currentDir = currentDirIn;
  activeItem = -1;	
  
  fileReader = new stk::FileRead();
  inFileBuffer = inFileBufferIn;
  fileWriter = new stk::FileWrite();
  waveformDisplay = waveformDisplayIn;
  acquireControlsWindow = acquireControlsWindowIn;
  
  loadDirectoryContents();
}


FileBrowseWindow :: ~FileBrowseWindow()
{
	delete vScroll;	
	delete fileReader;
}


void FileBrowseWindow :: draw()
{
  glColor3f( 1.0, 1.0, 1.0);
  	
  glLineWidth( 1.0 );	  	

 	glBegin( GL_LINE_STRIP );
 		glVertex3f(xMin, yMin, 0.0);
 		glVertex3f(xMin, yMax, 0.0);
 		glVertex3f(xMax, yMax, 0.0);
 		glVertex3f(xMax, yMin, 0.0);
 		glVertex3f(xMin, yMin, 0.0);
	glEnd();
	
	
  // map the view port to the window location
//  glViewport( xMin, yMin + itemDisplayPaddingY, xMax-xMin-scrollBarWidth, yMax-yMin - 2.0*itemDisplayPaddingY );    
  glViewport( xMin, yMin, xMax-xMin-scrollBarWidth, yMax-yMin );    
  // set the matrix mode to project
  glMatrixMode( GL_PROJECTION );
  // load the identity matrix
  glLoadIdentity( );
	//Work in ortho mode
//	gluOrtho2D(xMin, xMax-scrollBarWidth, yMin + itemDisplayPaddingY, yMax - itemDisplayPaddingY );
	gluOrtho2D(xMin, xMax-scrollBarWidth, yMin, yMax);
  // set the matrix mode to modelview
  glMatrixMode( GL_MODELVIEW );
  // load the identity matrix
  glLoadIdentity( );
  // position the view point
  gluLookAt( 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f );


	//Draw list of items
	for( int i=0; i < numItemsFitDisplay && minItemIndexDisplayed + i < itemDisplays.size(); i++ )
		itemDisplays[i + minItemIndexDisplayed]->drawWithOffset(minDisplayedPixelOffset - 2);	

		
	glColor3f( 0.0, 0.0, 0.0);
		
	glRectf(xMin + 1, yMax - 1 - 8 - titleBarHeight - 2, xMax - scrollBarWidth - 1, yMax - titleBarHeight - 2);
	glRectf(xMin + 1, yMin+1, xMax - scrollBarWidth - 1, yMin + 1 + 8);	


	
  // map the view port back to the whole window
  glViewport( 0, 0, windowWidth, windowHeight );
  // set the matrix mode to project
  glMatrixMode( GL_PROJECTION );
  // load the identity matrix
  glLoadIdentity( );
  // set to ortho for 2D version
	gluOrtho2D(0, windowWidth, 0, windowHeight);
  // set the matrix mode to modelview
  glMatrixMode( GL_MODELVIEW );
  // load the identity matrix
  glLoadIdentity( );
  // position the view point
  gluLookAt( 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f );	
  
  
  
  	//Title bar	
	
	glColor3f( 0.4, 0.4, 0.4 );
	glRectf( xMin + 1, yMax - 1 - titleBarHeight - 1, xMax - 1, yMax - 1);	

	glColor3f( 1.0, 1.0, 1.0 );			
	glBegin( GL_LINE_STRIP );
		glVertex3f( xMin, yMax - 1 - titleBarHeight - 1, 0.0);
		glVertex3f( xMax, yMax - 1 - titleBarHeight - 1, 0.0);		
	glEnd();

	glColor3f( 1.0, 1.0, 1.0 );							
	bitmapString( "Files", xMin + (xMax - xMin - 38 - scrollBarWidth)/2.0, yMax - titleBarHeight + 4, 99999, 2);
	
	
	vScroll->draw();
	  
}



void FileBrowseWindow :: setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, float windowWidthIn, float windowHeightIn)
{
	xMin = xMinIn;
	xMax = xMaxIn;
	yMin = yMinIn;
	yMax = yMaxIn;
	
	vScroll->setPosition(xMaxIn-scrollBarWidth, xMaxIn, yMinIn,  yMaxIn - 1 - titleBarHeight - 1);
	
	for( int i=0; i < itemDisplays.size(); i++ )
	{
		float xMinItemWindow = xMin + itemDisplayPaddingX;
		float xMaxItemWindow = xMax - scrollBarWidth - itemDisplayPaddingX;
		float yMinItemWindow = yMax - titleBarHeight - ((i + 1) * entryHeight - itemDisplayPaddingY/2) - 8;
		float yMaxItemWindow = yMax - titleBarHeight - itemDisplayPaddingY - (i * entryHeight + itemDisplayPaddingY/2) - 8;
		
		itemDisplays[i]->setPosition(xMinItemWindow, xMaxItemWindow, yMinItemWindow, yMaxItemWindow);
	}
	
	windowWidth = windowWidthIn;
	windowHeight = windowHeightIn;
  numItemsFitDisplay = ceil((yMax - yMin - titleBarHeight - 3 - 16)/entryHeight) + 1;
}



using namespace std;

void FileBrowseWindow :: loadDirectoryContents()
{
	//Clear old items
	itemDisplays.clear();
	
	//Try to open the directory
  DIR *dp;
  struct dirent *dirp;
  if((dp  = opendir(currentDir.c_str())) == NULL) 
  	cout << "Error(" << errno << ") opening " << currentDir << endl;

	string itemName;
	string fullPath;
	string fileExtension;
	struct stat statbuf;
	
	vector<string> wavNames;
	vector<string> dirNames;
	
	//For each item in the directory, add it to either the directory or .wav file lists if it is one
  while ((dirp = readdir(dp)) != NULL) 
  {   	
   	itemName = dirp->d_name;
//   	fullPath = currentDir + "/" + itemName;
   	fullPath = currentDir + itemName;
   	fullPath = fullPath.substr(2);
   	
//   	cout << fullPath << endl;
//   	cout << itemName << endl;
   	
   	//Get file extension if there is one
   	size_t lastDot = itemName.find_last_of( "." );
   	if( lastDot < itemName.length()-1 )
    	fileExtension = itemName.substr(lastDot+1);
   	else
   		fileExtension = "";
    
    
    //Check if this is a directory
    if (stat( fullPath.c_str(), &statbuf) == -1) 
     	cout << "Error while calling stat()" << endl;
 		else 
 		{
 			//If directory
 			if(	S_ISDIR(statbuf.st_mode) && !(itemName.compare(".") == 0) ) // && !(itemName.compare("..") == 0 ))
				dirNames.push_back(itemName);
			//else if wav file
			else if( fileExtension.compare("wav") == 0 )
				wavNames.push_back(itemName);
 		}
    	
  }
   
  closedir(dp);
 
 	float xMinItemDisplay;
	float xMaxItemDisplay;
	float yMinItemDisplay;
	float yMaxItemDisplay;
 
 	//Add these items to the display list, starting with the directories 
  for( int i=0; i < dirNames.size(); i++ )
  {
 		xMinItemDisplay = xMin + itemDisplayPaddingX;
		xMaxItemDisplay = xMax - scrollBarWidth - itemDisplayPaddingX;
		yMinItemDisplay = yMax - titleBarHeight - ((itemDisplays.size() + 1) * entryHeight - itemDisplayPaddingY/2) - 8;
		yMaxItemDisplay = yMax - titleBarHeight - itemDisplayPaddingY - (itemDisplays.size() * entryHeight + itemDisplayPaddingY/2) - 8;
	
  	itemDisplays.push_back( new FileItemDisplay( xMinItemDisplay, xMaxItemDisplay, yMinItemDisplay, yMaxItemDisplay, dirNames[i], true ) );	
  }
  
  for( int i=0; i < wavNames.size(); i++ )
  {
 		xMinItemDisplay = xMin + itemDisplayPaddingX;
		xMaxItemDisplay = xMax - scrollBarWidth - itemDisplayPaddingX;
		yMinItemDisplay = yMax - titleBarHeight - ((itemDisplays.size() + 1) * entryHeight - itemDisplayPaddingY/2) - 8;
		yMaxItemDisplay = yMax - titleBarHeight - itemDisplayPaddingY - (itemDisplays.size() * entryHeight + itemDisplayPaddingY/2) - 8;
		
  	itemDisplays.push_back( new FileItemDisplay( xMinItemDisplay, xMaxItemDisplay, yMinItemDisplay, yMaxItemDisplay, wavNames[i], false ) );
  }
  
 	vScroll->setNumItems( itemDisplays.size() + 1 );
  
}



void FileBrowseWindow :: mouseDown(Point2D mouseLoc)
{
	//If the click was inside the scroller
	if( vScroll->isInside(mouseLoc) )
	{
		//Do the scrollBar's mouseDown. This sets the scrollBar active and sets the initial mouse position.
		vScroll->mouseDown(mouseLoc);
		
		//If we did not click inside the active rect, update the scrollbar position based on the click
		if( !vScroll->isInsideActiveRect(mouseLoc) )
			vScroll->clickOutsideActiveRect(mouseLoc);
			
		minDisplayedPixelOffset = vScroll->getMinDisplayedPixelOffset();	
	  minItemIndexDisplayed = floor(minDisplayedPixelOffset/entryHeight);				
	}
	
	//Otherwise the click was inside the items menu area
	else
	{

		//if an item was previously active, make it unactive
		if(activeItem != -1)
			itemDisplays[activeItem]->setActive(false);
	
		active = false;
		
		activeItem = -1;
		int i=0;
		
		//See if the click was on any of the items
		while( i < itemDisplays.size() && activeItem == -1 )
		{
			Point2D offsetMouseLoc = mouseLoc;
			offsetMouseLoc.y = offsetMouseLoc.y - minDisplayedPixelOffset;
			
			if( itemDisplays[i]->isInside(offsetMouseLoc) )
			{
				itemDisplays[i]->mouseDown(offsetMouseLoc);	
				activeItem = i;
				active = true;
			}
			
			i++;
		}

	}
	
}


void FileBrowseWindow :: mouseUp(Point2D mouseLoc)
{
	vScroll->mouseUp(mouseLoc);
//	active = false;
}


void FileBrowseWindow :: mouseMotion(Point2D mouseLoc)
{
	if( vScroll->isActive() )
	{
		vScroll->mouseMotion(mouseLoc);
		minDisplayedPixelOffset = vScroll->getMinDisplayedPixelOffset();
	  minItemIndexDisplayed = floor(minDisplayedPixelOffset/entryHeight);		
	}
	
}



void FileBrowseWindow :: changeSelectedItem(bool up)
{
	if( up && activeItem > 0 )
	{
		itemDisplays[activeItem]->setActive(false);
		activeItem = activeItem - 1;
		itemDisplays[activeItem]->setActive(true);
		
		//If that item isn't fully displayed, make it so that it is
		if( activeItem == 0 )
		{
			minItemIndexDisplayed = 0;
			minDisplayedPixelOffset = 0.0;
		}
		else if( activeItem <= minItemIndexDisplayed )	//hanging off top
		{
			minItemIndexDisplayed = activeItem - 1;		
			minDisplayedPixelOffset = (minItemIndexDisplayed + 1) * entryHeight - 0.5*entryHeight;
		}
		//If that item is below the currently viewed items, jump down so that it shows at the bottom		
		else if( activeItem >= minItemIndexDisplayed + numItemsFitDisplay )		// - 1
		{
			minItemIndexDisplayed = activeItem + 1 - (numItemsFitDisplay - 1);
			minDisplayedPixelOffset = (minItemIndexDisplayed) * entryHeight + 0.3 * entryHeight + itemDisplayPaddingY + titleBarHeight + 2;
		}

		
		//update scrollbar to match minDisplayedPixelOffset
		vScroll->setMinDisplayedPixelOffset(minDisplayedPixelOffset);
		
	}
	
	
	else if( !up && activeItem < itemDisplays.size() - 1 )
	{
		itemDisplays[activeItem]->setActive(false);
		activeItem = activeItem + 1;	
		itemDisplays[activeItem]->setActive(true);				
		
		//If that item isn't fully displayed, make it so that it is
		if( activeItem == itemDisplays.size() - 1 && itemDisplays.size() * entryHeight > (yMax-2-titleBarHeight-2-16)-(yMin+1))
		{
			minItemIndexDisplayed = activeItem + 1 - (numItemsFitDisplay - 1);				
			minDisplayedPixelOffset = itemDisplays.size() * entryHeight - ((yMax-2-titleBarHeight-2)-(yMin+1)) - 3 + 18;
		}
		else if( activeItem >= minItemIndexDisplayed + numItemsFitDisplay - 2 && itemDisplays.size() * entryHeight > (yMax-2-titleBarHeight-2-16)-(yMin+1) )	//hanging off bottom
		{
			minItemIndexDisplayed = activeItem + 1 - (numItemsFitDisplay - 1) + 1;	
			minDisplayedPixelOffset = (minItemIndexDisplayed) * entryHeight + 0.3 * entryHeight + itemDisplayPaddingY + titleBarHeight + 2 - 30;
		}
		else if( activeItem <= minItemIndexDisplayed )
		{
			minItemIndexDisplayed = activeItem - 1;
			minDisplayedPixelOffset = (minItemIndexDisplayed + 1) * entryHeight - 0.5*entryHeight;				
		}
		
		
		//update scrollbar to match minDisplayedPixelOffset		
		vScroll->setMinDisplayedPixelOffset(minDisplayedPixelOffset);		
		
	}
	
}



void FileBrowseWindow :: scrollToBottom()
{
	if( itemDisplays.size() * entryHeight > (yMax-2-titleBarHeight-2)-(yMin+1) )
	{
		minItemIndexDisplayed = itemDisplays.size() - (numItemsFitDisplay - 1);
			
		minDisplayedPixelOffset = itemDisplays.size() * entryHeight - ((yMax-2-titleBarHeight-2)-(yMin+1)) - 3;
		vScroll->setMinDisplayedPixelOffset(minDisplayedPixelOffset);		
	}
}



void FileBrowseWindow :: scrollToIncludeActive()
{
	
	//If it is the first item
	if( activeItem == 0 )
	{
		minItemIndexDisplayed = 0;
		minDisplayedPixelOffset = 0.0;
	}
	//If it is the last item and the last item extends beyond the window
	else if( activeItem == itemDisplays.size() - 1 && itemDisplays.size() * entryHeight > (yMax-2-titleBarHeight-2)-(yMin+1))
	{
		minItemIndexDisplayed = activeItem + 1 - (numItemsFitDisplay - 1);				
		minDisplayedPixelOffset = itemDisplays.size() * entryHeight - ((yMax-2-titleBarHeight-2)-(yMin+1)) - 3 + 16;
	}
	//If that item is above the currently viewed items, jump up so that it shows at the top
	else if( activeItem <= minItemIndexDisplayed )
	{
		minItemIndexDisplayed = activeItem - 1;		
		minDisplayedPixelOffset = (minItemIndexDisplayed + 1) * entryHeight - 0.5*entryHeight;
	}
	//If that item is below the currently viewed items, jump down so that it shows at the bottom		
	else if( activeItem >= minItemIndexDisplayed + numItemsFitDisplay - 1 )
	{
		minItemIndexDisplayed = activeItem + 1 - (numItemsFitDisplay - 1);
		minDisplayedPixelOffset = (minItemIndexDisplayed) * entryHeight + 0.3 * entryHeight + itemDisplayPaddingY + titleBarHeight + 2 - 11;
	}
	
	//update scrollbar to match minDisplayedPixelOffset		
	vScroll->setMinDisplayedPixelOffset(minDisplayedPixelOffset);		
}



bool FileBrowseWindow :: activeItemIsWav()
{
	if( activeItem >= 0 )
		return itemDisplays[activeItem]->isWav();
	else
		return false;
}


void FileBrowseWindow :: openActiveItem()
{
	if( activeItemIsWav() )
	{
		//load wav
		std::string fullPath;
   	fullPath = currentDir + itemDisplays[activeItem]->getName();
   	fullPath = fullPath.substr(2);

		try{
			fileReader->open(fullPath);
		}
		catch( stk::StkError& err ) {
	    err.printMessage();
	    exit( 1 );
		}
		
		long fileLength = fileReader->fileSize();
		stk::StkFrames inFileBufferFrames(fileLength, 2);
		fileReader->read(inFileBufferFrames);
		
		inFileBuffer = new SAMPLE*[2];
		for( int i=0; i < 2; i++)
		{
			inFileBuffer[i] = new SAMPLE[fileLength];
			for( int j=0; j<fileLength; j++)
				inFileBuffer[i][j] = inFileBufferFrames(j,i);		
		}		

		acquireControlsWindow->setCurrentFileName( itemDisplays[activeItem]->getName() );
		waveformDisplay->updateData( inFileBuffer, 2, fileLength, 2 );

	}	
	
	else	//for now, item must be a directory if it isn't a .wav since those are the only types we load
	{
		string itemName =itemDisplays[activeItem]->getName();
		
		currentDir += itemName + "/";
		loadDirectoryContents();
		
	}
}


void FileBrowseWindow :: saveAudioData(AudioPlayer* audioPlayerIn, string fileName)
{
	//If not currently writing out a file
	if( !fileWriter->isOpen() )
	{
//		string fileName = "output.wav";
		string fullPath = currentDir + fileName;
		
		long playLength = audioPlayerIn->playLength;
		int numChannels = audioPlayerIn->numChannels;
		
		fileWriter->open(fullPath, audioPlayerIn->numChannels);
		
		stk::StkFrames outFileBufferFrames(playLength, numChannels);
		for( int i = 0; i < playLength; i++ )
		{
			for( int j = 0; j < numChannels; j++ )
			{
				outFileBufferFrames(i,j) = audioPlayerIn->bufferPointer[j][i];
			}
		}
		
		fileWriter->write(outFileBufferFrames);
		fileWriter->close();
		
		loadDirectoryContents();
	}
	
}


void FileBrowseWindow :: saveAllScraps(ScrapsWindow* scrapsWindowIn)
{

	string fileNameBase = "scrap";
	string fileNameEnd = ".wav";
	string fileName;
	
	for( int i = 0; i < scrapsWindowIn->getNumberOfScraps(); i++ )
	{
		stringstream fileNumber;
		fileNumber << (i+1);
		fileName = fileNameBase + fileNumber.str() + fileNameEnd;
		saveAudioData( scrapsWindowIn->getParticularScrapAudioPlayer(i), 	fileName );
	}	
}