#ifndef FILE_BROWSE_WINDOW
#define FILE_BROWSE_WINDOW

//Class which allows file browsing for importing of files

#include "uiElement.h"
#include "blockItemScrollBar.h"
#include "fileItemDisplay.h"
#include "scrapsWindow.h"
#include "math.h"
//#include "../audio/RtAudio.h"
#include "../audio/FileRead.h"
#include "../audio/FileWrite.h"
#include "audioPlayer.h"
#include "acquireViewWaveformDisplay.h"
#include "acquireControlsWindow.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <errno.h>
#include <vector>
#include <string>
#include <iostream>
#include <sstream>

class FileBrowseWindow : public UiElement
{
public:

	//Constructor
	FileBrowseWindow(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, float windowWidthIn, float windowHeightIn, SAMPLE** inFileBufferIn, AcquireViewWaveformDisplay* waveformDisplayIn, AcquireControlsWindow* acquireControlsWindowIn, std::string currentDirIn = "./data/");
	
	//Destructor
	~FileBrowseWindow();
	
	void draw();

	void mouseDown(Point2D mouseLoc);
	void mouseUp(Point2D mouseLoc);
  void mouseMotion(Point2D mouseLoc);  	
	void setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, float windowWidthIn, float windowHeightIn);
	
	void saveAudioData(AudioPlayer* audioPlayerIn, std::string fileName = "output.wav");
	void saveAllScraps(ScrapsWindow* scrapsWindowIn);
	void scrollToBottom();
	void scrollToIncludeActive();
	void changeSelectedItem(bool up);
	void openActiveItem();
	
protected:

	std::vector<FileItemDisplay*> itemDisplays;
	int activeItem;	
	std::string currentDir;

	stk::FileRead* fileReader;
	SAMPLE** inFileBuffer;
	stk::FileWrite* fileWriter;
	AcquireViewWaveformDisplay* waveformDisplay;
	AcquireControlsWindow* acquireControlsWindow;

	float entryHeight;
	float scrollBarWidth;
	float titleBarHeight;
	
	float windowWidth;
	float windowHeight;	
	
	float itemDisplayPaddingX;
	float itemDisplayPaddingY;
	
  float minDisplayedPixelOffset;
  long numItemsFitDisplay;
  long minItemIndexDisplayed;
  	
	BlockItemScrollBar* vScroll;
	
	void loadDirectoryContents();
	bool activeItemIsWav();	
	
};

#endif