//-----------------------------------------------------------------------------
// Name: reshapeFunc( )
// Desc: called when window size changes
//-----------------------------------------------------------------------------
void reshapeFunc( GLsizei w, GLsizei h )
{
		
    // save the new window size
    g_windowWidth = w; g_windowHeight = h;
    // map the view port to the client area
    glViewport( 0, 0, w, h );

    glMatrixMode( GL_PROJECTION );
    // load the identity matrix
    glLoadIdentity( );
    // set to ortho for 2D version
		gluOrtho2D(0, g_windowWidth, 0, g_windowHeight);
    // set the matrix mode to modelview
    glMatrixMode( GL_MODELVIEW );
    // load the identity matrix
    glLoadIdentity( );
    // position the view point
    gluLookAt( 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f );
    
    if( g_currentView == ACQUIRE )
    {
    	g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ]->setPosition(1, g_windowWidth, 1, g_windowHeight/3.0 + g_windowHeightAdjust);
    	
    	ScrapsWindow* scrapsWindow = (ScrapsWindow*)g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ];
    	scrapsWindow->setPosition(g_windowWidth / 5.0, g_windowWidth * 3.0 / 5.0, g_windowHeight/3.0 + 2 + g_windowHeightAdjust, g_windowHeight, g_windowWidth, g_windowHeight);
    	
    	FileBrowseWindow* fileBrowseWindow = (FileBrowseWindow*)g_acquireViewUiElements[ FILE_BROWSE_WINDOW_DISPLAY ];
    	fileBrowseWindow->setPosition(1, g_windowWidth / 5.0 - 2.0, g_windowHeight/3.0 + 2.0 + g_windowHeightAdjust, g_windowHeight, g_windowWidth, g_windowHeight);
    	
    	g_acquireViewUiElements[ ACQUIRE_CONTROLS_WINDOW_DISPLAY ]->setPosition( g_windowWidth * 3.0 / 5.0 + 2.0, g_windowWidth, g_windowHeight/3.0 + 2.0 + g_windowHeightAdjust, g_windowHeight / 3.0 + 50 + g_windowHeightAdjust);
    	
    	g_acquireViewUiElements[ SCRAP_DETAIL_WINDOW_DISPLAY ]->setPosition( g_windowWidth * 3.0 / 5.0 + 2.0, g_windowWidth, g_windowHeight / 3.0 + 52.0 + g_windowHeightAdjust, g_windowHeight);
    }
    else if( g_currentView == CREATE )
    {
     	ScrapsWindow* scrapsWindow = (ScrapsWindow*)g_createViewUiElements[ CREATE_SCRAPS_WINDOW ];
    	scrapsWindow->setPosition(1, g_windowWidth / 5.0 - 2.0, 1, g_windowHeight, g_windowWidth, g_windowHeight);
    	
    	ProcessesBrowseWindow* processesWindow = (ProcessesBrowseWindow*)g_createViewUiElements[ CREATE_PROCESSES_WINDOW ];
    	processesWindow->setPosition(g_windowWidth / 5.0, g_windowWidth, 4.2 * g_windowHeight / 5.0 + 2.0, g_windowHeight, g_windowWidth, g_windowHeight);
    	
    	g_createViewUiElements[ CREATE_PROCESS_WINDOW ]->setPosition( g_windowWidth / 5.0, g_windowWidth, 1, g_windowHeight * 4.2 / 5.0);
		
    }
}



//Function to deal with special keys (arrow keys)
void keyboardSpecial( int key, int x, int y)
{
	
	switch( key )
	{
		case GLUT_KEY_LEFT:
			if( g_currentView == ACQUIRE )
			{
				
				//If the acquire waveform display is the active audio element
				if( (g_audioActiveElement && (g_audioActiveElement == g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ])) || g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY]->isInside(g_mouseLoc) )
				{
					//Make a properly cast pointer to the display
					AcquireViewWaveformDisplay* display = (AcquireViewWaveformDisplay*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];

					//If using acquire time grid mode
					if( display->acquireTimeGridIsActive() )
					{
						//If the mouse is currently over the waveform display
						if( display->isInside(g_mouseLoc) )
						{
								//Decrement the number of beats displayed and force update the time grid display
								//Leave the playhead time grid alone - we don't want to change it until the next mouse down
								display->setAcquireTimeGridNumBeats( display->getAcquireTimeGridNumBeats() - 1 );	
								display->displayAcquireTimeGrid(g_mouseLoc);
								
								AcquireControlsWindow* controlsWindow = (AcquireControlsWindow*) g_acquireViewUiElements[ ACQUIRE_CONTROLS_WINDOW_DISPLAY ];
								int numBeats = display->getAcquireTimeGridNumBeats();
								if( numBeats < 0)
									numBeats = -numBeats;
									
								controlsWindow->setNumBeats( numBeats );								
						}
					}
					//Otherwise, if we are using playhead time grid mode
					// update both (only the playhead time grid is being displayed, but we want the acquire time grid to match if we go back to that mode)
					else if( display->playheadStartTimeGridIsActive() )
					{
						//Only update if we have more than two beats showing. We won't let this mode get to where no divisions are drawn because it could then be confused with the normal playhead mode
						if( display->getPlayheadStartTimeGridNumBeats() > 2 )
						{
//							display->setAcquireTimeGridNumBeats( display->getAcquireTimeGridNumBeats() - 1 );	
							display->setPlayheadStartTimeGridNumBeats( display->getPlayheadStartTimeGridNumBeats() - 1 );

							AcquireControlsWindow* controlsWindow = (AcquireControlsWindow*) g_acquireViewUiElements[ ACQUIRE_CONTROLS_WINDOW_DISPLAY ];
							int numBeats = display->getPlayheadStartTimeGridNumBeats();
							if( numBeats < 0)
								numBeats = -numBeats;
									
							controlsWindow->setNumBeats( numBeats );								

						}
					}

				}
			}
	  	break;		
		
		
		
	  case GLUT_KEY_RIGHT:
			if( g_currentView == ACQUIRE )
			{
				
				//If the acquire waveform display is the active audio element
				if( (g_audioActiveElement && (g_audioActiveElement == g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ]) ) || g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY]->isInside(g_mouseLoc) )
				{
					//Make a properly cast pointer to the display
					AcquireViewWaveformDisplay* display = (AcquireViewWaveformDisplay*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
			
					//If using acquire time grid mode
					if( display->acquireTimeGridIsActive() )
					{
						//If the mouse is currently over the waveform display
						if( display->isInside(g_mouseLoc) )
						{
							//Decrement the number of beats displayed and force update the time grid display
							//Leave the playhead time grid alone - we only want to update it on the next mouse down
							display->setAcquireTimeGridNumBeats( display->getAcquireTimeGridNumBeats() + 1 );
							display->displayAcquireTimeGrid(g_mouseLoc);

							AcquireControlsWindow* controlsWindow = (AcquireControlsWindow*) g_acquireViewUiElements[ ACQUIRE_CONTROLS_WINDOW_DISPLAY ];
							int numBeats = display->getAcquireTimeGridNumBeats();
							if( numBeats < 0)
								numBeats = -numBeats;
									
							controlsWindow->setNumBeats( numBeats );								
						}
					}
					//Otherwise, if we are using playhead time grid mode
					// update both (only the playhead time grid is being displayed, but we want the acquire time grid to match if we go back to that mode)
					else if( display->playheadStartTimeGridIsActive() )
					{
//						display->setAcquireTimeGridNumBeats( display->getAcquireTimeGridNumBeats() + 1 );	
						display->setPlayheadStartTimeGridNumBeats( display->getPlayheadStartTimeGridNumBeats() + 1 );
						
						AcquireControlsWindow* controlsWindow = (AcquireControlsWindow*) g_acquireViewUiElements[ ACQUIRE_CONTROLS_WINDOW_DISPLAY ];
						int numBeats = display->getPlayheadStartTimeGridNumBeats();
						if( numBeats < 0)
							numBeats = -numBeats;
									
						controlsWindow->setNumBeats( numBeats );								

					}
					
				}
				else if( g_audioActiveElement && (g_audioActiveElement == g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ]) )
				{
					ScrapsWindow* scrapsWindow = (ScrapsWindow*) g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ];
					scrapsWindow->openActiveScrapInDetail();
				}
			}
	  	break;	  
	  
	  
	  
	  case GLUT_KEY_UP:
 			if( g_currentView == ACQUIRE )
			{
				AcquireViewWaveformDisplay* waveformDisplay = (AcquireViewWaveformDisplay*) g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
								
				//If we have a scrap summary selected in the scraps window
				if( g_audioActiveElement && g_audioActiveElement == g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ] && (g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY]->isInside(g_mouseLoc) || g_acquireViewUiElements[ SCRAP_DETAIL_WINDOW_DISPLAY ]->isInside(g_mouseLoc) ))
				{
					ScrapsWindow* scrapsWindow = (ScrapsWindow*)g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ];
					scrapsWindow->changeSelectedScrap(true);	//true => up
					g_spaceBarAudioPlayer = scrapsWindow->getAudioPlayer();
				}
				else if( g_acquireViewUiElements[ FILE_BROWSE_WINDOW_DISPLAY ]->isActive() && g_acquireViewUiElements[ FILE_BROWSE_WINDOW_DISPLAY ]->isInside(g_mouseLoc) )
				{
					FileBrowseWindow* fileBrowseWindow = (FileBrowseWindow*)g_acquireViewUiElements[ FILE_BROWSE_WINDOW_DISPLAY ];
					fileBrowseWindow->changeSelectedItem(true);	//true => up
				}
				else if( waveformDisplay->playheadStartTimeGridIsActive() && waveformDisplay->isInside(g_mouseLoc) )
				{
						
				}
				
			}
			else if( g_currentView == CREATE )
			{
				if( g_audioActiveElement && g_audioActiveElement == g_createViewUiElements[ CREATE_SCRAPS_WINDOW ] && g_createViewUiElements[ CREATE_SCRAPS_WINDOW]->isInside(g_mouseLoc) )
				{
					ScrapsWindow* scrapsWindow = (ScrapsWindow*)g_createViewUiElements[ CREATE_SCRAPS_WINDOW ];
					scrapsWindow->changeSelectedScrap(true);	//true => up
					g_spaceBarAudioPlayer = scrapsWindow->getAudioPlayer();
				}
				else if( g_createViewUiElements[ CREATE_PROCESSES_WINDOW]->isActive() && g_createViewUiElements[ CREATE_PROCESSES_WINDOW ]->isInside(g_mouseLoc) )
				{
					ProcessesBrowseWindow* processesBrowseWindow = (ProcessesBrowseWindow*)g_createViewUiElements[ CREATE_PROCESSES_WINDOW ];
					processesBrowseWindow->changeSelectedItem(true);	//true => up
				}
			}
	  	break;
	  
  	case GLUT_KEY_DOWN:
 			if( g_currentView == ACQUIRE )
			{
				AcquireViewWaveformDisplay* waveformDisplay = (AcquireViewWaveformDisplay*) g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
				
				//If we have a scrap summary selected in the scraps window
				if( g_audioActiveElement && g_audioActiveElement == g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ] && (g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY]->isInside(g_mouseLoc) || g_acquireViewUiElements[ SCRAP_DETAIL_WINDOW_DISPLAY ]->isInside(g_mouseLoc) ) )
				{
					ScrapsWindow* scrapsWindow = (ScrapsWindow*)g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ];
					scrapsWindow->changeSelectedScrap(false);	//false => down
					g_spaceBarAudioPlayer = scrapsWindow->getAudioPlayer();
				}
				else if( g_acquireViewUiElements[ FILE_BROWSE_WINDOW_DISPLAY ]->isActive() && g_acquireViewUiElements[ FILE_BROWSE_WINDOW_DISPLAY ]->isInside(g_mouseLoc) )
				{
					FileBrowseWindow* fileBrowseWindow = (FileBrowseWindow*)g_acquireViewUiElements[ FILE_BROWSE_WINDOW_DISPLAY ];
					fileBrowseWindow->changeSelectedItem(false);	//false => down
				}	
				else if( waveformDisplay->playheadStartTimeGridIsActive() && waveformDisplay->isInside(g_mouseLoc) )
				{
//					float currentBeatDivision = waveformDisplay->getPlayheadStartTimeGridBeatDivision();
//					waveformDisplay->setPlayheadStartTimeGridBeatDivision( currentBeatDivision * 2 );
				}
			
				
			}  	
			else if( g_currentView == CREATE )
			{
				if( g_audioActiveElement && g_audioActiveElement == g_createViewUiElements[ CREATE_SCRAPS_WINDOW ] && g_createViewUiElements[ CREATE_SCRAPS_WINDOW]->isInside(g_mouseLoc) )
				{
					ScrapsWindow* scrapsWindow = (ScrapsWindow*)g_createViewUiElements[ CREATE_SCRAPS_WINDOW ];
					scrapsWindow->changeSelectedScrap(false);	//false => down
					g_spaceBarAudioPlayer = scrapsWindow->getAudioPlayer();
				}
				else if( g_createViewUiElements[ CREATE_PROCESSES_WINDOW]->isActive() && g_createViewUiElements[ CREATE_PROCESSES_WINDOW ]->isInside(g_mouseLoc) )
				{
					ProcessesBrowseWindow* processesBrowseWindow = (ProcessesBrowseWindow*)g_createViewUiElements[ CREATE_PROCESSES_WINDOW ];
					processesBrowseWindow->changeSelectedItem(false);	//False => down
				}

			}
			
	  	break;
	}
    
}


//-----------------------------------------------------------------------------
// Name: keyboardFunc( )
// Desc: key event
//-----------------------------------------------------------------------------
void keyboardFunc( unsigned char key, int x, int y )
{
    switch( key )
    {
        case 'Q':
        case 'q':
 					exit(1);
 					break;
 				
 				//Trigger scraps players
 				case '1':
 				case '2':
 				case '3':
 				case '4':
 				case '5':
 				case '6':
 				case '7':
 				case '8':
 				case '9':
 				case '0':
	 				if( 1 )
	 				{
	 					if( g_spaceBarAudioPlayer && g_spaceBarAudioPlayer->isPlaying )
							g_spaceBarAudioPlayer->isPlaying = false;
						
						ScrapsWindow* scrapsWindow = (ScrapsWindow*) g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ];
						
						char* numberChar = (char*) &key;
						int number = atoi(numberChar) - 1;
						
						if( number == -1 )
							number = 9;
						
						if( number < scrapsWindow->getNumberOfScraps() )
						{
//		 					g_audioActiveElement = scrapsWindow;
		 					g_audioActiveElement = (AudioElement*) g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ];
		 					scrapsWindow->setParticularSummaryActive( number );	
		 					g_spaceBarAudioPlayer = scrapsWindow->getAudioPlayer();
						}
						
	 				}
 					break;
 				
 				//Set current audio player to loop
 				case 'l':
 					if( g_spaceBarAudioPlayer )
 						g_spaceBarAudioPlayer->isLooping = !g_spaceBarAudioPlayer->isLooping;

 					break;
 					
 				
 				//Set all scraps to loop or not loop
 				case 'L':
 					if( 1 )
 					{
 						ScrapsWindow* scrapsWindow = (ScrapsWindow*) g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ];
 						scrapsWindow->toggleAllScrapsLooping();
 					}
 					break;
 				
 				
 				//start currently playing sample over (if something is playing)
 				case '/':
 					if( g_spaceBarAudioPlayer && g_spaceBarAudioPlayer->isPlaying)
 						g_spaceBarAudioPlayer->currentIndex = g_spaceBarAudioPlayer->startSample;
 				
 					break;
 				
 				//Turn on time grid
 				case 'g':
 					if( g_currentView == ACQUIRE )
 					{
 						//If the mouse if over the waveform display
						if( g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ]->isInside(g_mouseLoc) )
 						{
 							AcquireViewWaveformDisplay* display = (AcquireViewWaveformDisplay*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];

	 						//If not currently in any grid mode
							if( !display->acquireTimeGridIsActive() && !display->playheadStartTimeGridIsActive() )
							{
								//Set both playhead and aquire grids active and drawing. This is tempo locked grid mode.
								display->setAcquireTimeGridActive(true);
								display->setPlayheadStartTimeGridActive(true);
								
								display->setDrawAcquireTimeGrid(true);
								display->setDrawPlayheadStartTimeGrid(true);
								
								//Change the selected play area to match the new tempo locked grid	
								display->updatePlayheadTimeGridToMatchAcquireSpecs();															
								
								//force dispay update before mouse movement for the acquire time grid
								if( display->isInside(g_mouseLoc) )
										display->displayAcquireTimeGrid(g_mouseLoc);
							}
							//Otherwise, if currently in tempo locked grid mode
							else if( display->acquireTimeGridIsActive() && display->playheadStartTimeGridIsActive() )
							{
								//Turn off the acquire time grid. Now we just have the playhead time grid. This is beat locked grid mode.
								display->setAcquireTimeGridActive(false);
								display->setDrawAcquireTimeGrid(false);
								
								//Set the playhead start time grid number of beats based on ending conditions of the previous mode
								if( abs( display->getAcquireTimeGridNumBeats() ) < 2 )
									display->setPlayheadStartTimeGridNumBeats( 2 );
								else if( display->getAcquireTimeGridNumBeats() < 0 )
									display->setPlayheadStartTimeGridNumBeats( -display->getAcquireTimeGridNumBeats() );		
								else
									display->setPlayheadStartTimeGridNumBeats( display->getAcquireTimeGridNumBeats() );
									
								//force dispay update before mouse movement for the playhead dsplay
								if( display->isInside(g_mouseLoc) )
										display->displayBeatLockedModePlayhead(g_mouseLoc);									
							}
							//Otherwise, we are in beat locked grid mode. Turn that grid off. This is normal mode.
							else
							{
								display->setPlayheadStartTimeGridActive(false);
								display->setDrawPlayheadStartTimeGrid(false);
								display->setDrawBeatLockedModePlayhead(false);
								
								//set the acquire time grid and playhead start time grid number of beats according to ending conditions
								//If we had a backwards selection, make the acquire time grid backwards with the same number of beats
								if( g_spaceBarAudioPlayer->playLength < 0 )
									display->setAcquireTimeGridNumBeats( -display->getPlayheadStartTimeGridNumBeats() );
								//If we had a forwards selection, make the acquire time grid forwards with the same number of beats
								else if( g_spaceBarAudioPlayer->playLength > 0 )
									display->setAcquireTimeGridNumBeats( display->getPlayheadStartTimeGridNumBeats() );
								//If we have a 0 length selection, use whatever the acquire time grid was last time we used it
								else
									display->setPlayheadStartTimeGridNumBeats( display->getAcquireTimeGridNumBeats() );	
									
							}
 						}
 					} 				
 					break;
 				
 				//Delete
 				case 127:
 					if( g_currentView == ACQUIRE )
 					{
 						//If there is an active, selected scrap in the scrap display window
						if( g_audioActiveElement && (g_audioActiveElement == g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ]))
 						{
 							if( g_spaceBarAudioPlayer->isPlaying )
 								g_spaceBarAudioPlayer->isPlaying = false;

							//make acquire waveform the active audio element / player 								
        			g_audioActiveElement = (AudioElement*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
        			g_spaceBarAudioPlayer = g_audioActiveElement->getAudioPlayer();	
	
							//delete that scrap
 							ScrapsWindow* scrapsWindow = (ScrapsWindow*)g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ];
 							scrapsWindow->deleteSelection();
 						}
 							
 					}
 					break;
 					
 					
 				//Delete all scraps	
 				case 'C':
 					if( g_currentView == ACQUIRE )
 					{
 						//If there is an active, selected scrap in the scrap display window
						if( g_audioActiveElement && (g_audioActiveElement == g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ]))
 						{
 							//stop it from playing if it currently is
 							if( g_spaceBarAudioPlayer->isPlaying )
 								g_spaceBarAudioPlayer->isPlaying = false;
 						}
							//delete all scraps
 							ScrapsWindow* scrapsWindow = (ScrapsWindow*)g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ];
 							scrapsWindow->deleteAllScraps();
 					}	
 					break;
 					
 				//Reverse	
 				case 'r':
 					if( g_currentView == ACQUIRE )
 					{

 						//If we are on the acquire view waveform display and audio is not playing
 						//Or if we are currently hovering over the acquire view waveform display and audio is not playing, reverse that instead of whatever else we have selected, temporarily
 						if( g_audioActiveElement && ( (g_audioActiveElement == g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ]) && !g_spaceBarAudioPlayer->isPlaying ) || ( g_audioActiveElement != g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ] && g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ]->isInside(g_mouseLoc)  ))
 						{
 							AcquireViewWaveformDisplay* display = (AcquireViewWaveformDisplay*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
							AudioPlayer* displayAudioPlayer = display->getAudioPlayer();
							
							//If we have something selected, reverse it
							if(displayAudioPlayer->playLength != 0 )
	 							display->reverseSelection();
 							
 							//If we are using the tempo locked time grid, reverse the cursor and update the mouse display
 							if( display->acquireTimeGridIsActive() )
 							{
 								display->setAcquireTimeGridNumBeats( -display->getAcquireTimeGridNumBeats() );
								display->displayAcquireTimeGrid(g_mouseLoc); 									
 							}
 						}
 						
 						
 						//Otherwise if we are on the scraps display window with an active scrap summary and audio is not playing
 						else if( g_audioActiveElement && (g_audioActiveElement == g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ]) && !g_spaceBarAudioPlayer->isPlaying)
 						{
 							ScrapsWindow* scrapsWindow = (ScrapsWindow*)g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ];
 							scrapsWindow->reverseSelection();
 							scrapsWindow->scrollToIncludeActive();
 						}
 							
 					}
 					break;
 					
 					
 				//Fullscreen	
        case 'z':
	        if( !g_fullscreen )
						glutFullScreen();
	        else
						glutReshapeWindow( g_windowWidth, g_windowHeight );
		        g_fullscreen = !g_fullscreen;
        	break;
        	
        	
        //Acquire
        case 'a':
       		if( g_currentView == ACQUIRE )
       		{
       			AudioElement* acquireWaveformDisplay = (AudioElement*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
       			AudioPlayer* acquirePlayer = acquireWaveformDisplay->getAudioPlayer();
       			if( (acquirePlayer->startSample != -1) && (acquirePlayer->playLength != 0))
       			{
       				ScrapsWindow* scrapsWindow = (ScrapsWindow*)g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ];
							AcquireViewWaveformDisplay* display = (AcquireViewWaveformDisplay*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];

	 						//If not currently in playhead start grid mode
							if( !(!display->acquireTimeGridIsActive() && display->playheadStartTimeGridIsActive())  )
							{
								AudioElement* tempAudio = (AudioElement*) g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
								AudioPlayer* tempPlayer = tempAudio->getAudioPlayer();
								
								//Add scrap, defaulted to looping if we acquired it as such, otherwise default to not looping
								if(tempPlayer->isLooping)
									scrapsWindow->addScrap( acquirePlayer->bufferPointer, acquirePlayer->startSample, acquirePlayer->playLength, acquirePlayer->numChannels, true);
								else
	  	     				scrapsWindow->addScrap( acquirePlayer->bufferPointer, acquirePlayer->startSample, acquirePlayer->playLength, acquirePlayer->numChannels);
							}
							//Otherwise, it is in playhead time grid mode. Grab each section of the grid separately
							else 
							{
								long divisionLengthSamples = display->getPlayheadStartTimeGridNumSamplesPerDivision();
								
								for( int i = 0; i < display->getPlayheadStartTimeGridNumBeats(); i++ )
								{
									scrapsWindow->addScrap( acquirePlayer->bufferPointer, acquirePlayer->startSample + i*divisionLengthSamples, divisionLengthSamples, acquirePlayer->numChannels);
								}	
							}
							
							
       				scrapsWindow->scrollToBottom();
       			}
       		}        	
        	break;
        	
        	
        //Save
       	case 's':
       		if( g_currentView == ACQUIRE )
       		{
       			//If we have a scrap summary selected in the scraps window
						if( g_audioActiveElement && g_audioActiveElement == g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ] && g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY]->isInside(g_mouseLoc) )
						{
							ScrapsWindow* scrapsWindow = (ScrapsWindow*)g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ];
							FileBrowseWindow* browseWindow = (FileBrowseWindow*)g_acquireViewUiElements[ FILE_BROWSE_WINDOW_DISPLAY ];
							
							//Save the audio data which is currently active in the scraps window
							browseWindow->saveAudioData(scrapsWindow->getAudioPlayer());
						}
       			
       		}
       		break;
       		
       		
      	//Save all scraps
      	case 'S':
      		if( g_currentView == ACQUIRE )
      		{
      			ScrapsWindow* scrapsWindow = (ScrapsWindow*)g_acquireViewUiElements[ SCRAPS_WINDOW_DISPLAY ];
      			FileBrowseWindow* browseWindow = (FileBrowseWindow*)g_acquireViewUiElements[ FILE_BROWSE_WINDOW_DISPLAY ];
      			browseWindow->saveAllScraps(scrapsWindow);
      		}
      		break;	
        	
        case 9:	//tab
        	if( g_currentView == ACQUIRE )
        	{
        		g_currentView = CREATE;
        		reshapeFunc( g_windowWidth, g_windowHeight );	
        	}
       		else if( g_currentView == CREATE )
       		{
       			g_currentView = ACQUIRE;
        		reshapeFunc( g_windowWidth, g_windowHeight );       			
       		}
        	
        	break;
        	
        //Play
       	case 32: //space key
//       		if( g_currentView == ACQUIRE )
//       		{
       			if( g_spaceBarAudioPlayer )
       			{ 
       				
       				//If currently playing, stop playing
							if( g_spaceBarAudioPlayer->isPlaying )
							{
								g_spaceBarAudioPlayer->isPlaying = false;
								
								if( g_tempAudioElementHolding )
								{
									g_audioActiveElement = g_tempAudioActiveElement;
									g_spaceBarAudioPlayer = g_audioActiveElement->getAudioPlayer();
									g_tempAudioActiveElement = NULL;
									g_tempAudioElementHolding = false;	
								}
							}

       				//If we selected something else but are currently hovering over the acquire waveform, play that instead temporarily
       				else if( g_audioActiveElement != g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ] && g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ]->isInside(g_mouseLoc) && g_currentView == ACQUIRE )
       				{
       					//set the current element to stop playing
       					g_spaceBarAudioPlayer->isPlaying = false;
       					
       					g_tempAudioActiveElement = g_audioActiveElement;
       					g_tempAudioElementHolding = true;
       					
       					//switch to the acquire view waveform display and play from the selection start
       					g_audioActiveElement = (AudioElement*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
       					g_spaceBarAudioPlayer	= g_audioActiveElement->getAudioPlayer();
       					g_spaceBarAudioPlayer->currentIndex = g_spaceBarAudioPlayer->startSample;
       					g_spaceBarAudioPlayer->isPlaying = true;
       				}
       				
       				//Otherwise, if not playing, start playing whatever is selected currently
							else if( !g_spaceBarAudioPlayer->isPlaying )
							{
								g_spaceBarAudioPlayer->isPlaying = true;
								g_spaceBarAudioPlayer->currentIndex = g_spaceBarAudioPlayer->startSample;
							}

       			}
//       		}
       				
	       	break;
	       	
	      //Enter or 'o' 	
	      case 13: 	
       	case 'o':
       		if( g_currentView == ACQUIRE )
       		{
       			//open file or folder if the file browse window is active
						if( g_acquireViewUiElements[ FILE_BROWSE_WINDOW_DISPLAY ]->isActive() )
						{
							//If the acquire view waveform display is playing, stop it before loading
							if( g_spaceBarAudioPlayer && g_spaceBarAudioPlayer->isPlaying && g_audioActiveElement == g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY] )
								g_spaceBarAudioPlayer->isPlaying = false;
								
								
							FileBrowseWindow* browseWindow = (FileBrowseWindow*)g_acquireViewUiElements[ FILE_BROWSE_WINDOW_DISPLAY ];
							browseWindow->openActiveItem();
							
							//If no audio is playing, set the active audio element to be the newly loaded waveform
							if( !g_spaceBarAudioPlayer || (g_spaceBarAudioPlayer && !g_spaceBarAudioPlayer->isPlaying) )
							{
	     					g_audioActiveElement = (AudioElement*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
  	   					g_spaceBarAudioPlayer	= g_audioActiveElement->getAudioPlayer();
							}
						}
						
       		}
       		else if( g_currentView == CREATE )
       		{
       			ProcessesBrowseWindow* processesBrowseWindow = (ProcessesBrowseWindow*) g_createViewUiElements[ CREATE_PROCESSES_WINDOW ];
       			processesBrowseWindow->openActiveItem();
       		}
       		break;
    }
    
    glutPostRedisplay( );
}



//-----------------------------------------------------------------------------
// Name: mouseFunc( )
// Desc: handles mouse stuff
//-----------------------------------------------------------------------------
void mouseFunc( int button, int state, int x, int y )
{
	Point2D mouseLoc(x, g_windowHeight - y);
	
    if( button == GLUT_LEFT_BUTTON )
    {
        // when left mouse button is down
        if( state == GLUT_DOWN )
        {
        		int i = 0;
        		bool foundElement = false;
        		int foundIndex;
        		
        		if( g_currentView == ACQUIRE )
        		{
	        		while( !foundElement && i < g_acquireViewUiElements.size() )
	        		{
								if( g_acquireViewUiElements[i]->isInside(mouseLoc) )
								{
									foundElement = true;
									foundIndex = i;
									g_acquireViewUiElements[i]->mouseDown(mouseLoc);
									
									g_dragActiveElement = g_acquireViewUiElements[i];
									
									if( g_dragActiveElement == g_acquireViewUiElements[ ACQUIRE_CONTROLS_WINDOW_DISPLAY ] )
										g_waveformDragStartY = mouseLoc.y;
									
									if( g_acquireViewUiElements[i]->isAudioElement(mouseLoc) )
									{
										//Set whatever the previous audio player was to be no longer playing
										if( g_audioActiveElement && g_spaceBarAudioPlayer && (g_acquireViewUiElements[i] != g_audioActiveElement) )
											g_spaceBarAudioPlayer->isPlaying = false;

										//If we clicked on the scraps window but not on a scrap waveform, leave the acquire waveform as the active audio element
										//Otherwise, set the active audio element to be the window we clicked on
										if( i == SCRAPS_WINDOW_DISPLAY && !g_acquireViewUiElements[i]->isActive() )
					        			g_audioActiveElement = (AudioElement*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
			        			else
											g_audioActiveElement = (AudioElement*)g_acquireViewUiElements[i];
											
										g_spaceBarAudioPlayer = g_audioActiveElement->getAudioPlayer();
									}
									else if( i == FILE_BROWSE_WINDOW_DISPLAY )
									{
										
									}
									
								}
								i++;
	        		}
	        		
	        		if( foundElement )
	        		{
//		        		for( int j=0; j < g_acquireViewUiElements.size(); j++ )
//		        		{
//		        			if( j != foundIndex )
//		        				g_acquireViewUiElements[j]->setActive(false);
//		        		}
	        		}
	        		else
	        		{
	        			//Set all elements inactive except the acquire waveform view
	        			for( int i=1; i < g_acquireViewUiElements.size(); i++)
	        				g_acquireViewUiElements[i]->setActive(false);
	        				
	        			g_dragActiveElement = NULL;
	        			g_audioActiveElement = (AudioElement*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
	        			g_spaceBarAudioPlayer = g_audioActiveElement->getAudioPlayer();	
	        		}
	        		
        		}
        		else if( g_currentView == CREATE )
        		{
        			
	        		while( !foundElement && i < g_createViewUiElements.size() )
	        		{
								if( g_createViewUiElements[i]->isInside(mouseLoc) )
								{
									foundElement = true;
									foundIndex = i;
									g_createViewUiElements[i]->mouseDown(mouseLoc);
									
									g_dragActiveElement = g_createViewUiElements[i];
									
									if( g_createViewUiElements[i]->isAudioElement(mouseLoc) )
									{
										//Set whatever the previous audio player was to be no longer playing
										if( g_audioActiveElement && g_spaceBarAudioPlayer && (g_createViewUiElements[i] != g_audioActiveElement) )
											g_spaceBarAudioPlayer->isPlaying = false;

										g_audioActiveElement = (AudioElement*)g_createViewUiElements[i];
										g_spaceBarAudioPlayer = g_audioActiveElement->getAudioPlayer();
									}									
								}
								i++;
	        		}
	        		
	        		if( !foundElement )
	        		{
	        			//Set all elements inactive except the acquire waveform view
	        			for( int i=1; i < g_createViewUiElements.size(); i++)
	        				g_createViewUiElements[i]->setActive(false);
	        				
	        			g_dragActiveElement = NULL;
	        			g_audioActiveElement = (AudioElement*)g_createViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
	        			g_spaceBarAudioPlayer = g_audioActiveElement->getAudioPlayer();	
	        		}

        		}

        }
        else
        {
        	if( g_dragActiveElement )
        	{	     			
        		//If we were dragging on the waveform view scroll bar and let go of the mouse outside of the waveform display
        		if( g_dragActiveElement == g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ] && !g_dragActiveElement->isActive() && !g_dragActiveElement->isInside(mouseLoc) )
        		{
        			//Cast a pointer to the display
 							AcquireViewWaveformDisplay* display = (AcquireViewWaveformDisplay*)g_dragActiveElement;
 							//If in tempo locked mode, don't draw the acquire grid mouse follower
 							if( display->acquireTimeGridIsActive() )
 								display->setDrawAcquireTimeGrid(false);	
							//If in beat locked mode, don't draw the playhead mouse follower
							else if( display->playheadStartTimeGridIsActive() )
								display->setDrawBeatLockedModePlayhead(false);								
        		}
        		
        		else if (g_dragActiveElement == g_acquireViewUiElements[ ACQUIRE_CONTROLS_WINDOW_DISPLAY ])
        		{
        			g_windowHeightAdjustOld = g_windowHeightAdjust;
        		}
        		
	        	g_dragActiveElement->mouseUp(mouseLoc);
	        	g_dragActiveElement = NULL;	
        	}
        }
    }
    else if ( button == GLUT_RIGHT_BUTTON )
    {
        // when right mouse button down
        if( state == GLUT_DOWN )
        {
        }
        else
        {
        }
    }
    else
    {
    }
    
    glutPostRedisplay( );
}


//-----------------------------------------------------------------------------
// Name: mouseDrag( )
// Desc: handles mouse dragging
//-----------------------------------------------------------------------------
void mouseDrag( int x, int y ) 
{
	Point2D mouseLoc(x, g_windowHeight - y);

	//Check if NULL first
	if( g_dragActiveElement )
		g_dragActiveElement->mouseMotion(mouseLoc);
		
	if( g_dragActiveElement && g_dragActiveElement == g_acquireViewUiElements[ ACQUIRE_CONTROLS_WINDOW_DISPLAY ] )
	{
		float dragDiff = mouseLoc.y - g_waveformDragStartY;

		if( g_windowHeightAdjustOld + dragDiff > 350 )
			dragDiff = 350 - g_windowHeightAdjustOld;
		if( g_windowHeightAdjustOld + dragDiff < -100 )
			dragDiff = -100 - g_windowHeightAdjustOld;

		g_windowHeightAdjust = g_windowHeightAdjustOld + dragDiff;
		
		reshapeFunc(g_windowWidth, g_windowHeight);
		
	}			
		
  glutPostRedisplay();
}


//-----------------------------------------------------------------------------
// Name: passiveFunc( )
// Desc: handles passive mouse movement
//-----------------------------------------------------------------------------
void passiveFunc( int x, int y ) {

	g_mouseLoc.x = x;
	g_mouseLoc.y = g_windowHeight - y;
	
	Point2D mouseLoc(x, g_windowHeight - y);	

	//display time grid when hovering over acquire view waveform display if we are in tempo lock mode
	if( g_currentView == ACQUIRE )
	{
		AcquireViewWaveformDisplay* display = (AcquireViewWaveformDisplay*)g_acquireViewUiElements[ ACQUIRE_WAVEFORM_DISPLAY ];
		
		//If in acquire time grid mode (tempo locked) and the mouse is over the waveform display, draw that time grid
		if( display->acquireTimeGridIsActive() && display->isInside(mouseLoc))
			display->displayAcquireTimeGrid(mouseLoc);
		else
			display->setDrawAcquireTimeGrid(false);
		
		//If in beat locked time grid mode and the mouse is over the waveform display, draw a playhead (to distinguish from normal mode)
		if( !display->acquireTimeGridIsActive() && display->playheadStartTimeGridIsActive() && display->isInside(mouseLoc))
			display->displayBeatLockedModePlayhead(mouseLoc);
		else
			display->setDrawBeatLockedModePlayhead(false);

	}

	glutPostRedisplay();
}

//-----------------------------------------------------------------------------
// Name: idleFunc( )
// Desc: callback from GLUT
//-----------------------------------------------------------------------------
void idleFunc( )
{
    // render the scene
    glutPostRedisplay( );
}




//-----------------------------------------------------------------------------
// Name: displayFunc( )
// Desc: callback function invoked to draw the client area
//-----------------------------------------------------------------------------
void displayFunc( )
{
	// clear the color and depth buffers
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 
	
	if( g_currentView == ACQUIRE )
	{
		for ( int i = 0; i < g_acquireViewUiElements.size(); i++ )
			g_acquireViewUiElements[i]->draw();
	}
	else if( g_currentView == CREATE )
	{
		for ( int i = 0; i < g_createViewUiElements.size(); i++ )
			g_createViewUiElements[i]->draw();
	}
	
	
	usleep( 1000 );
	
  glFlush( );
  glutSwapBuffers();
}