#include "lineDisplay2D.h"

LineDisplay2D :: LineDisplay2D(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, SAMPLE* dataIn, long dataLengthIn, int minDataIndexDisplayedIn, int maxDataIndexDisplayedIn, SAMPLE maxDataAmplitudeDisplayedIn, bool drawZeroLineIn ) : UiElement(xMinIn, xMaxIn, yMinIn, yMaxIn)
{
	dataLength = dataLengthIn;
	data = dataIn;
	
	minDataIndexDisplayed = minDataIndexDisplayedIn;
	maxDataIndexDisplayed = maxDataIndexDisplayedIn;
	maxDataAmplitudeDisplayed = maxDataAmplitudeDisplayedIn;

	drawZeroLine = drawZeroLineIn;
	
	displayedIndexRangeLength = maxDataIndexDisplayed - minDataIndexDisplayed + 1;
	
	if( displayedIndexRangeLength > 2.0 * (xMax - xMin) )
		displayLength = ceil(2.0 * (xMax - xMin));
	else
		displayLength = displayedIndexRangeLength;
		
	displayXVals = new float[displayLength];
	displayYVals = new float[displayLength];
	
	generateDisplayVals();
}


LineDisplay2D :: ~LineDisplay2D()
{
	delete []displayXVals;
	delete []displayYVals;
}


void LineDisplay2D :: draw()
{
	if( drawZeroLine )
 	{
 		glColor3f( 0.7, 0.7, 0.7 );
 		glLineWidth( 0.7 );
 		
 		glBegin( GL_LINE_STRIP );
 			glVertex3f(xMin, (yMax + yMin) / 2.0, 0.0);
 			glVertex3f(xMax, (yMax + yMin) / 2.0, 0.0); 			
		glEnd();
 	}

	if( isActive() )
	  glColor3f( 1.0, 1.0, 1.0);
  else
		glColor3f( 0.6, 0.6, 0.6);
	  
  glLineWidth( 1.0 );	  	
	
 	glBegin( GL_LINE_STRIP );
	 	for( int i = 0; i < displayLength; i++ )
			glVertex3f(displayXVals[i], displayYVals[i], 0.0);
 	glEnd();

}


void LineDisplay2D :: drawWithOffset(float pixelOffsetIn)
{
	float yMinDisp = yMin + pixelOffsetIn;
	float yMaxDisp = yMax + pixelOffsetIn;
	
	if( drawZeroLine )
 	{
 		glColor3f( 0.7, 0.7, 0.7 );
 		glLineWidth( 0.7 );
 		
 		glBegin( GL_LINE_STRIP );
 			glVertex3f(xMin, (yMaxDisp + yMinDisp) / 2.0, 0.0);
 			glVertex3f(xMax, (yMaxDisp + yMinDisp) / 2.0, 0.0); 			
		glEnd();
 	}

	if( isActive() )
	  glColor3f( 1.0, 1.0, 1.0);
  else
		glColor3f( 0.6, 0.6, 0.6);
	  
  glLineWidth( 1.0 );	  	
	
 	glBegin( GL_LINE_STRIP );
	 	for( int i = 0; i < displayLength; i++ )
			glVertex3f(displayXVals[i], displayYVals[i] + pixelOffsetIn, 0.0);
 	glEnd();

}



void LineDisplay2D :: changeDisplayedIndexRange( int minDataIndexDisplayedIn, int maxDataIndexDisplayedIn )
{
	minDataIndexDisplayed = minDataIndexDisplayedIn;
	maxDataIndexDisplayed = maxDataIndexDisplayedIn;
	
	displayedIndexRangeLength = maxDataIndexDisplayed - minDataIndexDisplayed + 1;
	
	int newDisplayLength;
	
	if( displayedIndexRangeLength > 2.0 * (xMax - xMin) )
		newDisplayLength = ceil(2 * (xMax - xMin));
	else
		newDisplayLength = displayedIndexRangeLength;
		
	if( newDisplayLength != displayLength )
	{
		delete []displayXVals;
		delete []displayYVals;
			
		displayXVals = new float[newDisplayLength];
		displayYVals = new float[newDisplayLength];
	}
	
	displayLength = newDisplayLength;
	
	generateDisplayVals();
}


void LineDisplay2D :: generateDisplayVals()
{	
	if( displayLength == displayedIndexRangeLength )
	{
		for( int i = 0; i < displayLength; i++ )
			displayXVals[i] = xMin + ( (1.0*i) / (displayLength-1.0) ) * (xMax - xMin);				
		
		for( int i = 0; i < displayLength; i++ )
		{
			if( data[i] < -1.0 * maxDataAmplitudeDisplayed )
				displayYVals[i] = yMin;	
			else if( data[i] > maxDataAmplitudeDisplayed )
				displayYVals[i] = yMax;
			else
				displayYVals[i] = yMin + ( (data[minDataIndexDisplayed+i] + maxDataAmplitudeDisplayed) / (2.0 * maxDataAmplitudeDisplayed) ) * (yMax - yMin);
		}
	}
	else
	{
		for( int i = 0; i < displayLength; i=i+2 )
		{
			displayXVals[i] = xMin + ( (1.0*i) / (displayLength-1.0) ) * (xMax - xMin);		
			displayXVals[i+1] = xMin + ( (1.0*i) / (displayLength-1.0) ) * (xMax - xMin);
		}
			
		float numIndicesPerPixel = (1.0 * displayedIndexRangeLength) / displayLength;
//		std::cout << "displayedIndexRangeLength: " << displayedIndexRangeLength << std::endl;
//		std::cout << "Indices per pixel: " << numIndicesPerPixel << std::endl;

		SAMPLE currentMin;
		SAMPLE currentMax;
		
		for( int i = 0; i < displayLength; i++ )
		{
			currentMin = 1.0;
			currentMax = -1.0;
			
			for( int j = floor(i*numIndicesPerPixel) - numIndicesPerPixel/1.5; j < floor((i+1)*numIndicesPerPixel + numIndicesPerPixel/1.5) && j < dataLength; j++ )
			{
				if( j < 0 )
					j = 0;
					
				if( data[minDataIndexDisplayed+j] > currentMax )
					currentMax = data[minDataIndexDisplayed+j];
					
				if( data[minDataIndexDisplayed+j] < currentMin )
					currentMin = data[minDataIndexDisplayed+j];	
			}
			
			if( currentMin < -1.0 * maxDataAmplitudeDisplayed )
				currentMin = -1.0 * maxDataAmplitudeDisplayed;	
				
			if( currentMax > maxDataAmplitudeDisplayed )
				currentMax = 1.0 * maxDataAmplitudeDisplayed;	

			displayYVals[i++] = yMin + ( (currentMin + maxDataAmplitudeDisplayed) / (2.0 * maxDataAmplitudeDisplayed) ) * (yMax - yMin);
			displayYVals[i] = yMin + ( (currentMax + maxDataAmplitudeDisplayed) / (2.0 * maxDataAmplitudeDisplayed) ) * (yMax - yMin);

		}

		
	}
}