#include "ProcessesBrowseWindow.h"

ProcessesBrowseWindow :: ProcessesBrowseWindow(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, float windowWidthIn, float windowHeightIn, ProcessWindow* processWindowIn): UiElement(xMinIn, xMaxIn, yMinIn, yMaxIn)
{
	entryHeight = 15.0;	
	scrollBarWidth = 15.0;
	titleBarHeight = 23.0;
	scrollableMenuWidth = 200.0;
	
	windowWidth = windowWidthIn;
	windowHeight = windowHeightIn;	
	
	itemDisplayPaddingX = 3.0;
	itemDisplayPaddingY = 0.0;	
	
//	vScroll = new BlockItemScrollBar(xMax-scrollBarWidth, xMax, yMin, yMax - 1 - titleBarHeight - 1, entryHeight, 0, 0);	
	vScroll = new BlockItemScrollBar(xMin+scrollableMenuWidth-scrollBarWidth, xMinIn+scrollableMenuWidth, yMin, yMax - 1 - titleBarHeight - 1, entryHeight, 0, 0);	
	
	minDisplayedPixelOffset = 0.0;
  numItemsFitDisplay = ceil((yMax - yMin - titleBarHeight - 3 - 14)/entryHeight) + 1;
  minItemIndexDisplayed = floor(minDisplayedPixelOffset/entryHeight);	
  
  activeItem = -1;	
    
  processWindow = processWindowIn;  
    
  loadProcesses();
}


ProcessesBrowseWindow :: ~ProcessesBrowseWindow()
{
	delete vScroll;	
}


void ProcessesBrowseWindow :: draw()
{
  glColor3f( 1.0, 1.0, 1.0);
  	
  glLineWidth( 1.0 );	  	

 	glBegin( GL_LINE_STRIP );
 		glVertex3f(xMin, yMin, 0.0);
 		glVertex3f(xMin, yMax, 0.0);
 		glVertex3f(xMax, yMax, 0.0);
 		glVertex3f(xMax, yMin, 0.0);
 		glVertex3f(xMin, yMin, 0.0);
	glEnd();
	
	
  // map the view port to the window location
//  glViewport( xMin, yMin + itemDisplayPaddingY, xMax-xMin-scrollBarWidth, yMax-yMin - 2.0*itemDisplayPaddingY );    
  glViewport( xMin, yMin, xMax-xMin-scrollBarWidth, yMax-yMin );    
  // set the matrix mode to project
  glMatrixMode( GL_PROJECTION );
  // load the identity matrix
  glLoadIdentity( );
	//Work in ortho mode
//	gluOrtho2D(xMin, xMax-scrollBarWidth, yMin + itemDisplayPaddingY, yMax - itemDisplayPaddingY );
	gluOrtho2D(xMin, xMax-scrollBarWidth, yMin, yMax);
  // set the matrix mode to modelview
  glMatrixMode( GL_MODELVIEW );
  // load the identity matrix
  glLoadIdentity( );
  // position the view point
  gluLookAt( 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f );


	//Draw list of items
	for( int i=0; i < numItemsFitDisplay && minItemIndexDisplayed + i < itemDisplays.size(); i++ )
		itemDisplays[i + minItemIndexDisplayed]->drawWithOffset(minDisplayedPixelOffset - 2);	

		
	glColor3f( 0.0, 0.0, 0.0);
		
	glRectf(xMin + 1, yMax - 1 - 8 - titleBarHeight - 2, xMax - scrollBarWidth - 1, yMax - titleBarHeight - 2);
	glRectf(xMin + 1, yMin+1, xMax - scrollBarWidth - 1, yMin + 1 + 8);	


	
  // map the view port back to the whole window
  glViewport( 0, 0, windowWidth, windowHeight );
  // set the matrix mode to project
  glMatrixMode( GL_PROJECTION );
  // load the identity matrix
  glLoadIdentity( );
  // set to ortho for 2D version
	gluOrtho2D(0, windowWidth, 0, windowHeight);
  // set the matrix mode to modelview
  glMatrixMode( GL_MODELVIEW );
  // load the identity matrix
  glLoadIdentity( );
  // position the view point
  gluLookAt( 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f );	
  
  
  
  	//Title bar	
	
	glColor3f( 0.4, 0.4, 0.4 );
	glRectf( xMin + 1, yMax - 1 - titleBarHeight - 1, xMax - 1, yMax - 1);	

	glColor3f( 1.0, 1.0, 1.0 );			
	glBegin( GL_LINE_STRIP );
		glVertex3f( xMin, yMax - 1 - titleBarHeight - 1, 0.0);
		glVertex3f( xMax, yMax - 1 - titleBarHeight - 1, 0.0);		
	glEnd();

	glColor3f( 1.0, 1.0, 1.0 );							
	bitmapString( "Processes", xMin + (xMax - xMin - 85.5)/2.0, yMax - titleBarHeight + 4, 99999, 2);
	
	
	//Description
	if( activeItem != -1 )
	{
		bitmapString( "Description:", xMin + scrollableMenuWidth + 10, yMax - titleBarHeight - 20, xMax);
		std::string description = itemDisplays[activeItem]->getDescription();
		
		int i = 0;
		int row = 1;
		
		glRasterPos2i(xMin+scrollableMenuWidth + 30, yMax-titleBarHeight - 35);
		void * font = GLUT_BITMAP_HELVETICA_12;
		GLfloat* position = new GLfloat[4];
		
		for( int i=0; i < description.length(); i++)
		{
			glGetFloatv( GL_CURRENT_RASTER_POSITION, position );
			
			if( position[0] >= xMax - 25 )
				glRasterPos2i(xMin+scrollableMenuWidth + 30, yMax-titleBarHeight - 35 - 15*row++);

				
	    glutBitmapCharacter(font, description[i]);
		}
		delete position;
		
	}
	
	vScroll->draw();
	  
}



void ProcessesBrowseWindow :: setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, float windowWidthIn, float windowHeightIn)
{
	xMin = xMinIn;
	xMax = xMaxIn;
	yMin = yMinIn;
	yMax = yMaxIn;
	
//	vScroll->setPosition(xMaxIn-scrollBarWidth, xMaxIn, yMinIn,  yMaxIn - 1 - titleBarHeight - 1);
	vScroll->setPosition(xMinIn+scrollableMenuWidth-scrollBarWidth, xMinIn+scrollableMenuWidth, yMinIn,  yMaxIn - 1 - titleBarHeight - 1);
	
	for( int i=0; i < itemDisplays.size(); i++ )
	{
		float xMinItemWindow = xMin + itemDisplayPaddingX;
//		float xMaxItemWindow = xMax - scrollBarWidth - itemDisplayPaddingX;
		float xMaxItemWindow = xMin + scrollableMenuWidth - scrollBarWidth - itemDisplayPaddingX;
		float yMinItemWindow = yMax - titleBarHeight - ((i + 1) * entryHeight - itemDisplayPaddingY/2) - 8;
		float yMaxItemWindow = yMax - titleBarHeight - itemDisplayPaddingY - (i * entryHeight + itemDisplayPaddingY/2) - 8;
		
		itemDisplays[i]->setPosition(xMinItemWindow, xMaxItemWindow, yMinItemWindow, yMaxItemWindow);
	}
	
	windowWidth = windowWidthIn;
	windowHeight = windowHeightIn;
  numItemsFitDisplay = ceil((yMax - yMin - titleBarHeight - 3 - 16)/entryHeight) + 1;
}


void ProcessesBrowseWindow :: loadProcesses()
{

	//POPULATE PROCESSES HERE
	std::vector<std::string> processNames;
	std::vector<std::string> processDescriptions;
	
	processNames.push_back( "Transfer Amplitude Envelope" );
	processNames.push_back( "Loose Timing" );
	processNames.push_back( "Trim Transient" );	
	processNames.push_back( "Long Grain Synthesis" );
	processNames.push_back( "Interleaved Sources" );
	processNames.push_back( "Swarm" );
	processNames.push_back( "Complimentary Amplitude" );
	processNames.push_back( "Splice and Shuffle" );
	processNames.push_back( "Amplitude Trigger" );
	processNames.push_back( "Transfer Tail" );
	processNames.push_back( "Concatenate" );
	processNames.push_back( "U-Turn" );
	processNames.push_back( "Time Stretch" );
	processNames.push_back( "Pitch Shift" );

	
	processDescriptions.push_back( "This process alters the amplitude envelope of one sound to match that of another." );
	processDescriptions.push_back( "This process randomly aggregates a collection of transient sounds into an adjustably loose group, ranging from a very tight, precise percussive hit to a sloppy, drawn out, clap-like grouping." );	
	processDescriptions.push_back( "This process identifies the attack of a transient sound and outputs a trimmed sample with the pre-attack section removed." );
	processDescriptions.push_back( "This process applies a window to an input sound and repeats it a specified number of times at a specified overlap to create sustained sounds." );
	processDescriptions.push_back( "This process accepts some number of sources and then randomly interleaves their content to generate a sound which is as long as the longest input sound that transitions between the timbres of the input sounds at a specified rate." );
	processDescriptions.push_back( "This process takes a single or multiple sounds, generates a specified number of instances of each and triggers them randomly with a specified density over some window of time." );
	processDescriptions.push_back( "This process alters the amplitude envelope of one sound such that it always adds to a constant when combined with another specified sound. Alternatively, this process can be applied to groups of sounds such that their amplitude envelopes collectively sum to a constant. Each sound's envelope is kept at its natural level in one portion of the sound's duration." );
	processDescriptions.push_back( "This process slipts the input sound or sounds into a specified number of segments and randomly shuffles them into a new order to form the output sound." );	
	processDescriptions.push_back( "This process uses the amplitude envelope of one sound to control the density at which another sample or group of samples is triggered." );
	processDescriptions.push_back( "This process identifies the tail portion of two transient sounds and exchanges them." );
	processDescriptions.push_back( "Simple end-to-start concatenation." );
	processDescriptions.push_back( "Outputs the sound played forward and then backward, trimming to an attack on either end if desired." );
	processDescriptions.push_back( "Time stretch a sound." );
	processDescriptions.push_back( "Pitch shift a sound." );	
				
	
 	float xMinItemDisplay;
	float xMaxItemDisplay;
	float yMinItemDisplay;
	float yMaxItemDisplay;
 
 	//Add these items to the display list, starting with the directories 
  for( int i=0; i < processNames.size(); i++ )
  {
 		xMinItemDisplay = xMin + itemDisplayPaddingX;
//		xMaxItemDisplay = xMax - scrollBarWidth - itemDisplayPaddingX;
		xMaxItemDisplay = xMin + scrollableMenuWidth - scrollBarWidth - itemDisplayPaddingX;		
		yMinItemDisplay = yMax - titleBarHeight - ((itemDisplays.size() + 1) * entryHeight - itemDisplayPaddingY/2) - 8;
		yMaxItemDisplay = yMax - titleBarHeight - itemDisplayPaddingY - (itemDisplays.size() * entryHeight + itemDisplayPaddingY/2) - 8;
	
  	itemDisplays.push_back( new ProcessItemDisplay( xMinItemDisplay, xMaxItemDisplay, yMinItemDisplay, yMaxItemDisplay, processNames[i], processDescriptions[i] ) );	
  }
  
 	vScroll->setNumItems( itemDisplays.size() + 1 );
  
}



void ProcessesBrowseWindow :: mouseDown(Point2D mouseLoc)
{
	//If the click was inside the scroller
	if( vScroll->isInside(mouseLoc) )
	{
		//Do the scrollBar's mouseDown. This sets the scrollBar active and sets the initial mouse position.
		vScroll->mouseDown(mouseLoc);
		
		//If we did not click inside the active rect, update the scrollbar position based on the click
		if( !vScroll->isInsideActiveRect(mouseLoc) )
			vScroll->clickOutsideActiveRect(mouseLoc);
			
		minDisplayedPixelOffset = vScroll->getMinDisplayedPixelOffset();	
	  minItemIndexDisplayed = floor(minDisplayedPixelOffset/entryHeight);				
	}
	
	//Otherwise the click was inside the items menu area
	else
	{

		//if an item was previously active, make it unactive
		if(activeItem != -1)
			itemDisplays[activeItem]->setActive(false);
	
		active = false;
		
		activeItem = -1;
		int i=0;
		
		//See if the click was on any of the items
		while( i < itemDisplays.size() && activeItem == -1 )
		{
			Point2D offsetMouseLoc = mouseLoc;
			offsetMouseLoc.y = offsetMouseLoc.y - minDisplayedPixelOffset;
			
			if( itemDisplays[i]->isInside(offsetMouseLoc) )
			{
				itemDisplays[i]->mouseDown(offsetMouseLoc);	
				activeItem = i;
				active = true;
			}
			
			i++;
		}

	}
	
}


void ProcessesBrowseWindow :: mouseUp(Point2D mouseLoc)
{
	vScroll->mouseUp(mouseLoc);
//	active = false;
}


void ProcessesBrowseWindow :: mouseMotion(Point2D mouseLoc)
{
	if( vScroll->isActive() )
	{
		vScroll->mouseMotion(mouseLoc);
		minDisplayedPixelOffset = vScroll->getMinDisplayedPixelOffset();
	  minItemIndexDisplayed = floor(minDisplayedPixelOffset/entryHeight);		
	}
	
}



void ProcessesBrowseWindow :: changeSelectedItem(bool up)
{
	if( up && activeItem > 0 )
	{
		itemDisplays[activeItem]->setActive(false);
		activeItem = activeItem - 1;
		itemDisplays[activeItem]->setActive(true);
		
		//If that item isn't fully displayed, make it so that it is
		if( activeItem == 0 )
		{
			minItemIndexDisplayed = 0;
			minDisplayedPixelOffset = 0.0;
		}
		else if( activeItem <= minItemIndexDisplayed )	//hanging off top
		{
			minItemIndexDisplayed = activeItem - 1;		
			minDisplayedPixelOffset = (minItemIndexDisplayed + 1) * entryHeight - 0.5*entryHeight;
		}
		//If that item is below the currently viewed items, jump down so that it shows at the bottom		
		else if( activeItem >= minItemIndexDisplayed + numItemsFitDisplay )		// - 1
		{
			minItemIndexDisplayed = activeItem + 1 - (numItemsFitDisplay - 1);
			minDisplayedPixelOffset = (minItemIndexDisplayed) * entryHeight + 0.3 * entryHeight + itemDisplayPaddingY + titleBarHeight + 2;
		}

		
		//update scrollbar to match minDisplayedPixelOffset
		vScroll->setMinDisplayedPixelOffset(minDisplayedPixelOffset);
		
	}
	
	
	else if( !up && activeItem < itemDisplays.size() - 1 )
	{
		itemDisplays[activeItem]->setActive(false);
		activeItem = activeItem + 1;	
		itemDisplays[activeItem]->setActive(true);				
		
		//If that item isn't fully displayed, make it so that it is
		if( activeItem == itemDisplays.size() - 1 && itemDisplays.size() * entryHeight > (yMax-2-titleBarHeight-2-16)-(yMin+1))
		{
			minItemIndexDisplayed = activeItem + 1 - (numItemsFitDisplay - 1);				
			minDisplayedPixelOffset = itemDisplays.size() * entryHeight - ((yMax-2-titleBarHeight-2)-(yMin+1)) - 3 + 18;
		}
		else if( activeItem >= minItemIndexDisplayed + numItemsFitDisplay - 2 && itemDisplays.size() * entryHeight > (yMax-2-titleBarHeight-2-16)-(yMin+1) )	//hanging off bottom
		{
			minItemIndexDisplayed = activeItem + 1 - (numItemsFitDisplay - 1) + 1;	
			minDisplayedPixelOffset = (minItemIndexDisplayed) * entryHeight + 0.3 * entryHeight + itemDisplayPaddingY + titleBarHeight + 2 - 30;
		}
		else if( activeItem <= minItemIndexDisplayed )
		{
			minItemIndexDisplayed = activeItem - 1;
			minDisplayedPixelOffset = (minItemIndexDisplayed + 1) * entryHeight - 0.5*entryHeight;				
		}
		
		
		//update scrollbar to match minDisplayedPixelOffset		
		vScroll->setMinDisplayedPixelOffset(minDisplayedPixelOffset);		
		
	}
	
}



void ProcessesBrowseWindow :: scrollToBottom()
{
	if( itemDisplays.size() * entryHeight > (yMax-2-titleBarHeight-2)-(yMin+1) )
	{
		minItemIndexDisplayed = itemDisplays.size() - (numItemsFitDisplay - 1);
			
		minDisplayedPixelOffset = itemDisplays.size() * entryHeight - ((yMax-2-titleBarHeight-2)-(yMin+1)) - 3;
		vScroll->setMinDisplayedPixelOffset(minDisplayedPixelOffset);		
	}
}



void ProcessesBrowseWindow :: scrollToIncludeActive()
{
	
	//If it is the first item
	if( activeItem == 0 )
	{
		minItemIndexDisplayed = 0;
		minDisplayedPixelOffset = 0.0;
	}
	//If it is the last item and the last item extends beyond the window
	else if( activeItem == itemDisplays.size() - 1 && itemDisplays.size() * entryHeight > (yMax-2-titleBarHeight-2)-(yMin+1))
	{
		minItemIndexDisplayed = activeItem + 1 - (numItemsFitDisplay - 1);				
		minDisplayedPixelOffset = itemDisplays.size() * entryHeight - ((yMax-2-titleBarHeight-2)-(yMin+1)) - 3 + 16;
	}
	//If that item is above the currently viewed items, jump up so that it shows at the top
	else if( activeItem <= minItemIndexDisplayed )
	{
		minItemIndexDisplayed = activeItem - 1;		
		minDisplayedPixelOffset = (minItemIndexDisplayed + 1) * entryHeight - 0.5*entryHeight;
	}
	//If that item is below the currently viewed items, jump down so that it shows at the bottom		
	else if( activeItem >= minItemIndexDisplayed + numItemsFitDisplay - 1 )
	{
		minItemIndexDisplayed = activeItem + 1 - (numItemsFitDisplay - 1);
		minDisplayedPixelOffset = (minItemIndexDisplayed) * entryHeight + 0.3 * entryHeight + itemDisplayPaddingY + titleBarHeight + 2 - 11;
	}
	
	//update scrollbar to match minDisplayedPixelOffset		
	vScroll->setMinDisplayedPixelOffset(minDisplayedPixelOffset);		
}


void ProcessesBrowseWindow :: openActiveItem()
{
	
	//LOAD PROCESS HERE
	if( itemDisplays[activeItem]->getName().compare("Loose Timing") == 0 )
		processWindow->loadProcess();	//loadProcess(activeItem);
	
}