#include "scrapDetailWindow.h"

ScrapDetailWindow :: ScrapDetailWindow(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, SAMPLE** dataIn, int numChannelsIn, long dataLengthIn, TimeGridUtil* globalTimeGridUtilIn, bool scrollOnBottomIn, int numChannelsDisplayedIn): AudioElement(xMinIn, xMaxIn, yMinIn, yMaxIn)
{
	lockedForDataUpdate = true;
	titleBarHeight = 23.0;
	
	numChannels = numChannelsIn;
	dummyNumChannels = numChannelsIn;
	data =	new SAMPLE*[numChannels];
	dummyData = new SAMPLE*[numChannels];
	
	for( int i=0; i < numChannels; i++)
	{
		data[i] = dataIn[i];
		dummyData[i] = dataIn[i];	
	}
	
	dataLength = dataLengthIn;
	dummyDataLength = dataLength;

/* Not currently implemented	
	if( numChannels > 1 )
	{
		monoData = new SAMPLE[dataLength];	
		for( int i=0; i < dataLength; i++ )
			monoData[i] = data[0][i] + data[1][i];	//Do a max abs here to get envelope? Normalize?
	}
*/

	
	numIndicesPerSummarySample = 1.0;
	usingSummary = false;
	summaryData = new SAMPLE*[numChannels];
	summaryLength = 125000;
	//For good display performance, we don't want to iterate through more than summaryLength points
	if( dataLength > summaryLength )
	{
		usingSummary = true;
		for( int channel = 0; channel < numChannels; channel++)
		{
			summaryData[channel] = new SAMPLE[summaryLength];
			
			numIndicesPerSummarySample = (1.0 * dataLength) / summaryLength;
			
			SAMPLE currentMin;
			SAMPLE currentMax;
			
			for( int i = 0; i < summaryLength; i++ )
			{
				currentMin = 1.0;
				currentMax = -1.0;
				
				for( int j = floor(i*numIndicesPerSummarySample); j < floor((i+1)*numIndicesPerSummarySample); j++ )
				{
					if( data[channel][j] > currentMax )
						currentMax = data[channel][j];
						
					if( data[channel][j] < currentMin )
						currentMin = data[channel][j];	
				}
	
				summaryData[channel][i++] = currentMin;
				summaryData[channel][i] = currentMax;
			}
		}

	}

	
	//Display all channels if no number was specified	
	if( numChannelsDisplayedIn == 0 )
		numChannelsDisplayed = numChannelsIn;
	else
		numChannelsDisplayed = numChannelsDisplayedIn;
		
	scrollBarWidth = 15.0;
	waveformDisplayPadding = 5.0;
	
	//Default to display the entire waveform, start to finish, -1.0 to 1.0 amplitude, with a y = 0 line
	//Display size is smaller to leave space for scroll bars
	float xMinDisp = xMin + waveformDisplayPadding;
	float xMaxDisp = xMax - waveformDisplayPadding;
	float yMinDisp;
	float yMaxDisp;

	
	//Initialize displays and scroller in the correct positions
	if( scrollOnBottomIn )
	{
		yMinDisp = yMin + scrollBarWidth + waveformDisplayPadding;
		yMaxDisp = yMax - waveformDisplayPadding - titleBarHeight;
		if(usingSummary)
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMin, yMin+scrollBarWidth, true, summaryLength, 0, summaryLength-1, numIndicesPerSummarySample);
		else
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMin, yMin+scrollBarWidth, true, dataLength, 0, dataLength-1, numIndicesPerSummarySample);	
	}
	else
	{
		yMinDisp = yMin + waveformDisplayPadding;
		yMaxDisp = yMax - scrollBarWidth - waveformDisplayPadding - titleBarHeight;
		if(usingSummary)		
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMax-scrollBarWidth, yMax, true, summaryLength, 0, summaryLength-1, numIndicesPerSummarySample);
		else
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMax-scrollBarWidth, yMax, true, dataLength, 0, dataLength-1, numIndicesPerSummarySample);
	}
	
	//Initialize waveform displays
	waveformDisplays = new LineDisplay2D*[numChannels];
	if(usingSummary)
	{
		for( int i=0; i < numChannels; i++)
		{
			waveformDisplays[i] = new LineDisplay2D( xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)), summaryData[i], summaryLength, 0, summaryLength-1, 1.0, true );
			waveformDisplays[i]->setActive(true);
		}
	}
	else
	{
		for( int i=0; i < numChannels; i++)
		{
			waveformDisplays[i] = new LineDisplay2D( xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)), data[i], dataLength, 0, dataLength-1, 1.0, true );
			waveformDisplays[i]->setActive(true);			
		}
	}


	//Initialize the playheads
//	playheadStart = new Playhead(-1.0, -1.0, yMinDisp, yMaxDisp, xMin + 0.5*waveformDisplayPadding, xMaxDisp);
//	playheadStart->setPlayRegionOn(true);
	playheadMoving = new Playhead(-1.0, -1.0, yMinDisp, yMaxDisp, xMin + 0.5*waveformDisplayPadding, xMaxDisp);
//	mouseFollowPlayheadForBeatLockedTimeGridMode = new Playhead(-1.0, -1.0, yMinDisp, yMaxDisp, xMin + 0.5*waveformDisplayPadding, xMaxDisp);
//	drawBeatLockedModePlayhead = false;
	
	//Initialize the audio player
	audioPlayer = NULL;
	
	scrollOnBottom = scrollOnBottomIn;
/*	
	//Initialize playhead time grid
	playheadStartTimeGrid = new AcquireTimeGrid(-1.0, -1.0, yMinDisp, yMaxDisp, xMin + 0.5*waveformDisplayPadding, xMaxDisp, timeGridNumBeatsIn, timeGridBeatDivisionIn, globalTimeGridUtilIn, false);
	drawPlayheadStartTimeGrid = false;
	
	//Initialize mouse time grid
	acquireTimeGrid = new AcquireTimeGrid(-1.0, -1.0, yMinDisp, yMaxDisp, xMin + 0.5*waveformDisplayPadding, xMaxDisp, timeGridNumBeatsIn, timeGridBeatDivisionIn, globalTimeGridUtilIn, acquireTimeGridActiveIn);
	drawAcquireTimeGrid = false;
*/	
	//Ready to draw
	lockedForDataUpdate = false;
	
}


ScrapDetailWindow :: ~ScrapDetailWindow()
{
	delete []data;
	
	for( int i=0; i < numChannels; i++ )
	{
		delete waveformDisplays[i];
		if( summaryData[i] )
			delete summaryData[i];	
	}
	
	delete []waveformDisplays;
	delete []summaryData;
	delete hScroll;
//	delete acquireTimeGrid;
//	delete playheadStartTimeGrid;
//	delete mouseFollowPlayheadForBeatLockedTimeGridMode;
//	delete playheadStart;
	delete playheadMoving;
}




void ScrapDetailWindow :: updateData( SAMPLE** dataIn, int numChannelsIn, long dataLengthIn, int scrapNumberIn, AudioPlayer* audioPlayerIn, int numChannelsDisplayedIn )
{
	lockedForDataUpdate = true;

	//Clear stuff allocated for old data.	
	delete []data;
	delete hScroll;
	
	for( int i=0; i < numChannels; i++ )
	{
		delete waveformDisplays[i];
		if( dataLength > summaryLength )
		{
			delete summaryData[i];	
		}
	}
	
	delete []waveformDisplays;	
	delete []summaryData;	
//	delete audioPlayer;
	
	//Load new data
	numChannels = numChannelsIn;
	data =	new SAMPLE*[numChannels];
	
	for( int i=0; i < numChannels; i++)
		data[i] = dataIn[i];
	
	dataLength = dataLengthIn;
	
	numIndicesPerSummarySample = 1.0;
	usingSummary = false;
	summaryData = new SAMPLE*[numChannels];
	summaryLength = 250000;
	//For good display performance, we don't want to iterate through more than summaryLength points
	if( dataLength > summaryLength )
	{
		usingSummary = true;
		for( int channel = 0; channel < numChannels; channel++)
		{
			summaryData[channel] = new SAMPLE[summaryLength];
			
			numIndicesPerSummarySample = (1.0 * dataLength) / summaryLength;
			
			SAMPLE currentMin;
			SAMPLE currentMax;
			
			for( int i = 0; i < summaryLength; i++ )
			{
				currentMin = 1.0;
				currentMax = -1.0;
				
				for( int j = floor(i*numIndicesPerSummarySample); j < floor((i+1)*numIndicesPerSummarySample); j++ )
				{
					if( data[channel][j] > currentMax )
						currentMax = data[channel][j];
						
					if( data[channel][j] < currentMin )
						currentMin = data[channel][j];	
				}
	
				summaryData[channel][i++] = currentMin;
				summaryData[channel][i] = currentMax;
			}
		}

	}
	
	//Display all channels if no number was specified	
	if( numChannelsDisplayedIn == 0 )
		numChannelsDisplayed = numChannelsIn;
	else
		numChannelsDisplayed = numChannelsDisplayedIn;

	//Default to display the entire waveform, start to finish, -1.0 to 1.0 amplitude, with a y = 0 line
	//Display size is smaller to leave space for scroll bars
	float xMinDisp = xMin + waveformDisplayPadding;
	float xMaxDisp = xMax - waveformDisplayPadding;
	float yMinDisp;
	float yMaxDisp;
	
	//Initialize displays and scroller in the correct positions
	if( scrollOnBottom )
	{
		yMinDisp = yMin + scrollBarWidth + waveformDisplayPadding;
		yMaxDisp = yMax - waveformDisplayPadding - titleBarHeight;
		if(usingSummary)
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMin, yMin+scrollBarWidth, true, summaryLength, 0, summaryLength-1, numIndicesPerSummarySample);
		else
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMin, yMin+scrollBarWidth, true, dataLength, 0, dataLength-1, numIndicesPerSummarySample);	
	}
	else
	{
		yMinDisp = yMin + waveformDisplayPadding;
		yMaxDisp = yMax - scrollBarWidth - waveformDisplayPadding - titleBarHeight;
		if(usingSummary)		
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMax-scrollBarWidth, yMax, true, summaryLength, 0, summaryLength-1, numIndicesPerSummarySample);
		else
			hScroll = new AcquireViewScrollBar(xMin, xMax, yMax-scrollBarWidth, yMax, true, dataLength, 0, dataLength-1, numIndicesPerSummarySample);
	}
	
	//Initialize waveform displays
	waveformDisplays = new LineDisplay2D*[numChannels];
	if(usingSummary)
	{
		for( int i=0; i < numChannels; i++)
		{
			waveformDisplays[i] = new LineDisplay2D( xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)), summaryData[i], summaryLength, 0, summaryLength-1, 1.0, true );
			waveformDisplays[i]->setActive(true);
		}
	}
	else
	{
		for( int i=0; i < numChannels; i++)
		{
			waveformDisplays[i] = new LineDisplay2D( xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)), data[i], dataLength, 0, dataLength-1, 1.0, true );
			waveformDisplays[i]->setActive(true);			
		}
	}

	
	//Initialize the audio player
//	audioPlayer = new AudioPlayer(data, dataLength, numChannels, 0, 0, false, false);
	audioPlayer = audioPlayerIn;
	
//	playheadStart->setStartSample( -1 );
//	playheadStart->setPlayLength( 0 );
//	playheadStart->setXMin( -1 );
//	playheadStart->setXMax( -1 );
//	playheadStartTimeGrid->setXMin( -1 );
//	playheadStartTimeGrid->setXMax( -1 );
	


	scrapNumber = scrapNumberIn;
	
	lockedForDataUpdate = false;
}



void ScrapDetailWindow :: setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn)
{
	xMin = xMinIn;
	xMax = xMaxIn;
	yMin = yMinIn;
	yMax = yMaxIn;
	
	float xMinDisp = xMin + waveformDisplayPadding;
	float xMaxDisp = xMax - waveformDisplayPadding;
	float yMinDisp;
	float yMaxDisp;
	
	if( scrollOnBottom )
	{
		yMinDisp = yMin + scrollBarWidth + waveformDisplayPadding;
		yMaxDisp = yMax - waveformDisplayPadding - titleBarHeight;
		hScroll->setPosition(xMin, xMax, yMin, yMin+scrollBarWidth);
	}
	else
	{
		yMinDisp = yMin + waveformDisplayPadding;
		yMaxDisp = yMax - scrollBarWidth - waveformDisplayPadding - titleBarHeight;		
		hScroll->setPosition(xMin, xMax, yMax-scrollBarWidth, yMax);
	}
	
	for( int i=0; i < numChannels; i++ )
	{
		waveformDisplays[i]->resize(xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)));
		waveformDisplays[i]->generateDisplayVals();
	}

/*		
	playheadStart->setYPositions(yMinDisp, yMaxDisp);
	playheadStart->setXMinMaxDisp(xMinDisp, xMaxDisp);	
	
	playheadStartTimeGrid->setYPositions(yMinDisp, yMaxDisp);
	playheadStartTimeGrid->setXMinMaxDisp(xMinDisp, xMaxDisp);	
*/	
	playheadMoving->setYPositions(yMinDisp, yMaxDisp);
	playheadMoving->setXMinMaxDisp(xMinDisp, xMaxDisp);	
/*	
	mouseFollowPlayheadForBeatLockedTimeGridMode->setYPositions(yMinDisp, yMaxDisp);
	mouseFollowPlayheadForBeatLockedTimeGridMode->setXMinMaxDisp(xMinDisp, xMaxDisp);	

	acquireTimeGrid->setYPositions(yMinDisp, yMaxDisp);
	acquireTimeGrid->setXMinMaxDisp(xMinDisp, xMaxDisp);	
*/	
}



void ScrapDetailWindow :: mouseDown(Point2D mouseLoc)
{
	
	//If we clicked on the scroller -------------------------------
	if( hScroll->isInside(mouseLoc) )
	{
		bool originallyInsideActiveRect = true;
		if( !hScroll->isInsideActiveRect(mouseLoc) )
			originallyInsideActiveRect = false;

		//Do the scrollBar's mouseDown. This sets the scrollBar active.
		hScroll->mouseDown(mouseLoc);
		
		//If not inside the active rect -----------------------------
		//  Do immediate update (scroll)
		if( !originallyInsideActiveRect )
		{
/*			
			//Solve for playheadStart position in terms of display
			float playheadStartX;
			playheadStartX = dataIndexValueToXPos(playheadStart->getStartSample());		
			playheadStart->setXMin(playheadStartX);
			playheadStartTimeGrid->setXMin(playheadStartX);
			
			//Solve for play region end in terms of display, if one exists
			if( playheadStart->getPlayLength() != 0 )
			{
				float playheadEndX;
				playheadEndX = dataIndexValueToXPos(playheadStart->getStartSample() + playheadStart->getPlayLength());		
				playheadStart->setXMax(playheadEndX);
				playheadStartTimeGrid->setXMax(playheadEndX);				
			}
*/						
			//Don't need to check if useSummary should change since we cannot have any zoom in or out yet.
			//  Just adjust the waveform displays to scroll
			for( int channel = 0; channel < numChannels; channel++ )
				waveformDisplays[channel]->changeDisplayedIndexRange( hScroll->getMinIndexActive(), hScroll->getMaxIndexActive() );
		}
	}
}



void ScrapDetailWindow :: mouseUp(Point2D mouseLoc)
{
	hScroll->mouseUp(mouseLoc);

	active = false;
}	




void ScrapDetailWindow :: mouseMotion(Point2D mouseLoc)
{
	
	//If we clicked on the scroll bar originally
	if( hScroll->isActive() )
	{	
		
		//See if we need to switch into or out of summary mode due to zooming in / out
		if( usingSummary && ( (hScroll->getMaxIndexActive() - hScroll->getMinIndexActive()) * numIndicesPerSummarySample < summaryLength) )
		{
			for( int channel = 0; channel < numChannels; channel++ )
				waveformDisplays[channel]->setData( data[channel], dataLength );
				
			hScroll->changeIndexScale( dataLength );
			usingSummary = false;
		}
		else if( !usingSummary && ( hScroll->getMaxIndexActive() - hScroll->getMinIndexActive() > summaryLength) )
		{
			for( int channel = 0; channel < numChannels; channel++ )
				waveformDisplays[channel]->setData( summaryData[channel], summaryLength );									
				
			hScroll->changeIndexScale( summaryLength );
			usingSummary = true;					
		}
	
		//Call the scrollbar's mouseMotion fn. This handles updating the zoom, active scrollbar rectangle size, and active / displayed index region.
		hScroll->mouseMotion(mouseLoc);
/*		
		
		//Solve for playheadStart position in terms of display, if we have a playhead (program begins without one)
		if( playheadStart->getStartSample() != -1 )
		{
			float playheadStartX;
			playheadStartX = dataIndexValueToXPos(playheadStart->getStartSample());		
			playheadStart->setXMin(playheadStartX);
			playheadStartTimeGrid->setXMin(playheadStartX);			
		}
		
		//Solve for play region end in terms of display, if one exists
		if( playheadStart->getPlayLength() != 0 )
		{
			float playheadEndX;
			playheadEndX = dataIndexValueToXPos(playheadStart->getStartSample() + playheadStart->getPlayLength());		
			playheadStart->setXMax(playheadEndX);
			playheadStartTimeGrid->setXMax(playheadEndX);
*/			
/*	//If in playhead start time grid mode, update the time grid division length to match where we dragged to
	if( drawPlayheadStartTimeGrid && active && !drawAcquireTimeGrid )
	{
		long selectedRegionLengthSamples = xPosToDataIndexValue(mouseLoc.x) - playheadStart->getStartSample();
		playheadStartTimeGrid->setNumSamplesTotal( selectedRegionLengthSamples );
		playheadStartTimeGrid->setNumSamplesPerDivision( selectedRegionLengthSamples / playheadStartTimeGrid->getNumBeats() );
	}			
	
	*/
		//}
/*		
		//if we are displaying a time grid region, update those values as well so that the timegrid region follows the mouse, even when scrolling.
		if( drawAcquireTimeGrid )
		{
			long hoverIndexStart = xPosToDataIndexValue(mouseLoc.x);
			float endPosX = dataIndexValueToXPos(hoverIndexStart + acquireTimeGrid->getNumSamplesTotal() );
			
			float startPosX;
			if( mouseLoc.x < xMin + waveformDisplayPadding )
				startPosX = xMin + waveformDisplayPadding;
			else if( mouseLoc.x > xMax - waveformDisplayPadding )
				startPosX = xMax - waveformDisplayPadding;
			else
				startPosX = mouseLoc.x;
			
			acquireTimeGrid->setXMin(startPosX);
			acquireTimeGrid->setXMax(endPosX);			
		}
*/
		
		//Update waveform displays with the new displayed index range		
		for( int channel = 0; channel < numChannelsDisplayed; channel++ )
			waveformDisplays[channel]->changeDisplayedIndexRange( hScroll->getMinIndexActive(), hScroll->getMaxIndexActive() );
	}
}




void ScrapDetailWindow :: draw()
{
  glColor3f( 1.0, 1.0, 1.0);
  glLineWidth( 1.0 );	  	

 	glBegin( GL_LINE_STRIP );
 		glVertex3f(xMin, yMin, 0.0);
 		glVertex3f(xMin, yMax, 0.0);
 		glVertex3f(xMax, yMax, 0.0);
 		glVertex3f(xMax, yMin, 0.0);
 		glVertex3f(xMin, yMin, 0.0);
	glEnd();
	
	
 	//Title bar	
	
	glColor3f( 0.4, 0.4, 0.4 );
	glRectf( xMin + 1, yMax - 1 - titleBarHeight - 1, xMax - 1, yMax - 1);	

	glColor3f( 1.0, 1.0, 1.0 );			
	glBegin( GL_LINE_STRIP );
		glVertex3f( xMin, yMax - 1 - titleBarHeight - 1, 0.0);
		glVertex3f( xMax, yMax - 1 - titleBarHeight - 1, 0.0);		
	glEnd();

	glColor3f( 1.0, 1.0, 1.0 );							
	bitmapString( "Scrap Detail", xMin + (xMax - xMin - 85.5)/2.0, yMax - titleBarHeight + 4, 99999, 2);
	
	
	if( !lockedForDataUpdate )
	{
		
		
		for( int i = 0; i < numChannelsDisplayed; i++ )
			waveformDisplays[i]->draw();
		
		hScroll->draw();
/*	
		//Only draw if we have a playhead down
		if( drawPlayheadStartTimeGrid && playheadStart->getStartSample() != -1 && playheadStart->getPlayLength() != 0 )
			playheadStartTimeGrid->draw();
		else
			playheadStart->draw();	
*/		
		if( audioPlayer && audioPlayer->isPlaying )
		{		
			playheadMoving->setStartSample( audioPlayer->currentIndex );
			
			//Solve for playheadMoving position in terms of display
			float playheadMovingX;
			playheadMovingX = dataIndexValueToXPos(playheadMoving->getStartSample());	
			playheadMoving->setXMin(playheadMovingX);
			
			playheadMoving->draw();		
		}
/*		
		else if( drawAcquireTimeGrid )
		{
			acquireTimeGrid->draw();
		}
		else if( drawBeatLockedModePlayhead )
		{
			mouseFollowPlayheadForBeatLockedTimeGridMode->draw();	
		}
		
*/		
	}
	
}




float ScrapDetailWindow :: dataIndexValueToXPos( long indexVal )
{
	float xPosition;
	
	if( usingSummary )
	{
		xPosition = xMin + waveformDisplayPadding + ( ((indexVal / numIndicesPerSummarySample) - hScroll->getMinIndexActive()) / (1.0 * (hScroll->getMaxIndexActive() - hScroll->getMinIndexActive())) ) * (1.0 * (xMax - xMin - waveformDisplayPadding*2));
	}
	else
	{
		xPosition = xMin + waveformDisplayPadding + ( (1.0 * (indexVal - hScroll->getMinIndexActive())) / (1.0 * (hScroll->getMaxIndexActive() - hScroll->getMinIndexActive())) ) * (xMax - xMin - waveformDisplayPadding*2);
	}
	
	return xPosition;
	
}



void ScrapDetailWindow :: clearData()
{
	updateData( dummyData, dummyNumChannels, dummyDataLength, -1, NULL);
}