#ifndef SCRAP_DETAIL_WINDOW
#define SCRAP_DETAIL_WINDOW

//Class for the window that displays scraps in detail for individual adjustments

#include "audioElement.h"
#include "acquireViewScrollBar.h"
#include "lineDisplay2D.h"
#include "timeGridUtil.h"
#include "playhead.h"
#include <iostream>

class ScrapDetailWindow : public AudioElement
{
public:

	//Constructor
	ScrapDetailWindow(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, SAMPLE** dataIn, int numChannelsIn, long dataLengthIn,  TimeGridUtil* globalTimeGridUtilIn, bool scrollOnBottomIn = true, int numChannelsDisplayedIn = 0);
	
	//Destructor
	~ScrapDetailWindow();
	
	void draw();
	bool isAudioElement(Point2D mouseLoc) {return false; };
	
	void clearData();
	
	void mouseDown(Point2D mouseLoc);
	void mouseUp(Point2D mouseLoc);
  void mouseMotion(Point2D mouseLoc);	
  
  int getScrapNumber() { return scrapNumber; };
  int setScrapNumber( int scrapNumberIn ) {scrapNumber = scrapNumberIn; };
	
	void updateData( SAMPLE** dataIn, int numChannelsIn, long dataLengthIn, int scrapNumberIn, AudioPlayer* audioPlayerIn, int numChannelsDisplayedIn = 0 );	
	void setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn);
	
protected:

	SAMPLE** data;
	SAMPLE* monoData;	//Stores a mono mix if data is stereo for summary display
	SAMPLE** summaryData;	//Stores a reduced "envelope" of the data for more efficient display
	long dataLength;
	long summaryLength;
	float numIndicesPerSummarySample;
	bool usingSummary;
	
	SAMPLE** dummyData;	//For when we clear the display
	long dummyDataLength;
	int dummyNumChannels;
	
	long playLoc;	
	
	int numChannels;
	int numChannelsDisplayed;	
	
	float scrollBarWidth;
	float waveformDisplayPadding;
	bool scrollOnBottom;
	
	float titleBarHeight;
	
	AcquireViewScrollBar* hScroll;
	LineDisplay2D** waveformDisplays;
	
	Playhead* playheadMoving;	
	
	bool lockedForDataUpdate;
	
	int scrapNumber;
	float dataIndexValueToXPos(long indexVal);	
	
};

#endif