#ifndef SCRAP_SUMMARY_DISPLAY
#define SCRAP_SUMMARY_DISPLAY

//class for displaying a scrap summary in the scraps display

#include "audioElement.h"
#include "../defs.h"
#include "lineDisplay2D.h"
#include "playhead.h"
#include <iostream>
#include <cstdio>

class ScrapSummaryDisplay : public AudioElement
{
public:

	//Constructor
	ScrapSummaryDisplay(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, SAMPLE** dataIn, long startSampleIn, long playLengthIn, int numChannelsIn, int summaryNumIn = -1,  int numChannelsDisplayedIn = 0);
	
	//Destructor
	~ScrapSummaryDisplay();
	
	void draw();
	void drawWithOffset(float pixelOffsetIn);	
	void setActive(bool activeIn);
	
	void mouseDown(Point2D mouseLoc);
	void mouseUp(Point2D mouseLoc);
  void mouseMotion(Point2D mouseLoc);
  void setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn);
  void setSummaryNum(int summaryNumIn) { summaryNum = summaryNumIn; };
  
  void setPlayheadX(float xIn) { playhead->setXMin(xIn); }
  
	float dataIndexValueToXPos( long indexVal );	
	
	void reverseSelection();
	
protected:

	//Actually stores the data locally - this is just a scrap (not saved)
	SAMPLE** data;
	SAMPLE** summaryData;
	
	int summaryNum;
	
	long dataLength;
	long summaryLength;
	float numIndicesPerSummarySample;
	bool usingSummary;
	
	int numChannels;
	int numChannelsDisplayed;
	
	LineDisplay2D** waveformDisplays;
	Playhead* playhead;
	
	float waveformDisplayPaddingXLeft;
	float waveformDisplayPaddingXRight;
	float waveformDisplayPaddingY;	
	

};

#endif