#include "scrapsWindow.h"

ScrapsWindow :: ScrapsWindow(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, float windowWidthIn, float windowHeightIn, ScrapDetailWindow* detailWindowIn, AcquireControlsWindow* acquireControlsWindowIn): AudioElement(xMinIn, xMaxIn, yMinIn, yMaxIn)
{
	entryHeight = 60.0;
	scrollBarWidth = 15.0;
	titleBarHeight = 23.0;
	
	windowWidth = windowWidthIn;
	windowHeight = windowHeightIn;
	
	summaryDisplayPaddingX = 3.0;
	summaryDisplayPaddingY = 2.0;

  numScrapsFitDisplay = ceil((yMax - yMin - titleBarHeight - 3)/entryHeight) + 1;

	vScroll = new BlockItemScrollBar(xMax-scrollBarWidth, xMax, yMin, yMax - 1 - titleBarHeight - 1, entryHeight, 0, 0);
	minDisplayedPixelOffset = 0.0;
  minScrapIndexDisplayed = floor(minDisplayedPixelOffset/entryHeight);	
	
	activeSummary = -1;
	activePlayer = NULL;
	
	scrapDetailWindow = detailWindowIn;
	acquireControlsWindow = acquireControlsWindowIn;
}


void ScrapsWindow :: addScrap(SAMPLE** dataIn, long startSampleIn, long playLengthIn, int numChannelsIn, bool isLoopingIn)
{
	float xMinSummaryWindow = xMin + summaryDisplayPaddingX;
	float xMaxSummaryWindow = xMax - scrollBarWidth - summaryDisplayPaddingX;
	float yMinSummaryWindow = yMax - titleBarHeight - ((scrapSummaryDisplays.size() + 1) * entryHeight - summaryDisplayPaddingY/2);
	float yMaxSummaryWindow = yMax - titleBarHeight - summaryDisplayPaddingY - (scrapSummaryDisplays.size() * entryHeight + summaryDisplayPaddingY/2);
	
	scrapSummaryDisplays.push_back( new ScrapSummaryDisplay(xMinSummaryWindow, xMaxSummaryWindow, yMinSummaryWindow, yMaxSummaryWindow, dataIn, startSampleIn, playLengthIn, numChannelsIn, scrapSummaryDisplays.size() + 1) );
	
	if( isLoopingIn )
	{
		AudioPlayer* tempPlayer = scrapSummaryDisplays[scrapSummaryDisplays.size()-1]->getAudioPlayer();
		tempPlayer->isLooping = true;	
	}
	
	vScroll->setNumItems(vScroll->getNumItems() + 1);
}


void ScrapsWindow :: draw()
{
  glColor3f( 1.0, 1.0, 1.0);
  glLineWidth( 1.0 );	  	

 	glBegin( GL_LINE_STRIP );
 		glVertex3f(xMin, yMin, 0.0);
 		glVertex3f(xMin, yMax, 0.0);
 		glVertex3f(xMax, yMax, 0.0);
 		glVertex3f(xMax, yMin, 0.0);
 		glVertex3f(xMin, yMin, 0.0);
	glEnd();
	
	
  // map the view port to the window location
  glViewport( xMin, yMin + summaryDisplayPaddingY + 1, xMax-xMin, yMax-yMin - 2.0*summaryDisplayPaddingY - titleBarHeight - 2 );    
  // set the matrix mode to project
  glMatrixMode( GL_PROJECTION );
  // load the identity matrix
  glLoadIdentity( );
  // working in ortho mode for this line plot
	gluOrtho2D(xMin, xMax, yMin + summaryDisplayPaddingY, yMax - summaryDisplayPaddingY - titleBarHeight - 1);
  // set the matrix mode to modelview
  glMatrixMode( GL_MODELVIEW );
  // load the identity matrix
  glLoadIdentity( );
  // position the view point
  gluLookAt( 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f );


//	std::cout << "MinIndexDisplayed: " << minScrapIndexDisplayed << std::endl;
//	std::cout << "PixelOffset: " << minDisplayedPixelOffset << std::endl;	


	for( int i=0; i < numScrapsFitDisplay && minScrapIndexDisplayed + i < scrapSummaryDisplays.size(); i++ )
		scrapSummaryDisplays[minScrapIndexDisplayed + i]->drawWithOffset(minDisplayedPixelOffset);


  // map the view port back to the whole window
  glViewport( 0, 0, windowWidth, windowHeight );
  // set the matrix mode to project
  glMatrixMode( GL_PROJECTION );
  // load the identity matrix
  glLoadIdentity( );
  // set to ortho for 2D version
	gluOrtho2D(0, windowWidth, 0, windowHeight);
  // set the matrix mode to modelview
  glMatrixMode( GL_MODELVIEW );
  // load the identity matrix
  glLoadIdentity( );
  // position the view point
  gluLookAt( 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f );


	//Title bar	
	
	glColor3f( 0.4, 0.4, 0.4 );
	glRectf( xMin + 1, yMax - 1 - titleBarHeight - 1, xMax - 1, yMax - 1);	

	glColor3f( 1.0, 1.0, 1.0 );			
	glBegin( GL_LINE_STRIP );
		glVertex3f( xMin, yMax - 1 - titleBarHeight - 1, 0.0);
		glVertex3f( xMax, yMax - 1 - titleBarHeight - 1, 0.0);		
	glEnd();

	glColor3f( 1.0, 1.0, 1.0 );							
	bitmapString( "Scraps", xMin + (xMax - xMin - 58 - scrollBarWidth)/2.0, yMax - titleBarHeight + 4, 99999, 2);
	
//	GLfloat* positionFin = new GLfloat[4];
//	glGetFloatv( GL_CURRENT_RASTER_POSITION, positionFin );	
//	std::cout << positionFin[0] - ( xMin + (xMax - xMin - 58 - scrollBarWidth)/2.0 ) << std::endl;
	
	vScroll->draw();
}


void ScrapsWindow :: setActive(bool activeIn)
{
	if(activeIn)
		active = true;
	else
	{
		active = false;
		if(activeSummary != -1)
		{
			scrapSummaryDisplays[activeSummary]->setActive(false);
		}
		activeSummary = -1;
		activePlayer = NULL;
	}
}

bool ScrapsWindow :: isAudioElement(Point2D mouseLoc)
{
	return true;
}
 
 
AudioPlayer* ScrapsWindow :: getAudioPlayer()
{
	if(activeSummary != -1)
		return scrapSummaryDisplays[activeSummary]->getAudioPlayer();
	else
		return NULL;	
}



void ScrapsWindow :: mouseDown(Point2D mouseLoc)
{
	//If the click was inside the scroller
	if( vScroll->isInside(mouseLoc) )
	{
		//Do the scrollBar's mouseDown. This sets the scrollBar active and sets the initial mouse position.
		vScroll->mouseDown(mouseLoc);
		
		//If we did not click inside the active rect, update the scrollbar position based on the click
		if( !vScroll->isInsideActiveRect(mouseLoc) )
			vScroll->clickOutsideActiveRect(mouseLoc);
			
		minDisplayedPixelOffset = vScroll->getMinDisplayedPixelOffset();	
	  minScrapIndexDisplayed = floor(minDisplayedPixelOffset/entryHeight);				
	}
	
	//Otherwise the click was inside the scraps menu area
	else
	{
		
		//if a summary was previously active, make it unactive
		if(activeSummary != -1)
			scrapSummaryDisplays[activeSummary]->setActive(false);
	
		activePlayer = NULL;
		active = false;
		
		activeSummary = -1;
		int i=0;
		
		//See if the click was on any of the summaries
		while( i < scrapSummaryDisplays.size() && activeSummary == -1 )
		{
			Point2D offsetMouseLoc = mouseLoc;
			offsetMouseLoc.y = offsetMouseLoc.y - minDisplayedPixelOffset;
			
			if( scrapSummaryDisplays[i]->isInside(offsetMouseLoc) )
			{
				scrapSummaryDisplays[i]->mouseDown(offsetMouseLoc);	
				activeSummary = i;
				activePlayer = scrapSummaryDisplays[i]->getAudioPlayer();
				active = true;
			}
			
			i++;
		}
		
	}
	
}


void ScrapsWindow :: mouseUp(Point2D mouseLoc)
{
	vScroll->mouseUp(mouseLoc);
//	active = false;
}


void ScrapsWindow :: mouseMotion(Point2D mouseLoc)
{
	if( vScroll->isActive() )
	{
		vScroll->mouseMotion(mouseLoc);
		minDisplayedPixelOffset = vScroll->getMinDisplayedPixelOffset();
	  minScrapIndexDisplayed = floor(minDisplayedPixelOffset/entryHeight);		
	}
	
}


void ScrapsWindow :: reverseSelection()
{
	//shouldn't be called if not active, but just to be careful...
	if( active && activeSummary != -1 )	
	{
		scrapSummaryDisplays[activeSummary]->reverseSelection();	
		
		if( activeSummary == scrapDetailWindow->getScrapNumber() )
		{
			AudioPlayer* tempPlayer = scrapSummaryDisplays[activeSummary]->getAudioPlayer();
			scrapDetailWindow->updateData( tempPlayer->bufferPointer, tempPlayer->numChannels, tempPlayer->bufferLength, activeSummary, tempPlayer);
		}
	}
}


void ScrapsWindow :: deleteSelection()
{
	//shouldn't be called if not active, but just to be careful...
	if( active && activeSummary != -1 )	
	{		
		scrapSummaryDisplays.erase(scrapSummaryDisplays.begin()+activeSummary);
		for( int i=0; i < scrapSummaryDisplays.size(); i++ )
		{
			float xMinSummaryWindow = xMin + summaryDisplayPaddingX;
			float xMaxSummaryWindow = xMax - scrollBarWidth - summaryDisplayPaddingX;
			float yMinSummaryWindow = yMax - titleBarHeight - ((i + 1) * entryHeight - summaryDisplayPaddingY/2);
			float yMaxSummaryWindow = yMax - titleBarHeight - summaryDisplayPaddingY - (i * entryHeight + summaryDisplayPaddingY/2);
			
			scrapSummaryDisplays[i]->setPosition(xMinSummaryWindow, xMaxSummaryWindow, yMinSummaryWindow, yMaxSummaryWindow);
			scrapSummaryDisplays[i]->setSummaryNum(i+1);
		}
		
		if( scrapDetailWindow->getScrapNumber() == activeSummary )
		{
			scrapDetailWindow->clearData();
			acquireControlsWindow->setDetailNum( -1 );
		}
		else if( scrapDetailWindow->getScrapNumber() > activeSummary )
		{
			scrapDetailWindow->setScrapNumber( scrapDetailWindow->getScrapNumber() - 1 );
			acquireControlsWindow->setDetailNum( acquireControlsWindow->getDetailNum() - 1);
		}

		vScroll->setNumItems( vScroll->getNumItems() - 1 );
		minDisplayedPixelOffset = vScroll->getMinDisplayedPixelOffset();
	  minScrapIndexDisplayed = floor(minDisplayedPixelOffset/entryHeight);				
		
		active = false;
		activeSummary = -1;
	}
}


void ScrapsWindow :: setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, float windowWidthIn, float windowHeightIn)
{
	xMin = xMinIn;
	xMax = xMaxIn;
	yMin = yMinIn;
	yMax = yMaxIn;
	
	vScroll->setPosition(xMaxIn-scrollBarWidth, xMaxIn, yMinIn, yMaxIn - 1 - titleBarHeight - 1);
	
	for( int i=0; i < scrapSummaryDisplays.size(); i++ )
	{
		float xMinSummaryWindow = xMin + summaryDisplayPaddingX;
		float xMaxSummaryWindow = xMax - scrollBarWidth - summaryDisplayPaddingX;
		float yMinSummaryWindow = yMax - titleBarHeight - ((i + 1) * entryHeight - summaryDisplayPaddingY/2);
		float yMaxSummaryWindow = yMax - titleBarHeight - summaryDisplayPaddingY - (i * entryHeight + summaryDisplayPaddingY/2);
		
		scrapSummaryDisplays[i]->setPosition(xMinSummaryWindow, xMaxSummaryWindow, yMinSummaryWindow, yMaxSummaryWindow);
	}
	
	windowWidth = windowWidthIn;
	windowHeight = windowHeightIn;
  numScrapsFitDisplay = ceil((yMax - yMin - titleBarHeight - 3)/entryHeight) + 1;	
	if( activeSummary != -1 && (activeSummary <= minScrapIndexDisplayed || activeSummary >= minScrapIndexDisplayed + numScrapsFitDisplay))
	{
		if( activeSummary == scrapSummaryDisplays.size() - 1 )
			scrollToBottom();
		else
			scrollToIncludeActive();
	}
	else if( minScrapIndexDisplayed + numScrapsFitDisplay >= scrapSummaryDisplays.size() )
		scrollToBottom();
	else if( scrapSummaryDisplays.size() < numScrapsFitDisplay )
	{
		minDisplayedPixelOffset = 0.0;
		vScroll->setMinDisplayedPixelOffset(minDisplayedPixelOffset);
	}
}


void ScrapsWindow :: changeSelectedScrap(bool up)
{
	if( up && activeSummary > 0 )
	{
		if( scrapSummaryDisplays[activeSummary]->isPlaying() )
		{
			activePlayer->isPlaying = false;
			scrapSummaryDisplays[activeSummary]->setActive(false);
			activeSummary = activeSummary - 1;
			activePlayer = scrapSummaryDisplays[activeSummary]->getAudioPlayer();
			scrapSummaryDisplays[activeSummary]->setActive(true);
			activePlayer->isPlaying = true;
		}
		else
		{
			scrapSummaryDisplays[activeSummary]->setActive(false);
			activeSummary = activeSummary - 1;
			activePlayer = scrapSummaryDisplays[activeSummary]->getAudioPlayer();			
			scrapSummaryDisplays[activeSummary]->setActive(true);
		}
		
		//If that scrap isn't fully displayed, make it so that it is
		if( activeSummary == 0 )
		{
			minScrapIndexDisplayed = 0;
			minDisplayedPixelOffset = 0.0;
		}
		else if( activeSummary <= minScrapIndexDisplayed )	//hanging off top
		{
			minScrapIndexDisplayed = activeSummary - 1;		
			minDisplayedPixelOffset = (minScrapIndexDisplayed + 1) * entryHeight - 0.3*entryHeight;
		}
		//If that scrap is below the currently viewed scraps, jump down so that it shows at the bottom		
		else if( activeSummary >= minScrapIndexDisplayed + numScrapsFitDisplay - 1 )
		{
			minScrapIndexDisplayed = activeSummary + 1 - (numScrapsFitDisplay - 1);
			minDisplayedPixelOffset = (minScrapIndexDisplayed) * entryHeight + 0.4 * entryHeight + summaryDisplayPaddingY + titleBarHeight + 2;
		}


		//Don't let the scroll leave black space if we have more entries than we can display
		if( scrapSummaryDisplays.size() * entryHeight - ((yMax-2-titleBarHeight-2)-(yMin+1)) > 0 && minDisplayedPixelOffset > scrapSummaryDisplays.size() * entryHeight - ((yMax-2-titleBarHeight-2)-(yMin+1)) )
			minDisplayedPixelOffset = scrapSummaryDisplays.size() * entryHeight - ((yMax-2-titleBarHeight-2)-(yMin+1));
		
		
		//update scrollbar to match minDisplayedPixelOffset
		vScroll->setMinDisplayedPixelOffset(minDisplayedPixelOffset);
		
	}
	
	
	else if( !up && activeSummary < scrapSummaryDisplays.size() - 1 )
	{
		if( scrapSummaryDisplays[activeSummary]->isPlaying() )
		{
			activePlayer->isPlaying = false;
			scrapSummaryDisplays[activeSummary]->setActive(false);
			activeSummary = activeSummary + 1;
			activePlayer = scrapSummaryDisplays[activeSummary]->getAudioPlayer();
			scrapSummaryDisplays[activeSummary]->setActive(true);
			activePlayer->isPlaying = true;
		}
		else
		{		
			scrapSummaryDisplays[activeSummary]->setActive(false);
			activeSummary = activeSummary + 1;
			activePlayer = scrapSummaryDisplays[activeSummary]->getAudioPlayer();			
			scrapSummaryDisplays[activeSummary]->setActive(true);				
		}
		
		//If that scrap isn't fully displayed, make it so that it is
		if( activeSummary == scrapSummaryDisplays.size() - 1 && scrapSummaryDisplays.size() * entryHeight > (yMax-2-titleBarHeight-2)-(yMin+1))
		{
			minScrapIndexDisplayed = activeSummary + 1 - (numScrapsFitDisplay - 1);				
			minDisplayedPixelOffset = scrapSummaryDisplays.size() * entryHeight - ((yMax-2-titleBarHeight-2)-(yMin+1)) - 3;
		}
		else if( activeSummary >= minScrapIndexDisplayed + numScrapsFitDisplay - 2 && scrapSummaryDisplays.size() * entryHeight > (yMax-2-titleBarHeight-2)-(yMin+1) )	//hanging off bottom
		{
			minScrapIndexDisplayed = activeSummary + 1 - (numScrapsFitDisplay - 1);	
			minDisplayedPixelOffset = (minScrapIndexDisplayed) * entryHeight + 0.4 * entryHeight + summaryDisplayPaddingY + titleBarHeight + 2;
		}
		else if( activeSummary <= minScrapIndexDisplayed )
		{
			minScrapIndexDisplayed = activeSummary - 1;
			minDisplayedPixelOffset = (minScrapIndexDisplayed + 1) * entryHeight - 0.3*entryHeight;				
		}
		
		
		//update scrollbar to match minDisplayedPixelOffset		
		vScroll->setMinDisplayedPixelOffset(minDisplayedPixelOffset);		
		
	}
}



void ScrapsWindow :: scrollToBottom()
{
	if( scrapSummaryDisplays.size() * entryHeight > (yMax-2-titleBarHeight-2)-(yMin+1) )
	{
		minScrapIndexDisplayed = scrapSummaryDisplays.size() - (numScrapsFitDisplay - 1);
			
		minDisplayedPixelOffset = scrapSummaryDisplays.size() * entryHeight - ((yMax-2-titleBarHeight-2)-(yMin+1)) - 3;
		vScroll->setMinDisplayedPixelOffset(minDisplayedPixelOffset);		
	}
}



void ScrapsWindow :: scrollToIncludeActive()
{
	
	//If it is the first scrap
	if( activeSummary == 0 )
	{
		minScrapIndexDisplayed = 0;
		minDisplayedPixelOffset = 0.0;
	}
	//If it is the last scrap and the last scrap extends beyond the window
	else if( activeSummary == scrapSummaryDisplays.size() - 1 && scrapSummaryDisplays.size() * entryHeight > (yMax-2-titleBarHeight-2)-(yMin+1))
	{
		minScrapIndexDisplayed = activeSummary + 1 - (numScrapsFitDisplay - 1);				
		minDisplayedPixelOffset = scrapSummaryDisplays.size() * entryHeight - ((yMax-2-titleBarHeight-2)-(yMin+1)) - 3;
	}
	//If that scrap is above the currently viewed scraps, jump up so that it shows at the top
	else if( activeSummary <= minScrapIndexDisplayed )
	{
		minScrapIndexDisplayed = activeSummary - 1;		
		minDisplayedPixelOffset = (minScrapIndexDisplayed + 1) * entryHeight - 0.3*entryHeight;
	}
	//If that scrap is below the currently viewed scraps, jump down so that it shows at the bottom		
	else if( activeSummary >= minScrapIndexDisplayed + numScrapsFitDisplay - 1 )
	{
		minScrapIndexDisplayed = activeSummary + 1 - (numScrapsFitDisplay - 1);
		minDisplayedPixelOffset = (minScrapIndexDisplayed) * entryHeight + 0.4 * entryHeight + summaryDisplayPaddingY + titleBarHeight + 2;
	}
	
	//update scrollbar to match minDisplayedPixelOffset		
	vScroll->setMinDisplayedPixelOffset(minDisplayedPixelOffset);		
}


/*
SAMPLE** ScrapsWindow :: getActiveScrapData()
{
	if( activeSummary != -1 )
		return scrapSummaryDisplays[activeSummary]->getDataPointer();
	else
		return NULL;
}
*/

void ScrapsWindow :: setParticularSummaryActive( int summaryNumIn )
{
	if( summaryNumIn < scrapSummaryDisplays.size() && summaryNumIn >= 0)
	{
		
		if( activeSummary	!= -1 )
		{
			if( scrapSummaryDisplays[activeSummary]->isPlaying() )
				activePlayer->isPlaying = false;

			scrapSummaryDisplays[activeSummary]->setActive(false);		
		}
		
		activeSummary = summaryNumIn;
		activePlayer = scrapSummaryDisplays[activeSummary]->getAudioPlayer();
		scrapSummaryDisplays[activeSummary]->setActive(true);
		activePlayer->isPlaying = true;		
		scrollToIncludeActive();
		active = true;
	}
		
}


void ScrapsWindow :: deleteAllScraps()
{
	scrapSummaryDisplays.clear();	
	
	vScroll->setNumItems( 0 );
	minDisplayedPixelOffset = 0;
  minScrapIndexDisplayed = 0;				
		
	active = false;
	activeSummary = -1;
	activePlayer = NULL;
	
	scrapDetailWindow->clearData();
}


void ScrapsWindow :: toggleAllScrapsLooping()
{
	AudioPlayer* tempPlayer;
	bool allLooping = true;
	
	//Set all scrap players to be looping	
	for( int i=0; i < scrapSummaryDisplays.size(); i++)
	{
		tempPlayer = scrapSummaryDisplays[i]->getAudioPlayer();
		if( !tempPlayer->isLooping )
		{
			tempPlayer->isLooping = true;
			allLooping = false;
		}
	}	
	
	//If all of the scrap players were previously looping, mark them all to be not looping
	if( allLooping )
	{
		for( int i=0; i < scrapSummaryDisplays.size(); i++)
		{
			tempPlayer = scrapSummaryDisplays[i]->getAudioPlayer();
			tempPlayer->isLooping = false;
		}		
	}
}


void ScrapsWindow :: openActiveScrapInDetail()
{
	if( activeSummary != -1)
	{
		AudioPlayer* tempPlayer = scrapSummaryDisplays[activeSummary]->getAudioPlayer();

		scrapDetailWindow->updateData( tempPlayer->bufferPointer, tempPlayer->numChannels, tempPlayer->bufferLength, activeSummary, tempPlayer);
		acquireControlsWindow->setDetailNum( activeSummary );
	}
}