#ifndef SCRAPS_WINDOW
#define SCRAPS_WINDOW

//Window for browsing several short audio clips

#include "audioElement.h"
#include "blockItemScrollBar.h"
#include "scrapSummaryDisplay.h"
#include "scrapDetailWindow.h"
#include "acquireControlsWindow.h"
#include "math.h"
#include <vector>

class ScrapsWindow : public AudioElement
{
public:

	//Constructor
	ScrapsWindow(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, float windowWidthIn, float windowHeightIn, ScrapDetailWindow* detailWindowIn, AcquireControlsWindow* acquireControlsWindowIn);
	
	//Destructor
	~ScrapsWindow() { delete vScroll; };
	
	void draw();
	
	void setActive(bool activeIn);

	bool isAudioElement(Point2D mouseLoc);
  AudioPlayer* getAudioPlayer();

	void mouseDown(Point2D mouseLoc);
	void mouseUp(Point2D mouseLoc);
  void mouseMotion(Point2D mouseLoc);  
	void setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, float windowWidthIn, float windowHeightIn);  
	
	void addScrap(SAMPLE** dataIn, long startSampleIn, long playLengthIn, int numChannelsIn, bool isLoopingIn = false);
	
	void scrollToBottom();
	void scrollToIncludeActive();
	void changeSelectedScrap(bool up);
	void reverseSelection();
	void deleteSelection();
	void deleteAllScraps();
	void toggleAllScrapsLooping();
	
	void openActiveScrapInDetail();
	
	int getNumberOfScraps() { return scrapSummaryDisplays.size(); };
	AudioPlayer* getParticularScrapAudioPlayer( int scrapNumber ) { return scrapSummaryDisplays[scrapNumber]->getAudioPlayer(); };
	void setParticularSummaryActive( int summaryNumIn );

protected:

	std::vector<ScrapSummaryDisplay*> scrapSummaryDisplays;
	int activeSummary;
	AudioPlayer* activePlayer;
	
	float entryHeight;
	
	float windowWidth;
	float windowHeight;
	
	float summaryDisplayPaddingX;
	float summaryDisplayPaddingY;
	float titleBarHeight;

  float minDisplayedPixelOffset;
  long numScrapsFitDisplay;
  long minScrapIndexDisplayed;

	BlockItemScrollBar* vScroll;
	float scrollBarWidth;		

	ScrapDetailWindow* scrapDetailWindow;
	AcquireControlsWindow* acquireControlsWindow;

};

#endif