#ifndef UI_ELEMENT
#define UI_ELEMENT

//Pure virtual base class for UI Elements

#include "glLibs.h"
#include "displayUtils.h"
#include "../defs.h"


class UiElement
{
public:

  // Constructor
  UiElement(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, bool activeIn = false): xMin(xMinIn), xMax(xMaxIn), yMin(yMinIn), yMax(yMaxIn), active(activeIn) {}
  
  // Destructor
  ~UiElement() {};

  // Draw self
  virtual void draw() = 0;
  
  // Mouse
  virtual void mouseDown(Point2D point) { return; };
  virtual void mouseUp(Point2D point) { return; };
  virtual void mouseMotion(Point2D point) { return; };
	virtual void setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn) {xMin = xMinIn; xMax = xMaxIn, yMin = yMinIn, yMax = yMaxIn; };

  // Return true if the given point is inside this element 
  virtual inline bool isInside(Point2D point) { return ((point.x <= xMax) && (point.x >= xMin) && (point.y <= yMax) && (point.y >= yMin)); }

  virtual inline bool isAudioElement(Point2D point) { return false; };

  virtual void setActive(bool activeIn) { active = activeIn; };
  bool isActive() { return active; };

protected:
  float xMin;
  float xMax;
  float yMin;
  float yMax;
  bool active;
};

#endif