#include "waveformDisplayWithScroll.h"

WaveformDisplayWithScroll :: WaveformDisplayWithScroll(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, SAMPLE** dataIn, int numChannelsIn, long dataLengthIn, bool scrollOnBottom, int numChannelsDisplayedIn): UiElement(xMinIn, xMaxIn, yMinIn, yMaxIn)
{
	numChannels = numChannelsIn;
	data =	new SAMPLE*[numChannels];
	
	for( int i=0; i < numChannels; i++)
		data[i] = dataIn[i];
	
	dataLength = dataLengthIn;

/* Not currently implemented		
	if( numChannels > 1 )
	{
		monoData = new SAMPLE[dataLength];	
		for( int i=0; i < dataLength; i++ )
			monoData[i] = data[0][i] + data[1][i];	//Do a max abs here to get envelope? Normalize?
	}
*/
	
	if( numChannelsDisplayedIn == 0 )
		numChannelsDisplayed = numChannelsIn;
	else
		numChannelsDisplayed = numChannelsDisplayedIn;
		
	scrollBarWidth = 15.0;
	waveformDisplayPadding = 5.0;
	
	//Default to display the entire waveform, start to finish, -1.0 to 1.0 amplitude, with a y = 0 line
	//Display size is smaller to leave space for scroll bars
	float xMinDisp = xMin + waveformDisplayPadding;
	float xMaxDisp = xMax - waveformDisplayPadding;
	float yMinDisp;
	float yMaxDisp;
	if( scrollOnBottom )
	{
		yMinDisp = yMin + scrollBarWidth + waveformDisplayPadding;
		yMaxDisp = yMax - waveformDisplayPadding;
		hScroll = new ScrollBar(xMin, xMax, yMin, yMin+scrollBarWidth, true, dataLength, 0, dataLength-1);
	}
	else
	{
		yMinDisp = yMin + waveformDisplayPadding;
		yMaxDisp = yMax - scrollBarWidth - waveformDisplayPadding;		
		hScroll = new ScrollBar(xMin, xMax, yMax-scrollBarWidth, yMax, true, dataLength, 0, dataLength-1);
	}
	
	waveformDisplays = new LineDisplay2D*[numChannels];
	for( int i=0; i < numChannels; i++)
		waveformDisplays[i] = new LineDisplay2D( xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)), data[i], dataLength, 0, dataLength-1, 1.0, true );
		
}


WaveformDisplayWithScroll :: ~WaveformDisplayWithScroll()
{
	delete []data;
	
	for( int i=0; i < numChannels; i++ )
		delete waveformDisplays[i];
	
	delete []waveformDisplays;
	delete hScroll;
}


void WaveformDisplayWithScroll :: draw()
{
  glColor3f( 1.0, 1.0, 1.0);
  glLineWidth( 1.0 );	  	

 	glBegin( GL_LINE_STRIP );
 		glVertex3f(xMin, yMin, 0.0);
 		glVertex3f(xMin, yMax, 0.0);
 		glVertex3f(xMax, yMax, 0.0);
 		glVertex3f(xMax, yMin, 0.0);
 		glVertex3f(xMin, yMin, 0.0);
	glEnd();
	
	for( int i = 0; i < numChannelsDisplayed; i++ )
		waveformDisplays[i]->draw();
	
	hScroll->draw();
}


void WaveformDisplayWithScroll :: mouseDown(Point2D mouseLoc)
{
	if( hScroll->isInside(mouseLoc) )
	{
		bool originallyInsideActiveRect = true;
		if( !hScroll->isInsideActiveRect(mouseLoc) )
			originallyInsideActiveRect = false;

		hScroll->mouseDown(mouseLoc);
		
		//If not inside the active rect, do immediate update (scroll)
		if( !originallyInsideActiveRect )
		{
				for( int channel = 0; channel < numChannels; channel++ )
					waveformDisplays[channel]->changeDisplayedIndexRange( hScroll->getMinIndexActive(), hScroll->getMaxIndexActive() );
		}
	}
}


void WaveformDisplayWithScroll :: mouseUp(Point2D mouseLoc)
{
	hScroll->mouseUp(mouseLoc);
}


void WaveformDisplayWithScroll :: mouseMotion(Point2D mouseLoc)
{
	if( hScroll->isActive() )
	{
		hScroll->mouseMotion(mouseLoc);
		
//		std::cout <<  hScroll->getMaxIndexActive() - hScroll->getMinIndexActive() << std::endl;
		
		for( int channel = 0; channel < numChannelsDisplayed; channel++ )
			waveformDisplays[channel]->changeDisplayedIndexRange( hScroll->getMinIndexActive(), hScroll->getMaxIndexActive() );
	}
}
